/********************************************************************************
** Form generated from reading UI file 'OpenCSGWarningDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPENCSGWARNINGDIALOG_H
#define UI_OPENCSGWARNINGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_OpenCSGWarningDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *warningText;
    QCheckBox *enableOpenCSGBox;
    QHBoxLayout *horizontalLayout;
    QCheckBox *showBox;
    QSpacerItem *horizontalSpacer;
    QPushButton *closeButton;

    void setupUi(QDialog *OpenCSGWarningDialog)
    {
        if (OpenCSGWarningDialog->objectName().isEmpty())
            OpenCSGWarningDialog->setObjectName(QString::fromUtf8("OpenCSGWarningDialog"));
        OpenCSGWarningDialog->resize(412, 275);
        verticalLayout = new QVBoxLayout(OpenCSGWarningDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        warningText = new QTextEdit(OpenCSGWarningDialog);
        warningText->setObjectName(QString::fromUtf8("warningText"));
        warningText->setReadOnly(true);

        verticalLayout->addWidget(warningText);

        enableOpenCSGBox = new QCheckBox(OpenCSGWarningDialog);
        enableOpenCSGBox->setObjectName(QString::fromUtf8("enableOpenCSGBox"));
        enableOpenCSGBox->setChecked(true);

        verticalLayout->addWidget(enableOpenCSGBox);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        showBox = new QCheckBox(OpenCSGWarningDialog);
        showBox->setObjectName(QString::fromUtf8("showBox"));
        showBox->setChecked(true);

        horizontalLayout->addWidget(showBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        closeButton = new QPushButton(OpenCSGWarningDialog);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));

        horizontalLayout->addWidget(closeButton);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(OpenCSGWarningDialog);
        QObject::connect(closeButton, SIGNAL(clicked()), OpenCSGWarningDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(OpenCSGWarningDialog);
    } // setupUi

    void retranslateUi(QDialog *OpenCSGWarningDialog)
    {
        OpenCSGWarningDialog->setWindowTitle(q_("OpenGL Warning", nullptr));
        warningText->setHtml(q_("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Lucida Grande'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p></body></html>", nullptr));
        enableOpenCSGBox->setText(q_("Enable OpenCSG", nullptr));
        showBox->setText(q_("Show this message again", nullptr));
        closeButton->setText(q_("Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class OpenCSGWarningDialog: public Ui_OpenCSGWarningDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OPENCSGWARNINGDIALOG_H
