/********************************************************************************
** Form generated from reading UI file 'launchingscreen.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAUNCHINGSCREEN_H
#define UI_LAUNCHINGSCREEN_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LaunchingScreen
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButtonNew;
    QPushButton *pushButtonOpen;
    QPushButton *pushButtonHelp;
    QLabel *label_2;
    QListWidget *recentList;
    QPushButton *openRecentButton;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_5;
    QTreeWidget *treeWidget;
    QPushButton *openExampleButton;
    QGridLayout *gridLayout_2;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *checkBox;
    QSpacerItem *horizontalSpacer;
    QLabel *versionNumberLabel;

    void setupUi(QDialog *LaunchingScreen)
    {
        if (LaunchingScreen->objectName().isEmpty())
            LaunchingScreen->setObjectName(QString::fromUtf8("LaunchingScreen"));
        LaunchingScreen->resize(620, 418);
        LaunchingScreen->setMinimumSize(QSize(600, 0));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/prefs3DView.png"), QSize(), QIcon::Normal, QIcon::Off);
        LaunchingScreen->setWindowIcon(icon);
        LaunchingScreen->setSizeGripEnabled(true);
        LaunchingScreen->setModal(false);
        gridLayout = new QGridLayout(LaunchingScreen);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(10, 20, 10, 20);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        pushButtonNew = new QPushButton(LaunchingScreen);
        pushButtonNew->setObjectName(QString::fromUtf8("pushButtonNew"));
        pushButtonNew->setStyleSheet(QString::fromUtf8("QPushButton {\n"
"    margin: 0;\n"
"    padding: 4px;\n"
"    color: white;\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #82dd97, stop: 1 #2a833f);\n"
"    border-radius: 2px;\n"
"}\n"
"QPushButton:hover {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #92eda7, stop: 1 #3a934f);\n"
"}\n"
"QPushButton:disabled {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #dfdfdf, stop: 1 #aeaeae);\n"
"}\n"
"QPushButton:pressed {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #2a833f, stop: 1 #82dd97);\n"
"}"));
        pushButtonNew->setFlat(true);

        horizontalLayout->addWidget(pushButtonNew);

        pushButtonOpen = new QPushButton(LaunchingScreen);
        pushButtonOpen->setObjectName(QString::fromUtf8("pushButtonOpen"));
        pushButtonOpen->setStyleSheet(QString::fromUtf8("QPushButton {\n"
"    margin: 0;\n"
"    padding: 4px;\n"
"    color: white;\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #82dd97, stop: 1 #2a833f);\n"
"    border-radius: 2px;\n"
"}\n"
"QPushButton:hover {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #92eda7, stop: 1 #3a934f);\n"
"}\n"
"QPushButton:disabled {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #dfdfdf, stop: 1 #aeaeae);\n"
"}\n"
"QPushButton:pressed {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #2a833f, stop: 1 #82dd97);\n"
"}"));
        pushButtonOpen->setFlat(true);

        horizontalLayout->addWidget(pushButtonOpen);

        pushButtonHelp = new QPushButton(LaunchingScreen);
        pushButtonHelp->setObjectName(QString::fromUtf8("pushButtonHelp"));
        pushButtonHelp->setStyleSheet(QString::fromUtf8("QPushButton {\n"
"    margin: 0;\n"
"    padding: 4px;\n"
"    color: white;\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #82dd97, stop: 1 #2a833f);\n"
"    border-radius: 2px;\n"
"}\n"
"QPushButton:hover {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #92eda7, stop: 1 #3a934f);\n"
"}\n"
"QPushButton:disabled {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #dfdfdf, stop: 1 #aeaeae);\n"
"}\n"
"QPushButton:pressed {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #2a833f, stop: 1 #82dd97);\n"
"}"));
        pushButtonHelp->setFlat(true);

        horizontalLayout->addWidget(pushButtonHelp);


        verticalLayout->addLayout(horizontalLayout);

        label_2 = new QLabel(LaunchingScreen);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        verticalLayout->addWidget(label_2);

        recentList = new QListWidget(LaunchingScreen);
        recentList->setObjectName(QString::fromUtf8("recentList"));
        recentList->setStyleSheet(QString::fromUtf8("QListWidget { background-color:transparent; }"));

        verticalLayout->addWidget(recentList);


        verticalLayout_2->addLayout(verticalLayout);

        openRecentButton = new QPushButton(LaunchingScreen);
        openRecentButton->setObjectName(QString::fromUtf8("openRecentButton"));
        openRecentButton->setEnabled(false);
        openRecentButton->setStyleSheet(QString::fromUtf8("QPushButton {\n"
"    margin: 0;\n"
"    padding: 4px;\n"
"    color: white;\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #82dd97, stop: 1 #2a833f);\n"
"    border-radius: 2px;\n"
"}\n"
"QPushButton:hover {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #92eda7, stop: 1 #3a934f);\n"
"}\n"
"QPushButton:disabled {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #dfdfdf, stop: 1 #aeaeae);\n"
"}\n"
"QPushButton:pressed {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #2a833f, stop: 1 #82dd97);\n"
"}"));
        openRecentButton->setFlat(true);

        verticalLayout_2->addWidget(openRecentButton);


        gridLayout->addLayout(verticalLayout_2, 6, 0, 1, 1);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(10, 20, 10, 20);
        label_5 = new QLabel(LaunchingScreen);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        verticalLayout_3->addWidget(label_5);

        treeWidget = new QTreeWidget(LaunchingScreen);
        treeWidget->setObjectName(QString::fromUtf8("treeWidget"));
        treeWidget->setStyleSheet(QString::fromUtf8("QTreeWidget { background-color:transparent; }"));
        treeWidget->header()->setVisible(false);

        verticalLayout_3->addWidget(treeWidget);

        openExampleButton = new QPushButton(LaunchingScreen);
        openExampleButton->setObjectName(QString::fromUtf8("openExampleButton"));
        openExampleButton->setEnabled(false);
        openExampleButton->setStyleSheet(QString::fromUtf8("QPushButton {\n"
"    margin: 0;\n"
"    padding: 4px;\n"
"    color: white;\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #82dd97, stop: 1 #2a833f);\n"
"    border-radius: 2px;\n"
"}\n"
"QPushButton:hover {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #92eda7, stop: 1 #3a934f);\n"
"}\n"
"QPushButton:disabled {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #dfdfdf, stop: 1 #aeaeae);\n"
"}\n"
"QPushButton:pressed {\n"
"    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #2a833f, stop: 1 #82dd97);\n"
"}"));
        openExampleButton->setFlat(true);

        verticalLayout_3->addWidget(openExampleButton);


        gridLayout->addLayout(verticalLayout_3, 6, 1, 1, 1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(10, -1, 10, -1);
        label = new QLabel(LaunchingScreen);
        label->setObjectName(QString::fromUtf8("label"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setMinimumSize(QSize(96, 96));
        label->setMaximumSize(QSize(96, 96));
        label->setPixmap(QPixmap(QString::fromUtf8(":/icons/openscad.png")));
        label->setScaledContents(true);
        label->setMargin(0);

        gridLayout_2->addWidget(label, 0, 0, 2, 1);

        horizontalSpacer_2 = new QSpacerItem(20, 2, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 0, 1, 1, 1);

        label_3 = new QLabel(LaunchingScreen);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(label_3, 0, 2, 2, 1);


        gridLayout->addLayout(gridLayout_2, 3, 0, 2, 2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        checkBox = new QCheckBox(LaunchingScreen);
        checkBox->setObjectName(QString::fromUtf8("checkBox"));
        checkBox->setEnabled(true);
        checkBox->setAutoFillBackground(false);
        checkBox->setChecked(false);

        horizontalLayout_2->addWidget(checkBox);

        horizontalSpacer = new QSpacerItem(2, 2, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        versionNumberLabel = new QLabel(LaunchingScreen);
        versionNumberLabel->setObjectName(QString::fromUtf8("versionNumberLabel"));
        versionNumberLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(versionNumberLabel);


        gridLayout->addLayout(horizontalLayout_2, 8, 0, 1, 2);


        retranslateUi(LaunchingScreen);

        pushButtonNew->setDefault(true);


        QMetaObject::connectSlotsByName(LaunchingScreen);
    } // setupUi

    void retranslateUi(QDialog *LaunchingScreen)
    {
        LaunchingScreen->setWindowTitle(q_("Welcome to OpenSCAD", nullptr));
        pushButtonNew->setText(q_("New", nullptr));
        pushButtonOpen->setText(q_("Open", nullptr));
        pushButtonHelp->setText(q_("Help", nullptr));
        label_2->setText(q_("Recents", nullptr));
        openRecentButton->setText(q_("Open Recent", nullptr));
        label_5->setText(q_("Examples", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = treeWidget->headerItem();
        ___qtreewidgetitem->setText(0, q_("Examples", nullptr));
        openExampleButton->setText(q_("Open Example", nullptr));
        label->setText(QString());
        label_3->setText(q_("<html><head/><body>\n"
"<p style=\"font-family: 'Open Sans', 'Droid Sans', 'sans-serif'; font-weight: bold; padding-bottom: 0; margin-bottom: 0; font-size: 22pt;\"><span style=\"color: green;\">Open</span>SCAD</p>\n"
"<p style=\"font-family: 'Open Sans', 'Droid Sans', 'sans-serif'; font-weight: normal; font-size: 14pt; padding-top: 0; margin-top: 0; margin-left: 2em;\">The Programmers Solid 3D CAD Modeller</p>\n"
"</body></html>\n"
"\n"
"\n"
"", nullptr));
        checkBox->setText(q_("Don't show again", nullptr));
        versionNumberLabel->setText(q_("Version", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LaunchingScreen: public Ui_LaunchingScreen {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LAUNCHINGSCREEN_H
