/*
 *  OpenSCAD (www.openscad.org)
 *  Copyright (C) 2009-2011 Clifford Wolf <clifford@clifford.at> and
 *                          Marius Kintel <marius@kintel.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  As a special exception, you have permission to link this program
 *  with the CGAL library and distribute executables, as long as you
 *  follow the requirements of the GNU GPL in regard to all of the
 *  software in the executable aside from CGAL.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <QDrag>
#include <QPixmap>
#include <QPainter>
#include <QMimeData>

#include "qtgettext.h"
#include "FontListDialog.h"

FontListTableView::FontListTableView(QWidget *parent) : QTableView(parent)
{
}

void FontListTableView::setDragText(const QString &text)
{
	this->text = text.trimmed();
}

void FontListTableView::startDrag(Qt::DropActions supportedActions)
{
	if (text.isEmpty()) return;

	auto mimeData = new QMimeData;
	mimeData->setText(text);

	QFontMetrics fm(font());
	QRect rect(0, 0, fm.width(text) + 8, fm.height() + 8);
	QPixmap pixmap(rect.width(), rect.height());
	pixmap.fill({240, 240, 240, 160});

	QPainter painter(&pixmap);
	painter.setFont(font());
	painter.drawText(rect, Qt::AlignCenter, text);
	painter.drawRect(0, 0, rect.width() - 1, rect.height() - 1);

	auto drag = new QDrag(this);
	drag->setPixmap(pixmap);
	drag->setMimeData(mimeData);
	drag->setHotSpot({-10, rect.height() + 6});
	drag->exec(supportedActions, Qt::CopyAction);
}
