/********************************************************************************
** Form generated from reading UI file 'Console.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONSOLE_H
#define UI_CONSOLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QPlainTextEdit>

QT_BEGIN_NAMESPACE

class Ui_Console
{
public:
    QAction *actionClear;
    QAction *actionSaveAs;

    void setupUi(QPlainTextEdit *Console)
    {
        if (Console->objectName().isEmpty())
            Console->setObjectName(QString::fromUtf8("Console"));
        Console->resize(400, 300);
        actionClear = new QAction(Console);
        actionClear->setObjectName(QString::fromUtf8("actionClear"));
        actionSaveAs = new QAction(Console);
        actionSaveAs->setObjectName(QString::fromUtf8("actionSaveAs"));

        retranslateUi(Console);

        QMetaObject::connectSlotsByName(Console);
    } // setupUi

    void retranslateUi(QPlainTextEdit *Console)
    {
        actionClear->setText(q_("Clear", nullptr));
#if QT_CONFIG(tooltip)
        actionClear->setToolTip(q_("Clear console", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveAs->setText(q_("Save As...", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAs->setToolTip(q_("Save console content to file", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)Console;
    } // retranslateUi

};

namespace Ui {
    class Console: public Ui_Console {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONSOLE_H
