/********************************************************************************
** Form generated from reading UI file 'FontListDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONTLISTDIALOG_H
#define UI_FONTLISTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include "FontListTableView.h"

QT_BEGIN_NAMESPACE

class Ui_FontListDialog
{
public:
    QGridLayout *gridLayout;
    QPushButton *okButton;
    QPushButton *copyButton;
    QSpacerItem *horizontalSpacer;
    QLineEdit *filterLineEdit;
    QLabel *label_2;
    FontListTableView *tableView;
    QLabel *label;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *FontListDialog)
    {
        if (FontListDialog->objectName().isEmpty())
            FontListDialog->setObjectName(QString::fromUtf8("FontListDialog"));
        FontListDialog->resize(762, 449);
        gridLayout = new QGridLayout(FontListDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setHorizontalSpacing(16);
        gridLayout->setVerticalSpacing(6);
        okButton = new QPushButton(FontListDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(okButton->sizePolicy().hasHeightForWidth());
        okButton->setSizePolicy(sizePolicy);

        gridLayout->addWidget(okButton, 4, 4, 1, 1);

        copyButton = new QPushButton(FontListDialog);
        copyButton->setObjectName(QString::fromUtf8("copyButton"));

        gridLayout->addWidget(copyButton, 4, 3, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 4, 2, 1, 1);

        filterLineEdit = new QLineEdit(FontListDialog);
        filterLineEdit->setObjectName(QString::fromUtf8("filterLineEdit"));

        gridLayout->addWidget(filterLineEdit, 4, 1, 1, 1);

        label_2 = new QLabel(FontListDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 4, 0, 1, 1);

        tableView = new FontListTableView(FontListDialog);
        tableView->setObjectName(QString::fromUtf8("tableView"));
        tableView->setDragEnabled(true);
        tableView->setDragDropMode(QAbstractItemView::DragOnly);
        tableView->setSelectionBehavior(QAbstractItemView::SelectRows);

        gridLayout->addWidget(tableView, 2, 0, 1, 5);

        label = new QLabel(FontListDialog);
        label->setObjectName(QString::fromUtf8("label"));
        label->setTextFormat(Qt::RichText);
        label->setWordWrap(true);

        gridLayout->addWidget(label, 0, 0, 1, 5);

        verticalSpacer = new QSpacerItem(20, 6, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);


        retranslateUi(FontListDialog);

        QMetaObject::connectSlotsByName(FontListDialog);
    } // setupUi

    void retranslateUi(QDialog *FontListDialog)
    {
        FontListDialog->setWindowTitle(q_("OpenSCAD Font List", nullptr));
        okButton->setText(q_("&OK", nullptr));
        copyButton->setText(q_("Copy to Clipboard", nullptr));
        label_2->setText(q_("Filter:", nullptr));
        label->setText(q_("<html><head/><body><p>This list shows the fonts currently registered with OpenSCAD.</p><p>Example:</p><pre style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Courier New,courier';\">  text(&quot;OpenSCAD&quot;, font = &quot;DejaVu Sans&quot;);</span></pre><pre style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Courier New,courier';\">  text(&quot;OpenSCAD&quot;, font = &quot;Liberation Sans:style=Italic&quot;);</span></pre></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FontListDialog: public Ui_FontListDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FONTLISTDIALOG_H
