/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -a openscad_adaptor.h: org.openscad.OpenSCAD.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef OPENSCAD_ADAPTOR_H
#define OPENSCAD_ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
QT_BEGIN_NAMESPACE
class QByteArray;
template<class T> class QList;
template<class Key, class Value> class QMap;
class QString;
class QStringList;
class QVariant;
QT_END_NAMESPACE

/*
 * Adaptor class for interface org.openscad.OpenSCAD
 */
class OpenSCADAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.openscad.OpenSCAD")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.openscad.OpenSCAD\">\n"
"    <method name=\"zoom\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"zoom\"/>\n"
"    </method>\n"
"    <method name=\"zoomTo\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"zoom\"/>\n"
"    </method>\n"
"    <method name=\"rotate\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"x\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"y\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"z\"/>\n"
"    </method>\n"
"    <method name=\"rotateTo\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"x\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"y\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"z\"/>\n"
"    </method>\n"
"    <method name=\"rotateByVector\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"x\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"y\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"z\"/>\n"
"    </method>\n"
"    <method name=\"translate\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"x\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"y\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"z\"/>\n"
"    </method>\n"
"    <method name=\"translateTo\">\n"
"      <arg direction=\"in\" type=\"d\" name=\"x\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"y\"/>\n"
"      <arg direction=\"in\" type=\"d\" name=\"z\"/>\n"
"    </method>\n"
"    <method name=\"action\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"buttonPress\">\n"
"      <arg direction=\"in\" type=\"u\" name=\"idx\"/>\n"
"    </method>\n"
"    <method name=\"getRotation\">\n"
"      <annotation value=\"QList&lt;double&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"      <arg direction=\"out\" type=\"ad\" name=\"rotation\"/>\n"
"    </method>\n"
"    <method name=\"getTranslation\">\n"
"      <annotation value=\"QList&lt;double&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"      <arg direction=\"out\" type=\"ad\" name=\"rotation\"/>\n"
"    </method>\n"
"    <method name=\"getActions\">\n"
"      <arg direction=\"out\" type=\"as\" name=\"names\"/>\n"
"      <annotation value=\"QStringList&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    OpenSCADAdaptor(QObject *parent);
    virtual ~OpenSCADAdaptor();

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void action(const QString &name);
    void buttonPress(uint idx);
    QStringList getActions();
    QList<double> getRotation();
    QList<double> getTranslation();
    void rotate(double x, double y, double z);
    void rotateByVector(double x, double y, double z);
    void rotateTo(double x, double y, double z);
    void translate(double x, double y, double z);
    void translateTo(double x, double y, double z);
    void zoom(double zoom);
    void zoomTo(double zoom);
Q_SIGNALS: // SIGNALS
};

#endif
