/********************************************************************************
** Form generated from reading UI file 'MainWindow.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Console.h"
#include "Dock.h"
#include "ErrorLog.h"
#include "QGLView.h"
#include "QWordSearchField.h"
#include "parameter/ParameterWidget.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *fileActionNew;
    QAction *fileActionNewWindow;
    QAction *fileActionOpen;
    QAction *fileActionOpenWindow;
    QAction *fileActionSave;
    QAction *fileActionSaveAs;
    QAction *fileActionSaveAll;
    QAction *fileActionReload;
    QAction *fileActionQuit;
    QAction *editActionUndo;
    QAction *editActionRedo;
    QAction *editActionRedo_2;
    QAction *editActionCut;
    QAction *editActionCopy;
    QAction *editActionPaste;
    QAction *editActionIndent;
    QAction *editActionComment;
    QAction *editActionUncomment;
    QAction *editActionNextTab;
    QAction *editActionPrevTab;
    QAction *editActionCopyViewport;
    QAction *editActionCopyVPT;
    QAction *editActionCopyVPR;
    QAction *editActionCopyVPD;
    QAction *editActionCopyVPF;
    QAction *editActionZoomTextIn;
    QAction *editActionZoomTextOut;
    QAction *windowActionHideEditor;
    QAction *designActionReloadAndPreview;
    QAction *designActionPreview;
    QAction *designActionRender;
    QAction *designAction3DPrint;
    QAction *designCheckValidity;
    QAction *designActionDisplayAST;
    QAction *designActionDisplayCSGTree;
    QAction *designActionDisplayCSGProducts;
    QAction *fileActionExportSTL;
    QAction *fileActionExportOFF;
    QAction *viewActionPreview;
    QAction *viewActionSurfaces;
    QAction *viewActionWireframe;
    QAction *viewActionThrownTogether;
    QAction *viewActionShowEdges;
    QAction *viewActionShowAxes;
    QAction *viewActionShowCrosshairs;
    QAction *viewActionShowScaleProportional;
    QAction *viewActionAnimate;
    QAction *viewActionTop;
    QAction *viewActionBottom;
    QAction *viewActionLeft;
    QAction *viewActionRight;
    QAction *viewActionFront;
    QAction *viewActionBack;
    QAction *viewActionDiagonal;
    QAction *viewActionCenter;
    QAction *viewActionPerspective;
    QAction *viewActionOrthogonal;
    QAction *windowActionHideConsole;
    QAction *helpActionAbout;
    QAction *helpActionManual;
    QAction *fileActionClearRecent;
    QAction *fileActionExportDXF;
    QAction *fileActionClose;
    QAction *editActionPreferences;
    QAction *editActionFind;
    QAction *editActionFindAndReplace;
    QAction *editActionFindNext;
    QAction *editActionFindPrevious;
    QAction *editActionUseSelectionForFind;
    QAction *editActionJumpToNextError;
    QAction *designActionFlushCaches;
    QAction *helpActionHomepage;
    QAction *designActionAutoReload;
    QAction *fileActionExportImage;
    QAction *fileActionExportCSG;
    QAction *helpActionLibraryInfo;
    QAction *fileShowLibraryFolder;
    QAction *viewActionResetView;
    QAction *helpActionFontInfo;
    QAction *fileActionExportSVG;
    QAction *fileActionExportAMF;
    QAction *fileActionExport3MF;
    QAction *viewActionZoomIn;
    QAction *viewActionZoomOut;
    QAction *viewActionViewAll;
    QAction *editActionConvertTabsToSpaces;
    QAction *editActionToggleBookmark;
    QAction *editActionNextBookmark;
    QAction *editActionPrevBookmark;
    QAction *viewActionHideEditorToolBar;
    QAction *editActionUnindent;
    QAction *helpActionCheatSheet;
    QAction *windowActionHideCustomizer;
    QAction *fileActionExportPDF;
    QAction *viewActionHide3DViewToolBar;
    QAction *windowActionHideErrorLog;
    QAction *windowActionSelectEditor;
    QAction *windowActionSelectConsole;
    QAction *windowActionSelectCustomizer;
    QAction *windowActionSelectErrorLog;
    QAction *windowActionNextWindow;
    QAction *windowActionPreviousWindow;
    QAction *editActionInsertTemplate;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout_2;
    QWidget *mainWidget;
    QVBoxLayout *verticalLayout_3;
    QFrame *frameCompileResult;
    QGridLayout *gridLayoutCompileResult;
    QLabel *labelCompileResultMessage;
    QToolButton *toolButtonCompileResultClose;
    QToolButton *toolButtonCompileResultIcon;
    QGLView *qglview;
    QFrame *animate_panel;
    QHBoxLayout *horizontalLayoutAnimate;
    QLabel *label_time;
    QLineEdit *e_tval;
    QLabel *label_fps;
    QLineEdit *e_fps;
    QLabel *label_steps;
    QLineEdit *e_fsteps;
    QCheckBox *e_dump;
    QToolBar *viewerToolBar;
    QMenuBar *menubar;
    QMenu *menu_File;
    QMenu *menuOpenRecent;
    QMenu *menuExamples;
    QMenu *menuExport;
    QMenu *menu_Edit;
    QMenu *menu_Design;
    QMenu *menu_View;
    QMenu *menuHelp;
    QMenu *menuWindow;
    QToolBar *tabToolBar;
    QWidget *tabToolBarContents;
    QVBoxLayout *verticalLayout_1;
    QStatusBar *statusbar;
    Dock *editorDock;
    QWidget *editorDockContents;
    QVBoxLayout *verticalLayout;
    QToolBar *editortoolbar;
    QFrame *find_panel;
    QGridLayout *gridLayout;
    QComboBox *findTypeComboBox;
    QWordSearchField *findInputField;
    QPushButton *findPrevButton;
    QPushButton *findNextButton;
    QPushButton *cancelButton;
    QLineEdit *replaceInputField;
    QPushButton *replaceButton;
    QPushButton *replaceAllButton;
    Dock *consoleDock;
    QWidget *consoleDockContents;
    QVBoxLayout *verticalLayout_5;
    Console *console;
    Dock *parameterDock;
    QWidget *dockWidgetContents;
    QGridLayout *gridLayoutParameterWidget;
    ParameterWidget *parameterWidget;
    Dock *errorLogDock;
    QWidget *errorLogDockContents;
    QVBoxLayout *verticalLayout_6;
    ErrorLog *errorLogWidget;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName(QString::fromUtf8("MainWindow"));
        MainWindow->resize(1118, 636);
        fileActionNew = new QAction(MainWindow);
        fileActionNew->setObjectName(QString::fromUtf8("fileActionNew"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/blackNew.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionNew->setIcon(icon);
        fileActionNewWindow = new QAction(MainWindow);
        fileActionNewWindow->setObjectName(QString::fromUtf8("fileActionNewWindow"));
        fileActionOpen = new QAction(MainWindow);
        fileActionOpen->setObjectName(QString::fromUtf8("fileActionOpen"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/Open-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionOpen->setIcon(icon1);
        fileActionOpenWindow = new QAction(MainWindow);
        fileActionOpenWindow->setObjectName(QString::fromUtf8("fileActionOpenWindow"));
        fileActionSave = new QAction(MainWindow);
        fileActionSave->setObjectName(QString::fromUtf8("fileActionSave"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/Save-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionSave->setIcon(icon2);
        fileActionSaveAs = new QAction(MainWindow);
        fileActionSaveAs->setObjectName(QString::fromUtf8("fileActionSaveAs"));
        fileActionSaveAll = new QAction(MainWindow);
        fileActionSaveAll->setObjectName(QString::fromUtf8("fileActionSaveAll"));
        fileActionReload = new QAction(MainWindow);
        fileActionReload->setObjectName(QString::fromUtf8("fileActionReload"));
        fileActionQuit = new QAction(MainWindow);
        fileActionQuit->setObjectName(QString::fromUtf8("fileActionQuit"));
        fileActionQuit->setMenuRole(QAction::QuitRole);
        editActionUndo = new QAction(MainWindow);
        editActionUndo->setObjectName(QString::fromUtf8("editActionUndo"));
        editActionUndo->setEnabled(false);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/Command-Undo-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        editActionUndo->setIcon(icon3);
        editActionUndo->setShortcutContext(Qt::WidgetWithChildrenShortcut);
        editActionRedo = new QAction(MainWindow);
        editActionRedo->setObjectName(QString::fromUtf8("editActionRedo"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/images/Command-Redo-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        editActionRedo->setIcon(icon4);
        editActionRedo->setShortcutContext(Qt::WidgetWithChildrenShortcut);
        editActionRedo_2 = new QAction(MainWindow);
        editActionRedo_2->setObjectName(QString::fromUtf8("editActionRedo_2"));
        editActionRedo_2->setShortcutContext(Qt::WidgetWithChildrenShortcut);
        editActionCut = new QAction(MainWindow);
        editActionCut->setObjectName(QString::fromUtf8("editActionCut"));
        editActionCut->setShortcutContext(Qt::WidgetWithChildrenShortcut);
        editActionCopy = new QAction(MainWindow);
        editActionCopy->setObjectName(QString::fromUtf8("editActionCopy"));
        editActionCopy->setShortcutContext(Qt::WidgetWithChildrenShortcut);
        editActionPaste = new QAction(MainWindow);
        editActionPaste->setObjectName(QString::fromUtf8("editActionPaste"));
        editActionPaste->setShortcutContext(Qt::WidgetWithChildrenShortcut);
        editActionIndent = new QAction(MainWindow);
        editActionIndent->setObjectName(QString::fromUtf8("editActionIndent"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/images/Increase-Indent-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        editActionIndent->setIcon(icon5);
        editActionComment = new QAction(MainWindow);
        editActionComment->setObjectName(QString::fromUtf8("editActionComment"));
        editActionUncomment = new QAction(MainWindow);
        editActionUncomment->setObjectName(QString::fromUtf8("editActionUncomment"));
        editActionNextTab = new QAction(MainWindow);
        editActionNextTab->setObjectName(QString::fromUtf8("editActionNextTab"));
        editActionPrevTab = new QAction(MainWindow);
        editActionPrevTab->setObjectName(QString::fromUtf8("editActionPrevTab"));
        editActionCopyViewport = new QAction(MainWindow);
        editActionCopyViewport->setObjectName(QString::fromUtf8("editActionCopyViewport"));
        editActionCopyVPT = new QAction(MainWindow);
        editActionCopyVPT->setObjectName(QString::fromUtf8("editActionCopyVPT"));
        editActionCopyVPR = new QAction(MainWindow);
        editActionCopyVPR->setObjectName(QString::fromUtf8("editActionCopyVPR"));
        editActionCopyVPD = new QAction(MainWindow);
        editActionCopyVPD->setObjectName(QString::fromUtf8("editActionCopyVPD"));
        editActionCopyVPF = new QAction(MainWindow);
        editActionCopyVPF->setObjectName(QString::fromUtf8("editActionCopyVPF"));
        editActionZoomTextIn = new QAction(MainWindow);
        editActionZoomTextIn->setObjectName(QString::fromUtf8("editActionZoomTextIn"));
        editActionZoomTextOut = new QAction(MainWindow);
        editActionZoomTextOut->setObjectName(QString::fromUtf8("editActionZoomTextOut"));
        windowActionHideEditor = new QAction(MainWindow);
        windowActionHideEditor->setObjectName(QString::fromUtf8("windowActionHideEditor"));
        windowActionHideEditor->setCheckable(true);
        designActionReloadAndPreview = new QAction(MainWindow);
        designActionReloadAndPreview->setObjectName(QString::fromUtf8("designActionReloadAndPreview"));
        designActionPreview = new QAction(MainWindow);
        designActionPreview->setObjectName(QString::fromUtf8("designActionPreview"));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/images/preview-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        designActionPreview->setIcon(icon6);
        designActionRender = new QAction(MainWindow);
        designActionRender->setObjectName(QString::fromUtf8("designActionRender"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/images/render-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        designActionRender->setIcon(icon7);
        designAction3DPrint = new QAction(MainWindow);
        designAction3DPrint->setObjectName(QString::fromUtf8("designAction3DPrint"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/images/3dprint-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        designAction3DPrint->setIcon(icon8);
        designCheckValidity = new QAction(MainWindow);
        designCheckValidity->setObjectName(QString::fromUtf8("designCheckValidity"));
        designActionDisplayAST = new QAction(MainWindow);
        designActionDisplayAST->setObjectName(QString::fromUtf8("designActionDisplayAST"));
        designActionDisplayCSGTree = new QAction(MainWindow);
        designActionDisplayCSGTree->setObjectName(QString::fromUtf8("designActionDisplayCSGTree"));
        designActionDisplayCSGProducts = new QAction(MainWindow);
        designActionDisplayCSGProducts->setObjectName(QString::fromUtf8("designActionDisplayCSGProducts"));
        fileActionExportSTL = new QAction(MainWindow);
        fileActionExportSTL->setObjectName(QString::fromUtf8("fileActionExportSTL"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/images/STL.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportSTL->setIcon(icon9);
        fileActionExportOFF = new QAction(MainWindow);
        fileActionExportOFF->setObjectName(QString::fromUtf8("fileActionExportOFF"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/images/OFF.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportOFF->setIcon(icon10);
        viewActionPreview = new QAction(MainWindow);
        viewActionPreview->setObjectName(QString::fromUtf8("viewActionPreview"));
        viewActionPreview->setCheckable(true);
        viewActionSurfaces = new QAction(MainWindow);
        viewActionSurfaces->setObjectName(QString::fromUtf8("viewActionSurfaces"));
        viewActionSurfaces->setCheckable(true);
        viewActionWireframe = new QAction(MainWindow);
        viewActionWireframe->setObjectName(QString::fromUtf8("viewActionWireframe"));
        viewActionWireframe->setCheckable(true);
        viewActionThrownTogether = new QAction(MainWindow);
        viewActionThrownTogether->setObjectName(QString::fromUtf8("viewActionThrownTogether"));
        viewActionThrownTogether->setCheckable(true);
        viewActionShowEdges = new QAction(MainWindow);
        viewActionShowEdges->setObjectName(QString::fromUtf8("viewActionShowEdges"));
        viewActionShowEdges->setCheckable(true);
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/images/Rotation-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionShowEdges->setIcon(icon11);
        viewActionShowAxes = new QAction(MainWindow);
        viewActionShowAxes->setObjectName(QString::fromUtf8("viewActionShowAxes"));
        viewActionShowAxes->setCheckable(true);
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/images/blackaxes.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionShowAxes->setIcon(icon12);
        viewActionShowCrosshairs = new QAction(MainWindow);
        viewActionShowCrosshairs->setObjectName(QString::fromUtf8("viewActionShowCrosshairs"));
        viewActionShowCrosshairs->setCheckable(true);
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/images/cross.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionShowCrosshairs->setIcon(icon13);
        viewActionShowScaleProportional = new QAction(MainWindow);
        viewActionShowScaleProportional->setObjectName(QString::fromUtf8("viewActionShowScaleProportional"));
        viewActionShowScaleProportional->setCheckable(true);
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/images/scalemarkers.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionShowScaleProportional->setIcon(icon14);
        viewActionAnimate = new QAction(MainWindow);
        viewActionAnimate->setObjectName(QString::fromUtf8("viewActionAnimate"));
        viewActionAnimate->setCheckable(true);
        viewActionTop = new QAction(MainWindow);
        viewActionTop->setObjectName(QString::fromUtf8("viewActionTop"));
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/images/blackUp.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionTop->setIcon(icon15);
        viewActionBottom = new QAction(MainWindow);
        viewActionBottom->setObjectName(QString::fromUtf8("viewActionBottom"));
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/images/blackbottom.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionBottom->setIcon(icon16);
        viewActionLeft = new QAction(MainWindow);
        viewActionLeft->setObjectName(QString::fromUtf8("viewActionLeft"));
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/images/blackleft.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionLeft->setIcon(icon17);
        viewActionRight = new QAction(MainWindow);
        viewActionRight->setObjectName(QString::fromUtf8("viewActionRight"));
        QIcon icon18;
        icon18.addFile(QString::fromUtf8(":/images/rightright.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionRight->setIcon(icon18);
        viewActionFront = new QAction(MainWindow);
        viewActionFront->setObjectName(QString::fromUtf8("viewActionFront"));
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/images/blackfront.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionFront->setIcon(icon19);
        viewActionBack = new QAction(MainWindow);
        viewActionBack->setObjectName(QString::fromUtf8("viewActionBack"));
        QIcon icon20;
        icon20.addFile(QString::fromUtf8(":/images/blackback.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionBack->setIcon(icon20);
        viewActionDiagonal = new QAction(MainWindow);
        viewActionDiagonal->setObjectName(QString::fromUtf8("viewActionDiagonal"));
        viewActionCenter = new QAction(MainWindow);
        viewActionCenter->setObjectName(QString::fromUtf8("viewActionCenter"));
        viewActionPerspective = new QAction(MainWindow);
        viewActionPerspective->setObjectName(QString::fromUtf8("viewActionPerspective"));
        viewActionPerspective->setCheckable(true);
        QIcon icon21;
        icon21.addFile(QString::fromUtf8(":/images/perspective1.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionPerspective->setIcon(icon21);
        viewActionOrthogonal = new QAction(MainWindow);
        viewActionOrthogonal->setObjectName(QString::fromUtf8("viewActionOrthogonal"));
        viewActionOrthogonal->setCheckable(true);
        QIcon icon22;
        icon22.addFile(QString::fromUtf8(":/images/orthogonal.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionOrthogonal->setIcon(icon22);
        windowActionHideConsole = new QAction(MainWindow);
        windowActionHideConsole->setObjectName(QString::fromUtf8("windowActionHideConsole"));
        windowActionHideConsole->setCheckable(true);
        helpActionAbout = new QAction(MainWindow);
        helpActionAbout->setObjectName(QString::fromUtf8("helpActionAbout"));
        helpActionAbout->setEnabled(true);
        helpActionAbout->setMenuRole(QAction::AboutRole);
        helpActionManual = new QAction(MainWindow);
        helpActionManual->setObjectName(QString::fromUtf8("helpActionManual"));
        fileActionClearRecent = new QAction(MainWindow);
        fileActionClearRecent->setObjectName(QString::fromUtf8("fileActionClearRecent"));
        fileActionExportDXF = new QAction(MainWindow);
        fileActionExportDXF->setObjectName(QString::fromUtf8("fileActionExportDXF"));
        QIcon icon23;
        icon23.addFile(QString::fromUtf8(":/images/DXF.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportDXF->setIcon(icon23);
        fileActionClose = new QAction(MainWindow);
        fileActionClose->setObjectName(QString::fromUtf8("fileActionClose"));
        editActionPreferences = new QAction(MainWindow);
        editActionPreferences->setObjectName(QString::fromUtf8("editActionPreferences"));
        editActionPreferences->setMenuRole(QAction::PreferencesRole);
        editActionFind = new QAction(MainWindow);
        editActionFind->setObjectName(QString::fromUtf8("editActionFind"));
        editActionFindAndReplace = new QAction(MainWindow);
        editActionFindAndReplace->setObjectName(QString::fromUtf8("editActionFindAndReplace"));
        editActionFindNext = new QAction(MainWindow);
        editActionFindNext->setObjectName(QString::fromUtf8("editActionFindNext"));
        editActionFindPrevious = new QAction(MainWindow);
        editActionFindPrevious->setObjectName(QString::fromUtf8("editActionFindPrevious"));
        editActionUseSelectionForFind = new QAction(MainWindow);
        editActionUseSelectionForFind->setObjectName(QString::fromUtf8("editActionUseSelectionForFind"));
        editActionJumpToNextError = new QAction(MainWindow);
        editActionJumpToNextError->setObjectName(QString::fromUtf8("editActionJumpToNextError"));
        designActionFlushCaches = new QAction(MainWindow);
        designActionFlushCaches->setObjectName(QString::fromUtf8("designActionFlushCaches"));
        helpActionHomepage = new QAction(MainWindow);
        helpActionHomepage->setObjectName(QString::fromUtf8("helpActionHomepage"));
        designActionAutoReload = new QAction(MainWindow);
        designActionAutoReload->setObjectName(QString::fromUtf8("designActionAutoReload"));
        designActionAutoReload->setCheckable(true);
        fileActionExportImage = new QAction(MainWindow);
        fileActionExportImage->setObjectName(QString::fromUtf8("fileActionExportImage"));
        QIcon icon24;
        icon24.addFile(QString::fromUtf8(":/images/PNG.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportImage->setIcon(icon24);
        fileActionExportCSG = new QAction(MainWindow);
        fileActionExportCSG->setObjectName(QString::fromUtf8("fileActionExportCSG"));
        QIcon icon25;
        icon25.addFile(QString::fromUtf8(":/images/CSG.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportCSG->setIcon(icon25);
        helpActionLibraryInfo = new QAction(MainWindow);
        helpActionLibraryInfo->setObjectName(QString::fromUtf8("helpActionLibraryInfo"));
        fileShowLibraryFolder = new QAction(MainWindow);
        fileShowLibraryFolder->setObjectName(QString::fromUtf8("fileShowLibraryFolder"));
        viewActionResetView = new QAction(MainWindow);
        viewActionResetView->setObjectName(QString::fromUtf8("viewActionResetView"));
        QIcon icon26;
        icon26.addFile(QString::fromUtf8(":/images/Command-Reset-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionResetView->setIcon(icon26);
        helpActionFontInfo = new QAction(MainWindow);
        helpActionFontInfo->setObjectName(QString::fromUtf8("helpActionFontInfo"));
        fileActionExportSVG = new QAction(MainWindow);
        fileActionExportSVG->setObjectName(QString::fromUtf8("fileActionExportSVG"));
        fileActionExportSVG->setIcon(icon25);
        fileActionExportAMF = new QAction(MainWindow);
        fileActionExportAMF->setObjectName(QString::fromUtf8("fileActionExportAMF"));
        QIcon icon27;
        icon27.addFile(QString::fromUtf8(":/images/AMF.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportAMF->setIcon(icon27);
        fileActionExport3MF = new QAction(MainWindow);
        fileActionExport3MF->setObjectName(QString::fromUtf8("fileActionExport3MF"));
        QIcon icon28;
        icon28.addFile(QString::fromUtf8(":/images/3MF.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExport3MF->setIcon(icon28);
        viewActionZoomIn = new QAction(MainWindow);
        viewActionZoomIn->setObjectName(QString::fromUtf8("viewActionZoomIn"));
        QIcon icon29;
        icon29.addFile(QString::fromUtf8(":/images/zoomin.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionZoomIn->setIcon(icon29);
        viewActionZoomOut = new QAction(MainWindow);
        viewActionZoomOut->setObjectName(QString::fromUtf8("viewActionZoomOut"));
        QIcon icon30;
        icon30.addFile(QString::fromUtf8(":/images/zoomout.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionZoomOut->setIcon(icon30);
        viewActionViewAll = new QAction(MainWindow);
        viewActionViewAll->setObjectName(QString::fromUtf8("viewActionViewAll"));
        QIcon icon31;
        icon31.addFile(QString::fromUtf8(":/images/zoom-all.png"), QSize(), QIcon::Normal, QIcon::Off);
        viewActionViewAll->setIcon(icon31);
        editActionConvertTabsToSpaces = new QAction(MainWindow);
        editActionConvertTabsToSpaces->setObjectName(QString::fromUtf8("editActionConvertTabsToSpaces"));
        editActionToggleBookmark = new QAction(MainWindow);
        editActionToggleBookmark->setObjectName(QString::fromUtf8("editActionToggleBookmark"));
        editActionNextBookmark = new QAction(MainWindow);
        editActionNextBookmark->setObjectName(QString::fromUtf8("editActionNextBookmark"));
        editActionPrevBookmark = new QAction(MainWindow);
        editActionPrevBookmark->setObjectName(QString::fromUtf8("editActionPrevBookmark"));
        viewActionHideEditorToolBar = new QAction(MainWindow);
        viewActionHideEditorToolBar->setObjectName(QString::fromUtf8("viewActionHideEditorToolBar"));
        viewActionHideEditorToolBar->setCheckable(true);
        editActionUnindent = new QAction(MainWindow);
        editActionUnindent->setObjectName(QString::fromUtf8("editActionUnindent"));
        QIcon icon32;
        icon32.addFile(QString::fromUtf8(":/images/Decrease-Indent-32.png"), QSize(), QIcon::Normal, QIcon::Off);
        editActionUnindent->setIcon(icon32);
        helpActionCheatSheet = new QAction(MainWindow);
        helpActionCheatSheet->setObjectName(QString::fromUtf8("helpActionCheatSheet"));
        windowActionHideCustomizer = new QAction(MainWindow);
        windowActionHideCustomizer->setObjectName(QString::fromUtf8("windowActionHideCustomizer"));
        windowActionHideCustomizer->setCheckable(true);
        windowActionHideCustomizer->setChecked(true);
        fileActionExportPDF = new QAction(MainWindow);
        fileActionExportPDF->setObjectName(QString::fromUtf8("fileActionExportPDF"));
        QIcon icon33;
        icon33.addFile(QString::fromUtf8(":/images/PDF.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileActionExportPDF->setIcon(icon33);
        viewActionHide3DViewToolBar = new QAction(MainWindow);
        viewActionHide3DViewToolBar->setObjectName(QString::fromUtf8("viewActionHide3DViewToolBar"));
        viewActionHide3DViewToolBar->setCheckable(true);
        windowActionHideErrorLog = new QAction(MainWindow);
        windowActionHideErrorLog->setObjectName(QString::fromUtf8("windowActionHideErrorLog"));
        windowActionHideErrorLog->setCheckable(true);
        windowActionHideErrorLog->setChecked(true);
        windowActionSelectEditor = new QAction(MainWindow);
        windowActionSelectEditor->setObjectName(QString::fromUtf8("windowActionSelectEditor"));
        windowActionSelectConsole = new QAction(MainWindow);
        windowActionSelectConsole->setObjectName(QString::fromUtf8("windowActionSelectConsole"));
        windowActionSelectCustomizer = new QAction(MainWindow);
        windowActionSelectCustomizer->setObjectName(QString::fromUtf8("windowActionSelectCustomizer"));
        windowActionSelectErrorLog = new QAction(MainWindow);
        windowActionSelectErrorLog->setObjectName(QString::fromUtf8("windowActionSelectErrorLog"));
        windowActionNextWindow = new QAction(MainWindow);
        windowActionNextWindow->setObjectName(QString::fromUtf8("windowActionNextWindow"));
        windowActionPreviousWindow = new QAction(MainWindow);
        windowActionPreviousWindow->setObjectName(QString::fromUtf8("windowActionPreviousWindow"));
        editActionInsertTemplate = new QAction(MainWindow);
        editActionInsertTemplate->setObjectName(QString::fromUtf8("editActionInsertTemplate"));
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        verticalLayout_2 = new QVBoxLayout(centralwidget);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        mainWidget = new QWidget(centralwidget);
        mainWidget->setObjectName(QString::fromUtf8("mainWidget"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mainWidget->sizePolicy().hasHeightForWidth());
        mainWidget->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(mainWidget);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        frameCompileResult = new QFrame(mainWidget);
        frameCompileResult->setObjectName(QString::fromUtf8("frameCompileResult"));
        frameCompileResult->setFrameShape(QFrame::Box);
        frameCompileResult->setFrameShadow(QFrame::Raised);
        gridLayoutCompileResult = new QGridLayout(frameCompileResult);
        gridLayoutCompileResult->setObjectName(QString::fromUtf8("gridLayoutCompileResult"));
        gridLayoutCompileResult->setHorizontalSpacing(4);
        gridLayoutCompileResult->setContentsMargins(6, 6, 6, 4);
        labelCompileResultMessage = new QLabel(frameCompileResult);
        labelCompileResultMessage->setObjectName(QString::fromUtf8("labelCompileResultMessage"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelCompileResultMessage->sizePolicy().hasHeightForWidth());
        labelCompileResultMessage->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        labelCompileResultMessage->setFont(font);
        labelCompileResultMessage->setWordWrap(true);

        gridLayoutCompileResult->addWidget(labelCompileResultMessage, 0, 1, 1, 1);

        toolButtonCompileResultClose = new QToolButton(frameCompileResult);
        toolButtonCompileResultClose->setObjectName(QString::fromUtf8("toolButtonCompileResultClose"));
        toolButtonCompileResultClose->setStyleSheet(QString::fromUtf8(""));
        QIcon icon34;
        icon34.addFile(QString::fromUtf8(":/icons/close.png"), QSize(), QIcon::Normal, QIcon::Off);
        toolButtonCompileResultClose->setIcon(icon34);
        toolButtonCompileResultClose->setIconSize(QSize(22, 22));
        toolButtonCompileResultClose->setToolButtonStyle(Qt::ToolButtonIconOnly);
        toolButtonCompileResultClose->setAutoRaise(true);

        gridLayoutCompileResult->addWidget(toolButtonCompileResultClose, 0, 2, 1, 1);

        toolButtonCompileResultIcon = new QToolButton(frameCompileResult);
        toolButtonCompileResultIcon->setObjectName(QString::fromUtf8("toolButtonCompileResultIcon"));
        toolButtonCompileResultIcon->setStyleSheet(QString::fromUtf8("QToolButton { border: none; }"));
        QIcon icon35;
        icon35.addFile(QString::fromUtf8(":/icons/information-icons-error.png"), QSize(), QIcon::Normal, QIcon::Off);
        toolButtonCompileResultIcon->setIcon(icon35);
        toolButtonCompileResultIcon->setIconSize(QSize(22, 22));
        toolButtonCompileResultIcon->setAutoRaise(false);

        gridLayoutCompileResult->addWidget(toolButtonCompileResultIcon, 0, 0, 1, 1);


        verticalLayout_3->addWidget(frameCompileResult);

        qglview = new QGLView(mainWidget);
        qglview->setObjectName(QString::fromUtf8("qglview"));
        sizePolicy.setHeightForWidth(qglview->sizePolicy().hasHeightForWidth());
        qglview->setSizePolicy(sizePolicy);
        qglview->setMinimumSize(QSize(200, 100));

        verticalLayout_3->addWidget(qglview);

        animate_panel = new QFrame(mainWidget);
        animate_panel->setObjectName(QString::fromUtf8("animate_panel"));
        animate_panel->setEnabled(true);
        animate_panel->setFrameShape(QFrame::NoFrame);
        animate_panel->setFrameShadow(QFrame::Raised);
        animate_panel->setLineWidth(0);
        horizontalLayoutAnimate = new QHBoxLayout(animate_panel);
        horizontalLayoutAnimate->setObjectName(QString::fromUtf8("horizontalLayoutAnimate"));
        horizontalLayoutAnimate->setContentsMargins(0, 0, 0, 0);
        label_time = new QLabel(animate_panel);
        label_time->setObjectName(QString::fromUtf8("label_time"));

        horizontalLayoutAnimate->addWidget(label_time);

        e_tval = new QLineEdit(animate_panel);
        e_tval->setObjectName(QString::fromUtf8("e_tval"));

        horizontalLayoutAnimate->addWidget(e_tval);

        label_fps = new QLabel(animate_panel);
        label_fps->setObjectName(QString::fromUtf8("label_fps"));

        horizontalLayoutAnimate->addWidget(label_fps);

        e_fps = new QLineEdit(animate_panel);
        e_fps->setObjectName(QString::fromUtf8("e_fps"));

        horizontalLayoutAnimate->addWidget(e_fps);

        label_steps = new QLabel(animate_panel);
        label_steps->setObjectName(QString::fromUtf8("label_steps"));

        horizontalLayoutAnimate->addWidget(label_steps);

        e_fsteps = new QLineEdit(animate_panel);
        e_fsteps->setObjectName(QString::fromUtf8("e_fsteps"));

        horizontalLayoutAnimate->addWidget(e_fsteps);

        e_dump = new QCheckBox(animate_panel);
        e_dump->setObjectName(QString::fromUtf8("e_dump"));

        horizontalLayoutAnimate->addWidget(e_dump);


        verticalLayout_3->addWidget(animate_panel);

        viewerToolBar = new QToolBar(mainWidget);
        viewerToolBar->setObjectName(QString::fromUtf8("viewerToolBar"));

        verticalLayout_3->addWidget(viewerToolBar);

        animate_panel->raise();
        qglview->raise();
        viewerToolBar->raise();
        frameCompileResult->raise();

        verticalLayout_2->addWidget(mainWidget);

        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName(QString::fromUtf8("menubar"));
        menubar->setGeometry(QRect(0, 0, 1118, 22));
        menu_File = new QMenu(menubar);
        menu_File->setObjectName(QString::fromUtf8("menu_File"));
        menuOpenRecent = new QMenu(menu_File);
        menuOpenRecent->setObjectName(QString::fromUtf8("menuOpenRecent"));
        menuExamples = new QMenu(menu_File);
        menuExamples->setObjectName(QString::fromUtf8("menuExamples"));
        menuExport = new QMenu(menu_File);
        menuExport->setObjectName(QString::fromUtf8("menuExport"));
        menu_Edit = new QMenu(menubar);
        menu_Edit->setObjectName(QString::fromUtf8("menu_Edit"));
        menu_Design = new QMenu(menubar);
        menu_Design->setObjectName(QString::fromUtf8("menu_Design"));
        menu_View = new QMenu(menubar);
        menu_View->setObjectName(QString::fromUtf8("menu_View"));
        menuHelp = new QMenu(menubar);
        menuHelp->setObjectName(QString::fromUtf8("menuHelp"));
        menuWindow = new QMenu(menubar);
        menuWindow->setObjectName(QString::fromUtf8("menuWindow"));
        MainWindow->setMenuBar(menubar);
        tabToolBar = new QToolBar(MainWindow);
        tabToolBar->setObjectName(QString::fromUtf8("tabToolBar"));
        tabToolBar->setMovable(false);
        tabToolBar->setFloatable(false);
        tabToolBarContents = new QWidget(tabToolBar);
        tabToolBarContents->setObjectName(QString::fromUtf8("tabToolBarContents"));
        sizePolicy.setHeightForWidth(tabToolBarContents->sizePolicy().hasHeightForWidth());
        tabToolBarContents->setSizePolicy(sizePolicy);
        verticalLayout_1 = new QVBoxLayout(tabToolBarContents);
        verticalLayout_1->setSpacing(0);
        verticalLayout_1->setObjectName(QString::fromUtf8("verticalLayout_1"));
        verticalLayout_1->setContentsMargins(0, 0, 0, 0);
        tabToolBar->addWidget(tabToolBarContents);
        MainWindow->addToolBar(Qt::TopToolBarArea, tabToolBar);
        statusbar = new QStatusBar(MainWindow);
        statusbar->setObjectName(QString::fromUtf8("statusbar"));
        statusbar->setMaximumSize(QSize(16777215, 23));
        MainWindow->setStatusBar(statusbar);
        editorDock = new Dock(MainWindow);
        editorDock->setObjectName(QString::fromUtf8("editorDock"));
        editorDockContents = new QWidget();
        editorDockContents->setObjectName(QString::fromUtf8("editorDockContents"));
        verticalLayout = new QVBoxLayout(editorDockContents);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        editortoolbar = new QToolBar(editorDockContents);
        editortoolbar->setObjectName(QString::fromUtf8("editortoolbar"));

        verticalLayout->addWidget(editortoolbar);

        find_panel = new QFrame(editorDockContents);
        find_panel->setObjectName(QString::fromUtf8("find_panel"));
        find_panel->setEnabled(true);
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(find_panel->sizePolicy().hasHeightForWidth());
        find_panel->setSizePolicy(sizePolicy2);
        find_panel->setFrameShape(QFrame::NoFrame);
        find_panel->setFrameShadow(QFrame::Raised);
        find_panel->setLineWidth(0);
        gridLayout = new QGridLayout(find_panel);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setVerticalSpacing(0);
        gridLayout->setContentsMargins(-1, 0, -1, 0);
        findTypeComboBox = new QComboBox(find_panel);
        findTypeComboBox->addItem(QString());
        findTypeComboBox->addItem(QString());
        findTypeComboBox->setObjectName(QString::fromUtf8("findTypeComboBox"));

        gridLayout->addWidget(findTypeComboBox, 0, 0, 1, 1);

        findInputField = new QWordSearchField(find_panel);
        findInputField->setObjectName(QString::fromUtf8("findInputField"));

        gridLayout->addWidget(findInputField, 0, 1, 1, 1);

        findPrevButton = new QPushButton(find_panel);
        findPrevButton->setObjectName(QString::fromUtf8("findPrevButton"));

        gridLayout->addWidget(findPrevButton, 0, 2, 1, 1);

        findNextButton = new QPushButton(find_panel);
        findNextButton->setObjectName(QString::fromUtf8("findNextButton"));

        gridLayout->addWidget(findNextButton, 0, 3, 1, 1);

        cancelButton = new QPushButton(find_panel);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        gridLayout->addWidget(cancelButton, 0, 4, 1, 1);

        replaceInputField = new QLineEdit(find_panel);
        replaceInputField->setObjectName(QString::fromUtf8("replaceInputField"));

        gridLayout->addWidget(replaceInputField, 1, 1, 1, 1);

        replaceButton = new QPushButton(find_panel);
        replaceButton->setObjectName(QString::fromUtf8("replaceButton"));

        gridLayout->addWidget(replaceButton, 1, 2, 1, 2);

        replaceAllButton = new QPushButton(find_panel);
        replaceAllButton->setObjectName(QString::fromUtf8("replaceAllButton"));

        gridLayout->addWidget(replaceAllButton, 1, 4, 1, 1);


        verticalLayout->addWidget(find_panel);

        editorDock->setWidget(editorDockContents);
        MainWindow->addDockWidget(Qt::LeftDockWidgetArea, editorDock);
        consoleDock = new Dock(MainWindow);
        consoleDock->setObjectName(QString::fromUtf8("consoleDock"));
        consoleDockContents = new QWidget();
        consoleDockContents->setObjectName(QString::fromUtf8("consoleDockContents"));
        consoleDockContents->setFocusPolicy(Qt::NoFocus);
        verticalLayout_5 = new QVBoxLayout(consoleDockContents);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        console = new Console(consoleDockContents);
        console->setObjectName(QString::fromUtf8("console"));
        console->setMinimumSize(QSize(10, 10));
        console->setFocusPolicy(Qt::StrongFocus);
        console->setReadOnly(true);

        verticalLayout_5->addWidget(console);

        consoleDock->setWidget(consoleDockContents);
        MainWindow->addDockWidget(Qt::BottomDockWidgetArea, consoleDock);
        parameterDock = new Dock(MainWindow);
        parameterDock->setObjectName(QString::fromUtf8("parameterDock"));
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName(QString::fromUtf8("dockWidgetContents"));
        gridLayoutParameterWidget = new QGridLayout(dockWidgetContents);
        gridLayoutParameterWidget->setSpacing(0);
        gridLayoutParameterWidget->setObjectName(QString::fromUtf8("gridLayoutParameterWidget"));
        gridLayoutParameterWidget->setContentsMargins(0, 0, 0, 0);
        parameterWidget = new ParameterWidget(dockWidgetContents);
        parameterWidget->setObjectName(QString::fromUtf8("parameterWidget"));

        gridLayoutParameterWidget->addWidget(parameterWidget, 0, 0, 1, 1);

        parameterDock->setWidget(dockWidgetContents);
        MainWindow->addDockWidget(Qt::RightDockWidgetArea, parameterDock);
        errorLogDock = new Dock(MainWindow);
        errorLogDock->setObjectName(QString::fromUtf8("errorLogDock"));
        errorLogDockContents = new QWidget();
        errorLogDockContents->setObjectName(QString::fromUtf8("errorLogDockContents"));
        errorLogDockContents->setFocusPolicy(Qt::StrongFocus);
        verticalLayout_6 = new QVBoxLayout(errorLogDockContents);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        errorLogWidget = new ErrorLog(errorLogDockContents);
        errorLogWidget->setObjectName(QString::fromUtf8("errorLogWidget"));
        errorLogWidget->setMinimumSize(QSize(10, 10));

        verticalLayout_6->addWidget(errorLogWidget);

        errorLogDock->setWidget(errorLogDockContents);
        MainWindow->addDockWidget(Qt::BottomDockWidgetArea, errorLogDock);

        viewerToolBar->addAction(designActionPreview);
        viewerToolBar->addAction(designActionRender);
        viewerToolBar->addSeparator();
        viewerToolBar->addAction(viewActionViewAll);
        viewerToolBar->addAction(viewActionZoomIn);
        viewerToolBar->addAction(viewActionZoomOut);
        viewerToolBar->addAction(viewActionResetView);
        viewerToolBar->addSeparator();
        viewerToolBar->addAction(viewActionRight);
        viewerToolBar->addAction(viewActionTop);
        viewerToolBar->addAction(viewActionBottom);
        viewerToolBar->addAction(viewActionLeft);
        viewerToolBar->addAction(viewActionFront);
        viewerToolBar->addAction(viewActionBack);
        viewerToolBar->addSeparator();
        viewerToolBar->addAction(viewActionPerspective);
        viewerToolBar->addAction(viewActionOrthogonal);
        viewerToolBar->addSeparator();
        viewerToolBar->addAction(viewActionShowAxes);
        viewerToolBar->addAction(viewActionShowScaleProportional);
        viewerToolBar->addAction(viewActionShowEdges);
        menubar->addAction(menu_File->menuAction());
        menubar->addAction(menu_Edit->menuAction());
        menubar->addAction(menu_Design->menuAction());
        menubar->addAction(menu_View->menuAction());
        menubar->addAction(menuWindow->menuAction());
        menubar->addAction(menuHelp->menuAction());
        menu_File->addAction(fileActionNew);
        menu_File->addAction(fileActionOpen);
        menu_File->addAction(menuOpenRecent->menuAction());
        menu_File->addAction(menuExamples->menuAction());
        menu_File->addAction(fileActionReload);
        menu_File->addSeparator();
        menu_File->addAction(fileActionNewWindow);
        menu_File->addAction(fileActionOpenWindow);
        menu_File->addAction(fileActionClose);
        menu_File->addSeparator();
        menu_File->addAction(fileActionSave);
        menu_File->addAction(fileActionSaveAs);
        menu_File->addAction(fileActionSaveAll);
        menu_File->addSeparator();
        menu_File->addAction(menuExport->menuAction());
        menu_File->addSeparator();
        menu_File->addAction(fileShowLibraryFolder);
        menu_File->addSeparator();
        menu_File->addAction(fileActionQuit);
        menuExport->addAction(fileActionExportSTL);
        menuExport->addAction(fileActionExportOFF);
        menuExport->addAction(fileActionExportAMF);
        menuExport->addAction(fileActionExport3MF);
        menuExport->addAction(fileActionExportDXF);
        menuExport->addAction(fileActionExportSVG);
        menuExport->addAction(fileActionExportCSG);
        menuExport->addAction(fileActionExportPDF);
        menuExport->addSeparator();
        menuExport->addAction(fileActionExportImage);
        menu_Edit->addAction(editActionUndo);
        menu_Edit->addAction(editActionRedo);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionCut);
        menu_Edit->addAction(editActionCopy);
        menu_Edit->addAction(editActionPaste);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionIndent);
        menu_Edit->addAction(editActionUnindent);
        menu_Edit->addAction(editActionComment);
        menu_Edit->addAction(editActionUncomment);
        menu_Edit->addAction(editActionConvertTabsToSpaces);
        menu_Edit->addAction(editActionToggleBookmark);
        menu_Edit->addAction(editActionNextBookmark);
        menu_Edit->addAction(editActionPrevBookmark);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionNextTab);
        menu_Edit->addAction(editActionPrevTab);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionCopyViewport);
        menu_Edit->addAction(editActionCopyVPT);
        menu_Edit->addAction(editActionCopyVPR);
        menu_Edit->addAction(editActionCopyVPD);
        menu_Edit->addAction(editActionCopyVPF);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionFind);
        menu_Edit->addAction(editActionFindAndReplace);
        menu_Edit->addAction(editActionFindNext);
        menu_Edit->addAction(editActionFindPrevious);
        menu_Edit->addAction(editActionUseSelectionForFind);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionJumpToNextError);
        menu_Edit->addSeparator();
        menu_Edit->addAction(editActionZoomTextIn);
        menu_Edit->addAction(editActionZoomTextOut);
        menu_Edit->addAction(editActionPreferences);
        menu_Design->addAction(designActionAutoReload);
        menu_Design->addAction(designActionReloadAndPreview);
        menu_Design->addAction(designActionPreview);
        menu_Design->addAction(designActionRender);
        menu_Design->addAction(designAction3DPrint);
        menu_Design->addSeparator();
        menu_Design->addAction(designCheckValidity);
        menu_Design->addAction(designActionDisplayAST);
        menu_Design->addAction(designActionDisplayCSGTree);
        menu_Design->addAction(designActionDisplayCSGProducts);
        menu_Design->addSeparator();
        menu_Design->addAction(designActionFlushCaches);
        menu_View->addAction(viewActionPreview);
        menu_View->addAction(viewActionSurfaces);
        menu_View->addAction(viewActionWireframe);
        menu_View->addAction(viewActionThrownTogether);
        menu_View->addSeparator();
        menu_View->addAction(viewActionShowEdges);
        menu_View->addAction(viewActionShowAxes);
        menu_View->addAction(viewActionShowScaleProportional);
        menu_View->addAction(viewActionShowCrosshairs);
        menu_View->addAction(viewActionAnimate);
        menu_View->addSeparator();
        menu_View->addAction(viewActionTop);
        menu_View->addAction(viewActionBottom);
        menu_View->addAction(viewActionLeft);
        menu_View->addAction(viewActionRight);
        menu_View->addAction(viewActionFront);
        menu_View->addAction(viewActionBack);
        menu_View->addAction(viewActionDiagonal);
        menu_View->addAction(viewActionCenter);
        menu_View->addAction(viewActionViewAll);
        menu_View->addAction(viewActionResetView);
        menu_View->addSeparator();
        menu_View->addAction(viewActionZoomIn);
        menu_View->addAction(viewActionZoomOut);
        menu_View->addSeparator();
        menu_View->addAction(viewActionPerspective);
        menu_View->addAction(viewActionOrthogonal);
        menu_View->addSeparator();
        menu_View->addAction(viewActionHideEditorToolBar);
        menu_View->addAction(viewActionHide3DViewToolBar);
        menuHelp->addAction(helpActionAbout);
        menuHelp->addAction(helpActionHomepage);
        menuHelp->addAction(helpActionManual);
        menuHelp->addAction(helpActionCheatSheet);
        menuHelp->addAction(helpActionLibraryInfo);
        menuHelp->addAction(helpActionFontInfo);
        menuWindow->addAction(windowActionSelectEditor);
        menuWindow->addAction(windowActionSelectConsole);
        menuWindow->addAction(windowActionSelectErrorLog);
        menuWindow->addAction(windowActionSelectCustomizer);
        menuWindow->addSeparator();
        menuWindow->addAction(windowActionNextWindow);
        menuWindow->addAction(windowActionPreviousWindow);
        menuWindow->addSeparator();
        menuWindow->addAction(windowActionHideEditor);
        menuWindow->addAction(windowActionHideConsole);
        menuWindow->addAction(windowActionHideErrorLog);
        menuWindow->addAction(windowActionHideCustomizer);
        editorDock->addAction(editActionRedo_2);
        editortoolbar->addAction(fileActionNew);
        editortoolbar->addAction(fileActionOpen);
        editortoolbar->addAction(fileActionSave);
        editortoolbar->addSeparator();
        editortoolbar->addAction(editActionUndo);
        editortoolbar->addAction(editActionRedo);
        editortoolbar->addAction(editActionUnindent);
        editortoolbar->addAction(editActionIndent);
        editortoolbar->addSeparator();
        editortoolbar->addAction(designActionPreview);
        editortoolbar->addAction(designActionRender);
        editortoolbar->addSeparator();
        editortoolbar->addAction(fileActionExportSTL);
        editortoolbar->addAction(designAction3DPrint);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        fileActionNew->setText(q_("&New File", nullptr));
#if QT_CONFIG(shortcut)
        fileActionNew->setShortcut(q_("Ctrl+N", nullptr));
#endif // QT_CONFIG(shortcut)
        fileActionNewWindow->setText(q_("New Window", nullptr));
        fileActionOpen->setText(q_("&Open File", nullptr));
#if QT_CONFIG(shortcut)
        fileActionOpen->setShortcut(q_("Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        fileActionOpenWindow->setText(q_("Open in New Window", nullptr));
        fileActionSave->setText(q_("&Save", nullptr));
#if QT_CONFIG(shortcut)
        fileActionSave->setShortcut(q_("Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        fileActionSaveAs->setText(q_("Save &As...", nullptr));
#if QT_CONFIG(shortcut)
        fileActionSaveAs->setShortcut(q_("Ctrl+Shift+S", nullptr));
#endif // QT_CONFIG(shortcut)
        fileActionSaveAll->setText(q_("Save All", nullptr));
        fileActionReload->setText(q_("&Reload", nullptr));
#if QT_CONFIG(shortcut)
        fileActionReload->setShortcut(q_("Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
        fileActionQuit->setText(q_("&Quit", nullptr));
#if QT_CONFIG(shortcut)
        fileActionQuit->setShortcut(q_("Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionUndo->setText(q_("&Undo", nullptr));
#if QT_CONFIG(shortcut)
        editActionUndo->setShortcut(q_("Ctrl+Z", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionRedo->setText(q_("&Redo", nullptr));
#if QT_CONFIG(shortcut)
        editActionRedo->setShortcut(q_("Ctrl+Shift+Z", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(shortcut)
        editActionRedo_2->setShortcut(q_("Ctrl+Y", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionCut->setText(q_("Cu&t", nullptr));
#if QT_CONFIG(shortcut)
        editActionCut->setShortcut(q_("Ctrl+X", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionCopy->setText(q_("&Copy", nullptr));
#if QT_CONFIG(shortcut)
        editActionCopy->setShortcut(q_("Ctrl+C", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionPaste->setText(q_("&Paste", nullptr));
#if QT_CONFIG(shortcut)
        editActionPaste->setShortcut(q_("Ctrl+V", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionIndent->setText(q_("&Indent", nullptr));
#if QT_CONFIG(shortcut)
        editActionIndent->setShortcut(q_("Ctrl+I", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionComment->setText(q_("C&omment", nullptr));
#if QT_CONFIG(shortcut)
        editActionComment->setShortcut(q_("Ctrl+D", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionUncomment->setText(q_("Unco&mment", nullptr));
#if QT_CONFIG(shortcut)
        editActionUncomment->setShortcut(q_("Ctrl+Shift+D", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionNextTab->setText(q_("Show Next Tab", nullptr));
#if QT_CONFIG(shortcut)
        editActionNextTab->setShortcut(q_("Ctrl+Tab", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionPrevTab->setText(q_("Show Previous Tab", nullptr));
#if QT_CONFIG(shortcut)
        editActionPrevTab->setShortcut(q_("Ctrl+Shift+Tab", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionCopyViewport->setText(q_("Copy viewport ima&ge", nullptr));
#if QT_CONFIG(shortcut)
        editActionCopyViewport->setShortcut(q_("Ctrl+Shift+C", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionCopyVPT->setText(q_("Copy viewport transl&ation", nullptr));
#if QT_CONFIG(shortcut)
        editActionCopyVPT->setShortcut(q_("Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionCopyVPR->setText(q_("Cop&y viewport rotation", nullptr));
        editActionCopyVPD->setText(q_("Copy vie&wport distance", nullptr));
        editActionCopyVPF->setText(q_("Copy vie&wport fov", nullptr));
        editActionZoomTextIn->setText(q_("Increase Font &Size", nullptr));
#if QT_CONFIG(shortcut)
        editActionZoomTextIn->setShortcut(q_("Ctrl++", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionZoomTextOut->setText(q_("Decrease Font Si&ze", nullptr));
#if QT_CONFIG(shortcut)
        editActionZoomTextOut->setShortcut(q_("Ctrl+-", nullptr));
#endif // QT_CONFIG(shortcut)
        windowActionHideEditor->setText(q_("Hide Editor", nullptr));
        designActionReloadAndPreview->setText(q_("&Reload and Preview", nullptr));
#if QT_CONFIG(shortcut)
        designActionReloadAndPreview->setShortcut(q_("F4", nullptr));
#endif // QT_CONFIG(shortcut)
        designActionPreview->setText(q_("&Preview", nullptr));
#if QT_CONFIG(shortcut)
        designActionPreview->setShortcut(q_("F5", nullptr));
#endif // QT_CONFIG(shortcut)
        designActionRender->setText(q_("R&ender", nullptr));
#if QT_CONFIG(shortcut)
        designActionRender->setShortcut(q_("F6", nullptr));
#endif // QT_CONFIG(shortcut)
        designAction3DPrint->setText(q_("&3D Print", nullptr));
#if QT_CONFIG(shortcut)
        designAction3DPrint->setShortcut(q_("F8", nullptr));
#endif // QT_CONFIG(shortcut)
        designCheckValidity->setText(q_("&Check Validity", nullptr));
        designActionDisplayAST->setText(q_("Display A&ST...", nullptr));
        designActionDisplayCSGTree->setText(q_("Display CSG &Tree...", nullptr));
        designActionDisplayCSGProducts->setText(q_("Display CSG Pr&oducts...", nullptr));
        fileActionExportSTL->setText(q_("Export as &STL...", nullptr));
#if QT_CONFIG(shortcut)
        fileActionExportSTL->setShortcut(q_("F7", nullptr));
#endif // QT_CONFIG(shortcut)
        fileActionExportOFF->setText(q_("Export as &OFF...", nullptr));
        viewActionPreview->setText(q_("Preview", nullptr));
#if QT_CONFIG(shortcut)
        viewActionPreview->setShortcut(q_("F9", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionSurfaces->setText(q_("Surfaces", nullptr));
#if QT_CONFIG(shortcut)
        viewActionSurfaces->setShortcut(q_("F10", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionWireframe->setText(q_("Wireframe", nullptr));
#if QT_CONFIG(shortcut)
        viewActionWireframe->setShortcut(q_("F11", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionThrownTogether->setText(q_("Thrown Together", nullptr));
#if QT_CONFIG(shortcut)
        viewActionThrownTogether->setShortcut(q_("F12", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionShowEdges->setText(q_("Show Edges", nullptr));
#if QT_CONFIG(shortcut)
        viewActionShowEdges->setShortcut(q_("Ctrl+1", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionShowAxes->setText(q_("Show Axes", nullptr));
#if QT_CONFIG(shortcut)
        viewActionShowAxes->setShortcut(q_("Ctrl+2", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionShowCrosshairs->setText(q_("Show Crosshairs", nullptr));
#if QT_CONFIG(shortcut)
        viewActionShowCrosshairs->setShortcut(q_("Ctrl+3", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionShowScaleProportional->setText(q_("Show Scale Markers", nullptr));
        viewActionAnimate->setText(q_("Animate", nullptr));
        viewActionTop->setText(q_("&Top", nullptr));
#if QT_CONFIG(shortcut)
        viewActionTop->setShortcut(q_("Ctrl+4", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionBottom->setText(q_("&Bottom", nullptr));
#if QT_CONFIG(shortcut)
        viewActionBottom->setShortcut(q_("Ctrl+5", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionLeft->setText(q_("&Left", nullptr));
#if QT_CONFIG(shortcut)
        viewActionLeft->setShortcut(q_("Ctrl+6", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionRight->setText(q_("&Right", nullptr));
#if QT_CONFIG(shortcut)
        viewActionRight->setShortcut(q_("Ctrl+7", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionFront->setText(q_("&Front", nullptr));
#if QT_CONFIG(shortcut)
        viewActionFront->setShortcut(q_("Ctrl+8", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionBack->setText(q_("Bac&k", nullptr));
#if QT_CONFIG(shortcut)
        viewActionBack->setShortcut(q_("Ctrl+9", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionDiagonal->setText(q_("&Diagonal", nullptr));
#if QT_CONFIG(shortcut)
        viewActionDiagonal->setShortcut(q_("Ctrl+0", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionCenter->setText(q_("Ce&nter", nullptr));
#if QT_CONFIG(shortcut)
        viewActionCenter->setShortcut(q_("Ctrl+Shift+0", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionPerspective->setText(q_("&Perspective", nullptr));
        viewActionOrthogonal->setText(q_("&Orthogonal", nullptr));
        windowActionHideConsole->setText(q_("Hide Console", nullptr));
        helpActionAbout->setText(q_("&About", nullptr));
        helpActionManual->setText(q_("&Documentation", nullptr));
        fileActionClearRecent->setText(q_("Clear Recent", nullptr));
        fileActionExportDXF->setText(q_("Export as &DXF...", nullptr));
        fileActionClose->setText(q_("&Close", nullptr));
#if QT_CONFIG(shortcut)
        fileActionClose->setShortcut(q_("Ctrl+W", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionPreferences->setText(q_("&Preferences", nullptr));
        editActionFind->setText(q_("&Find...", nullptr));
#if QT_CONFIG(shortcut)
        editActionFind->setShortcut(q_("Ctrl+F", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionFindAndReplace->setText(q_("Fin&d and Replace...", nullptr));
#if QT_CONFIG(shortcut)
        editActionFindAndReplace->setShortcut(q_("Ctrl+Alt+F", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionFindNext->setText(q_("Find Ne&xt", nullptr));
#if QT_CONFIG(shortcut)
        editActionFindNext->setShortcut(q_("Ctrl+G", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionFindPrevious->setText(q_("Find Pre&vious", nullptr));
#if QT_CONFIG(shortcut)
        editActionFindPrevious->setShortcut(q_("Ctrl+Shift+G", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionUseSelectionForFind->setText(q_("Use Se&lection for Find", nullptr));
#if QT_CONFIG(shortcut)
        editActionUseSelectionForFind->setShortcut(q_("Ctrl+E", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionJumpToNextError->setText(q_("Jump to next error", nullptr));
#if QT_CONFIG(shortcut)
        editActionJumpToNextError->setShortcut(q_("Ctrl+Alt+E", nullptr));
#endif // QT_CONFIG(shortcut)
        designActionFlushCaches->setText(q_("&Flush Caches", nullptr));
        helpActionHomepage->setText(q_("&OpenSCAD Homepage", nullptr));
        designActionAutoReload->setText(q_("&Automatic Reload and Preview", nullptr));
        fileActionExportImage->setText(q_("Export as &Image...", nullptr));
        fileActionExportCSG->setText(q_("Export as &CSG...", nullptr));
        helpActionLibraryInfo->setText(q_("&Library info", nullptr));
        fileShowLibraryFolder->setText(q_("Show &Library Folder...", nullptr));
        viewActionResetView->setText(q_("Reset View", nullptr));
        helpActionFontInfo->setText(q_("&Font List", nullptr));
        fileActionExportSVG->setText(q_("Export as S&VG...", nullptr));
        fileActionExportAMF->setText(q_("Export as &AMF...", nullptr));
        fileActionExport3MF->setText(q_("Export as &3MF...", nullptr));
        viewActionZoomIn->setText(q_("Zoom In", nullptr));
#if QT_CONFIG(shortcut)
        viewActionZoomIn->setShortcut(q_("Ctrl+]", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionZoomOut->setText(q_("Zoom Out", nullptr));
#if QT_CONFIG(shortcut)
        viewActionZoomOut->setShortcut(q_("Ctrl+[", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionViewAll->setText(q_("View All", nullptr));
#if QT_CONFIG(shortcut)
        viewActionViewAll->setShortcut(q_("Ctrl+Shift+V", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionConvertTabsToSpaces->setText(q_("Conv&ert Tabs to Spaces", nullptr));
        editActionToggleBookmark->setText(q_("Toggle Bookmark", nullptr));
#if QT_CONFIG(shortcut)
        editActionToggleBookmark->setShortcut(q_("Ctrl+F2", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionNextBookmark->setText(q_("Jump to next bookmark", nullptr));
#if QT_CONFIG(shortcut)
        editActionNextBookmark->setShortcut(q_("F2", nullptr));
#endif // QT_CONFIG(shortcut)
        editActionPrevBookmark->setText(q_("Jump to previous bookmark", nullptr));
#if QT_CONFIG(shortcut)
        editActionPrevBookmark->setShortcut(q_("Shift+F2", nullptr));
#endif // QT_CONFIG(shortcut)
        viewActionHideEditorToolBar->setText(q_("Hide Editor toolbar", nullptr));
        editActionUnindent->setText(q_("U&nindent", nullptr));
#if QT_CONFIG(shortcut)
        editActionUnindent->setShortcut(q_("Ctrl+Shift+I", nullptr));
#endif // QT_CONFIG(shortcut)
        helpActionCheatSheet->setText(q_("&Cheat Sheet", nullptr));
        windowActionHideCustomizer->setText(q_("Hide Customizer", nullptr));
        fileActionExportPDF->setText(q_("Export as PDF...", nullptr));
        viewActionHide3DViewToolBar->setText(q_("Hide 3D View toolbar", nullptr));
        windowActionHideErrorLog->setText(q_("Hide Error Log", nullptr));
#if QT_CONFIG(tooltip)
        windowActionHideErrorLog->setToolTip(q_("Hide ErrorLog", nullptr));
#endif // QT_CONFIG(tooltip)
        windowActionSelectEditor->setText(q_("&Editor", nullptr));
        windowActionSelectConsole->setText(q_("&Console", nullptr));
        windowActionSelectCustomizer->setText(q_("C&ustomizer", nullptr));
        windowActionSelectErrorLog->setText(q_("Error &Log", nullptr));
        windowActionNextWindow->setText(q_("&Next Window", nullptr));
        windowActionPreviousWindow->setText(q_("&Previous Window", nullptr));
        editActionInsertTemplate->setText(q_("Insert Template", nullptr));
#if QT_CONFIG(shortcut)
        editActionInsertTemplate->setShortcut(q_("Alt+Ins", nullptr));
#endif // QT_CONFIG(shortcut)
        labelCompileResultMessage->setText(q_("Message", nullptr));
        toolButtonCompileResultClose->setText(QString());
        toolButtonCompileResultIcon->setText(QString());
        label_time->setText(q_("Time:", nullptr));
        label_fps->setText(q_("FPS:", nullptr));
        label_steps->setText(q_("Steps:", nullptr));
        e_dump->setText(q_("Dump Pictures", nullptr));
        menu_File->setTitle(q_("&File", nullptr));
        menuOpenRecent->setTitle(q_("Recen&t Files", nullptr));
        menuExamples->setTitle(q_("&Examples", nullptr));
        menuExport->setTitle(q_("E&xport", nullptr));
        menu_Edit->setTitle(q_("&Edit", nullptr));
        menu_Design->setTitle(q_("&Design", nullptr));
        menu_View->setTitle(q_("&View", nullptr));
        menuHelp->setTitle(q_("&Help", nullptr));
        menuWindow->setTitle(q_("&Window", nullptr));
        tabToolBar->setWindowTitle(q_("Tabs", nullptr));
        findTypeComboBox->setItemText(0, q_("Find", nullptr));
        findTypeComboBox->setItemText(1, q_("Replace", nullptr));

        findInputField->setProperty("placeholderText", QVariant(q_("Search string", nullptr)));
        findPrevButton->setText(q_("Previous", nullptr));
        findNextButton->setText(q_("Next", nullptr));
        cancelButton->setText(q_("Done", nullptr));
        replaceInputField->setPlaceholderText(q_("Replacement string", nullptr));
        replaceButton->setText(q_("Replace", nullptr));
        replaceAllButton->setText(q_("All", nullptr));
        parameterDock->setWindowTitle(q_("Customizer Widget", nullptr));
        (void)MainWindow;
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
