/****************************************************************************
** Meta object code from reading C++ file 'ParameterWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/parameter/ParameterWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ParameterWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ParameterWidget_t {
    QByteArrayData data[15];
    char stringdata0[217];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ParameterWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ParameterWidget_t qt_meta_stringdata_ParameterWidget = {
    {
QT_MOC_LITERAL(0, 0, 15), // "ParameterWidget"
QT_MOC_LITERAL(1, 16, 16), // "previewRequested"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 18), // "rebuildParameterUI"
QT_MOC_LITERAL(4, 53, 14), // "onValueChanged"
QT_MOC_LITERAL(5, 68, 21), // "onPreviewTimerElapsed"
QT_MOC_LITERAL(6, 90, 23), // "onDescriptionLoDChanged"
QT_MOC_LITERAL(7, 114, 12), // "onSetChanged"
QT_MOC_LITERAL(8, 127, 3), // "idx"
QT_MOC_LITERAL(9, 131, 16), // "onSetNameChanged"
QT_MOC_LITERAL(10, 148, 8), // "onSetAdd"
QT_MOC_LITERAL(11, 157, 15), // "onSetSaveButton"
QT_MOC_LITERAL(12, 173, 11), // "onSetDelete"
QT_MOC_LITERAL(13, 185, 14), // "resetParameter"
QT_MOC_LITERAL(14, 200, 16) // "defaultParameter"

    },
    "ParameterWidget\0previewRequested\0\0"
    "rebuildParameterUI\0onValueChanged\0"
    "onPreviewTimerElapsed\0onDescriptionLoDChanged\0"
    "onSetChanged\0idx\0onSetNameChanged\0"
    "onSetAdd\0onSetSaveButton\0onSetDelete\0"
    "resetParameter\0defaultParameter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ParameterWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       1,    0,   77,    2, 0x26 /* Public | MethodCloned */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   78,    2, 0x09 /* Protected */,
       5,    0,   79,    2, 0x09 /* Protected */,
       6,    0,   80,    2, 0x09 /* Protected */,
       7,    1,   81,    2, 0x09 /* Protected */,
       9,    0,   84,    2, 0x09 /* Protected */,
      10,    0,   85,    2, 0x09 /* Protected */,
      11,    0,   86,    2, 0x09 /* Protected */,
      12,    0,   87,    2, 0x09 /* Protected */,
      13,    0,   88,    2, 0x09 /* Protected */,
      14,    0,   89,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ParameterWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ParameterWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->previewRequested((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->previewRequested(); break;
        case 2: _t->onValueChanged(); break;
        case 3: _t->onPreviewTimerElapsed(); break;
        case 4: _t->onDescriptionLoDChanged(); break;
        case 5: _t->onSetChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->onSetNameChanged(); break;
        case 7: _t->onSetAdd(); break;
        case 8: _t->onSetSaveButton(); break;
        case 9: _t->onSetDelete(); break;
        case 10: _t->resetParameter(); break;
        case 11: _t->defaultParameter(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ParameterWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParameterWidget::previewRequested)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ParameterWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ParameterWidget.data,
    qt_meta_data_ParameterWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ParameterWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ParameterWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ParameterWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ParameterWidget"))
        return static_cast< Ui::ParameterWidget*>(this);
    return QWidget::qt_metacast(_clname);
}

int ParameterWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ParameterWidget::previewRequested(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
