/****************************************************************************
** Meta object code from reading C++ file 'QGLView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/QGLView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QGLView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QGLView_t {
    QByteArrayData data[17];
    char stringdata0[213];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QGLView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QGLView_t qt_meta_stringdata_QGLView = {
    {
QT_MOC_LITERAL(0, 0, 7), // "QGLView"
QT_MOC_LITERAL(1, 8, 15), // "doAnimateUpdate"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 14), // "doSelectObject"
QT_MOC_LITERAL(4, 40, 17), // "screen_coordinate"
QT_MOC_LITERAL(5, 58, 6), // "ZoomIn"
QT_MOC_LITERAL(6, 65, 7), // "ZoomOut"
QT_MOC_LITERAL(7, 73, 8), // "updateGL"
QT_MOC_LITERAL(8, 82, 19), // "setMouseCentricZoom"
QT_MOC_LITERAL(9, 102, 3), // "var"
QT_MOC_LITERAL(10, 106, 30), // "display_opencsg_warning_dialog"
QT_MOC_LITERAL(11, 137, 9), // "showFaces"
QT_MOC_LITERAL(12, 147, 9), // "showEdges"
QT_MOC_LITERAL(13, 157, 8), // "showAxes"
QT_MOC_LITERAL(14, 166, 14), // "showCrosshairs"
QT_MOC_LITERAL(15, 181, 9), // "orthoMode"
QT_MOC_LITERAL(16, 191, 21) // "showScaleProportional"

    },
    "QGLView\0doAnimateUpdate\0\0doSelectObject\0"
    "screen_coordinate\0ZoomIn\0ZoomOut\0"
    "updateGL\0setMouseCentricZoom\0var\0"
    "display_opencsg_warning_dialog\0showFaces\0"
    "showEdges\0showAxes\0showCrosshairs\0"
    "orthoMode\0showScaleProportional"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QGLView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       6,   60, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    1,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   53,    2, 0x0a /* Public */,
       6,    0,   54,    2, 0x0a /* Public */,
       7,    0,   55,    2, 0x0a /* Public */,
       8,    1,   56,    2, 0x0a /* Public */,
      10,    0,   59,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,

 // properties: name, type, flags
      11, QMetaType::Bool, 0x00095103,
      12, QMetaType::Bool, 0x00095103,
      13, QMetaType::Bool, 0x00095103,
      14, QMetaType::Bool, 0x00095103,
      15, QMetaType::Bool, 0x00095103,
      16, QMetaType::Double, 0x00095103,

       0        // eod
};

void QGLView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QGLView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->doAnimateUpdate(); break;
        case 1: _t->doSelectObject((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 2: _t->ZoomIn(); break;
        case 3: _t->ZoomOut(); break;
        case 4: _t->updateGL(); break;
        case 5: _t->setMouseCentricZoom((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->display_opencsg_warning_dialog(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QGLView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGLView::doAnimateUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QGLView::*)(QPoint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QGLView::doSelectObject)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QGLView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->showFaces(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->showEdges(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->showAxes(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->showCrosshairs(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->orthoMode(); break;
        case 5: *reinterpret_cast< double*>(_v) = _t->showScaleProportional(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QGLView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setShowFaces(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setShowEdges(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setShowAxes(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setShowCrosshairs(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setOrthoMode(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setShowScaleProportional(*reinterpret_cast< double*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QGLView::staticMetaObject = { {
    QMetaObject::SuperData::link<QOpenGLWidget::staticMetaObject>(),
    qt_meta_stringdata_QGLView.data,
    qt_meta_data_QGLView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QGLView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGLView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QGLView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "GLView"))
        return static_cast< GLView*>(this);
    return QOpenGLWidget::qt_metacast(_clname);
}

int QGLView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QOpenGLWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QGLView::doAnimateUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QGLView::doSelectObject(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
