/****************************************************************************
** Meta object code from reading C++ file 'editor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/editor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_EditorInterface_t {
    QByteArrayData data[32];
    char stringdata0[408];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EditorInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EditorInterface_t qt_meta_stringdata_EditorInterface = {
    {
QT_MOC_LITERAL(0, 0, 15), // "EditorInterface"
QT_MOC_LITERAL(1, 16, 15), // "contentsChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 19), // "modificationChanged"
QT_MOC_LITERAL(4, 53, 16), // "EditorInterface*"
QT_MOC_LITERAL(5, 70, 20), // "showContextMenuEvent"
QT_MOC_LITERAL(6, 91, 3), // "pos"
QT_MOC_LITERAL(7, 95, 6), // "zoomIn"
QT_MOC_LITERAL(8, 102, 7), // "zoomOut"
QT_MOC_LITERAL(9, 110, 18), // "setContentModified"
QT_MOC_LITERAL(10, 129, 17), // "isContentModified"
QT_MOC_LITERAL(11, 147, 15), // "indentSelection"
QT_MOC_LITERAL(12, 163, 17), // "unindentSelection"
QT_MOC_LITERAL(13, 181, 16), // "commentSelection"
QT_MOC_LITERAL(14, 198, 18), // "uncommentSelection"
QT_MOC_LITERAL(15, 217, 12), // "setPlainText"
QT_MOC_LITERAL(16, 230, 14), // "highlightError"
QT_MOC_LITERAL(17, 245, 20), // "unhighlightLastError"
QT_MOC_LITERAL(18, 266, 18), // "setHighlightScheme"
QT_MOC_LITERAL(19, 285, 6), // "insert"
QT_MOC_LITERAL(20, 292, 7), // "setText"
QT_MOC_LITERAL(21, 300, 4), // "undo"
QT_MOC_LITERAL(22, 305, 4), // "redo"
QT_MOC_LITERAL(23, 310, 3), // "cut"
QT_MOC_LITERAL(24, 314, 4), // "copy"
QT_MOC_LITERAL(25, 319, 5), // "paste"
QT_MOC_LITERAL(26, 325, 8), // "initFont"
QT_MOC_LITERAL(27, 334, 16), // "displayTemplates"
QT_MOC_LITERAL(28, 351, 14), // "toggleBookmark"
QT_MOC_LITERAL(29, 366, 12), // "nextBookmark"
QT_MOC_LITERAL(30, 379, 12), // "prevBookmark"
QT_MOC_LITERAL(31, 392, 15) // "jumpToNextError"

    },
    "EditorInterface\0contentsChanged\0\0"
    "modificationChanged\0EditorInterface*\0"
    "showContextMenuEvent\0pos\0zoomIn\0zoomOut\0"
    "setContentModified\0isContentModified\0"
    "indentSelection\0unindentSelection\0"
    "commentSelection\0uncommentSelection\0"
    "setPlainText\0highlightError\0"
    "unhighlightLastError\0setHighlightScheme\0"
    "insert\0setText\0undo\0redo\0cut\0copy\0"
    "paste\0initFont\0displayTemplates\0"
    "toggleBookmark\0nextBookmark\0prevBookmark\0"
    "jumpToNextError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EditorInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      28,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  154,    2, 0x06 /* Public */,
       3,    2,  155,    2, 0x06 /* Public */,
       5,    1,  160,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  163,    2, 0x0a /* Public */,
       8,    0,  164,    2, 0x0a /* Public */,
       9,    1,  165,    2, 0x0a /* Public */,
      10,    0,  168,    2, 0x0a /* Public */,
      11,    0,  169,    2, 0x0a /* Public */,
      12,    0,  170,    2, 0x0a /* Public */,
      13,    0,  171,    2, 0x0a /* Public */,
      14,    0,  172,    2, 0x0a /* Public */,
      15,    1,  173,    2, 0x0a /* Public */,
      16,    1,  176,    2, 0x0a /* Public */,
      17,    0,  179,    2, 0x0a /* Public */,
      18,    1,  180,    2, 0x0a /* Public */,
      19,    1,  183,    2, 0x0a /* Public */,
      20,    1,  186,    2, 0x0a /* Public */,
      21,    0,  189,    2, 0x0a /* Public */,
      22,    0,  190,    2, 0x0a /* Public */,
      23,    0,  191,    2, 0x0a /* Public */,
      24,    0,  192,    2, 0x0a /* Public */,
      25,    0,  193,    2, 0x0a /* Public */,
      26,    2,  194,    2, 0x0a /* Public */,
      27,    0,  199,    2, 0x0a /* Public */,
      28,    0,  200,    2, 0x0a /* Public */,
      29,    0,  201,    2, 0x0a /* Public */,
      30,    0,  202,    2, 0x0a /* Public */,
      31,    0,  203,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QPoint,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::UInt,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void EditorInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditorInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->contentsChanged(); break;
        case 1: _t->modificationChanged((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< EditorInterface*(*)>(_a[2]))); break;
        case 2: _t->showContextMenuEvent((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 3: _t->zoomIn(); break;
        case 4: _t->zoomOut(); break;
        case 5: _t->setContentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: { bool _r = _t->isContentModified();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->indentSelection(); break;
        case 8: _t->unindentSelection(); break;
        case 9: _t->commentSelection(); break;
        case 10: _t->uncommentSelection(); break;
        case 11: _t->setPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->highlightError((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->unhighlightLastError(); break;
        case 14: _t->setHighlightScheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->insert((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->setText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->undo(); break;
        case 18: _t->redo(); break;
        case 19: _t->cut(); break;
        case 20: _t->copy(); break;
        case 21: _t->paste(); break;
        case 22: _t->initFont((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 23: _t->displayTemplates(); break;
        case 24: _t->toggleBookmark(); break;
        case 25: _t->nextBookmark(); break;
        case 26: _t->prevBookmark(); break;
        case 27: _t->jumpToNextError(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< EditorInterface* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditorInterface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditorInterface::contentsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (EditorInterface::*)(bool , EditorInterface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditorInterface::modificationChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (EditorInterface::*)(const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditorInterface::showContextMenuEvent)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject EditorInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_EditorInterface.data,
    qt_meta_data_EditorInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EditorInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditorInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EditorInterface.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int EditorInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void EditorInterface::contentsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void EditorInterface::modificationChanged(bool _t1, EditorInterface * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void EditorInterface::showContextMenuEvent(const QPoint & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
