/********************************************************************************
** Form generated from reading UI file 'ParameterEntryWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PARAMETERENTRYWIDGET_H
#define UI_PARAMETERENTRYWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ParameterEntryWidget
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *stackedWidgetBelow;
    QWidget *pageSlider;
    QGridLayout *gridLayout_3;
    QSlider *slider;
    QWidget *pageComboBox;
    QGridLayout *gridLayout_4;
    QComboBox *comboBox;
    QWidget *pageText;
    QGridLayout *gridLayout_5;
    QLineEdit *lineEdit;
    QWidget *pageVector;
    QGridLayout *gridLayout_2;
    QDoubleSpinBox *doubleSpinBox2;
    QDoubleSpinBox *doubleSpinBox1;
    QDoubleSpinBox *doubleSpinBox3;
    QSpacerItem *horizontalSpacer;
    QDoubleSpinBox *doubleSpinBox4;
    QHBoxLayout *horizontalLayout;
    QFormLayout *formLayout;
    QLabel *labelParameter;
    QLabel *labelDescription;
    QLabel *labelInline;
    QStackedWidget *stackedWidgetRight;
    QWidget *pageCheckBox;
    QGridLayout *gridLayout1;
    QCheckBox *checkBox;
    QWidget *pageSpin;
    QGridLayout *gridLayout2;
    QDoubleSpinBox *doubleSpinBox;

    void setupUi(QWidget *ParameterEntryWidget)
    {
        if (ParameterEntryWidget->objectName().isEmpty())
            ParameterEntryWidget->setObjectName(QString::fromUtf8("ParameterEntryWidget"));
        ParameterEntryWidget->resize(308, 54);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ParameterEntryWidget->sizePolicy().hasHeightForWidth());
        ParameterEntryWidget->setSizePolicy(sizePolicy);
        ParameterEntryWidget->setMinimumSize(QSize(0, 0));
        gridLayout = new QGridLayout(ParameterEntryWidget);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(4, 0, 4, 0);
        stackedWidgetBelow = new QStackedWidget(ParameterEntryWidget);
        stackedWidgetBelow->setObjectName(QString::fromUtf8("stackedWidgetBelow"));
        stackedWidgetBelow->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(stackedWidgetBelow->sizePolicy().hasHeightForWidth());
        stackedWidgetBelow->setSizePolicy(sizePolicy1);
        stackedWidgetBelow->setLineWidth(0);
        pageSlider = new QWidget();
        pageSlider->setObjectName(QString::fromUtf8("pageSlider"));
        sizePolicy.setHeightForWidth(pageSlider->sizePolicy().hasHeightForWidth());
        pageSlider->setSizePolicy(sizePolicy);
        gridLayout_3 = new QGridLayout(pageSlider);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        slider = new QSlider(pageSlider);
        slider->setObjectName(QString::fromUtf8("slider"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Maximum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(slider->sizePolicy().hasHeightForWidth());
        slider->setSizePolicy(sizePolicy2);
        slider->setOrientation(Qt::Horizontal);

        gridLayout_3->addWidget(slider, 0, 0, 1, 1);

        stackedWidgetBelow->addWidget(pageSlider);
        pageComboBox = new QWidget();
        pageComboBox->setObjectName(QString::fromUtf8("pageComboBox"));
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::Ignored);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(pageComboBox->sizePolicy().hasHeightForWidth());
        pageComboBox->setSizePolicy(sizePolicy3);
        gridLayout_4 = new QGridLayout(pageComboBox);
        gridLayout_4->setSpacing(0);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        comboBox = new QComboBox(pageComboBox);
        comboBox->setObjectName(QString::fromUtf8("comboBox"));
        QSizePolicy sizePolicy4(QSizePolicy::Minimum, QSizePolicy::Maximum);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(comboBox->sizePolicy().hasHeightForWidth());
        comboBox->setSizePolicy(sizePolicy4);
        comboBox->setFocusPolicy(Qt::StrongFocus);

        gridLayout_4->addWidget(comboBox, 0, 0, 1, 1);

        stackedWidgetBelow->addWidget(pageComboBox);
        pageText = new QWidget();
        pageText->setObjectName(QString::fromUtf8("pageText"));
        QSizePolicy sizePolicy5(QSizePolicy::Preferred, QSizePolicy::Ignored);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(pageText->sizePolicy().hasHeightForWidth());
        pageText->setSizePolicy(sizePolicy5);
        gridLayout_5 = new QGridLayout(pageText);
        gridLayout_5->setSpacing(0);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        lineEdit = new QLineEdit(pageText);
        lineEdit->setObjectName(QString::fromUtf8("lineEdit"));
        lineEdit->setEnabled(true);
        sizePolicy2.setHeightForWidth(lineEdit->sizePolicy().hasHeightForWidth());
        lineEdit->setSizePolicy(sizePolicy2);
        lineEdit->setAutoFillBackground(true);
        lineEdit->setFrame(true);

        gridLayout_5->addWidget(lineEdit, 1, 0, 1, 1);

        stackedWidgetBelow->addWidget(pageText);
        pageVector = new QWidget();
        pageVector->setObjectName(QString::fromUtf8("pageVector"));
        sizePolicy5.setHeightForWidth(pageVector->sizePolicy().hasHeightForWidth());
        pageVector->setSizePolicy(sizePolicy5);
        gridLayout_2 = new QGridLayout(pageVector);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 0, 4);
        doubleSpinBox2 = new QDoubleSpinBox(pageVector);
        doubleSpinBox2->setObjectName(QString::fromUtf8("doubleSpinBox2"));
        sizePolicy4.setHeightForWidth(doubleSpinBox2->sizePolicy().hasHeightForWidth());
        doubleSpinBox2->setSizePolicy(sizePolicy4);
        doubleSpinBox2->setFocusPolicy(Qt::StrongFocus);
        doubleSpinBox2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox2->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBox2->setDecimals(0);
        doubleSpinBox2->setMaximum(1000.000000000000000);

        gridLayout_2->addWidget(doubleSpinBox2, 0, 1, 1, 1);

        doubleSpinBox1 = new QDoubleSpinBox(pageVector);
        doubleSpinBox1->setObjectName(QString::fromUtf8("doubleSpinBox1"));
        sizePolicy4.setHeightForWidth(doubleSpinBox1->sizePolicy().hasHeightForWidth());
        doubleSpinBox1->setSizePolicy(sizePolicy4);
        doubleSpinBox1->setFocusPolicy(Qt::StrongFocus);
        doubleSpinBox1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox1->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBox1->setDecimals(0);
        doubleSpinBox1->setMaximum(1000.000000000000000);

        gridLayout_2->addWidget(doubleSpinBox1, 0, 0, 1, 1);

        doubleSpinBox3 = new QDoubleSpinBox(pageVector);
        doubleSpinBox3->setObjectName(QString::fromUtf8("doubleSpinBox3"));
        sizePolicy4.setHeightForWidth(doubleSpinBox3->sizePolicy().hasHeightForWidth());
        doubleSpinBox3->setSizePolicy(sizePolicy4);
        doubleSpinBox3->setFocusPolicy(Qt::StrongFocus);
        doubleSpinBox3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox3->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBox3->setDecimals(0);
        doubleSpinBox3->setMaximum(1000.000000000000000);

        gridLayout_2->addWidget(doubleSpinBox3, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 5, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 4, 1, 1);

        doubleSpinBox4 = new QDoubleSpinBox(pageVector);
        doubleSpinBox4->setObjectName(QString::fromUtf8("doubleSpinBox4"));
        sizePolicy4.setHeightForWidth(doubleSpinBox4->sizePolicy().hasHeightForWidth());
        doubleSpinBox4->setSizePolicy(sizePolicy4);
        doubleSpinBox4->setFocusPolicy(Qt::StrongFocus);
        doubleSpinBox4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox4->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBox4->setDecimals(0);
        doubleSpinBox4->setMaximum(1000.000000000000000);

        gridLayout_2->addWidget(doubleSpinBox4, 0, 3, 1, 1);

        stackedWidgetBelow->addWidget(pageVector);

        gridLayout->addWidget(stackedWidgetBelow, 2, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setHorizontalSpacing(0);
        formLayout->setVerticalSpacing(0);
        formLayout->setContentsMargins(-1, 0, -1, 0);
        labelParameter = new QLabel(ParameterEntryWidget);
        labelParameter->setObjectName(QString::fromUtf8("labelParameter"));
        QSizePolicy sizePolicy6(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(labelParameter->sizePolicy().hasHeightForWidth());
        labelParameter->setSizePolicy(sizePolicy6);
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        labelParameter->setFont(font);
        labelParameter->setLineWidth(0);
        labelParameter->setTextFormat(Qt::PlainText);
        labelParameter->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::LabelRole, labelParameter);

        labelDescription = new QLabel(ParameterEntryWidget);
        labelDescription->setObjectName(QString::fromUtf8("labelDescription"));
        QSizePolicy sizePolicy7(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(labelDescription->sizePolicy().hasHeightForWidth());
        labelDescription->setSizePolicy(sizePolicy7);
        labelDescription->setLineWidth(0);
        labelDescription->setTextFormat(Qt::PlainText);
        labelDescription->setWordWrap(true);

        formLayout->setWidget(1, QFormLayout::SpanningRole, labelDescription);

        labelInline = new QLabel(ParameterEntryWidget);
        labelInline->setObjectName(QString::fromUtf8("labelInline"));
        QSizePolicy sizePolicy8(QSizePolicy::Ignored, QSizePolicy::Preferred);
        sizePolicy8.setHorizontalStretch(0);
        sizePolicy8.setVerticalStretch(0);
        sizePolicy8.setHeightForWidth(labelInline->sizePolicy().hasHeightForWidth());
        labelInline->setSizePolicy(sizePolicy8);
        labelInline->setLineWidth(0);
        labelInline->setTextFormat(Qt::PlainText);

        formLayout->setWidget(0, QFormLayout::FieldRole, labelInline);


        horizontalLayout->addLayout(formLayout);

        stackedWidgetRight = new QStackedWidget(ParameterEntryWidget);
        stackedWidgetRight->setObjectName(QString::fromUtf8("stackedWidgetRight"));
        sizePolicy.setHeightForWidth(stackedWidgetRight->sizePolicy().hasHeightForWidth());
        stackedWidgetRight->setSizePolicy(sizePolicy);
        stackedWidgetRight->setLineWidth(0);
        pageCheckBox = new QWidget();
        pageCheckBox->setObjectName(QString::fromUtf8("pageCheckBox"));
        sizePolicy.setHeightForWidth(pageCheckBox->sizePolicy().hasHeightForWidth());
        pageCheckBox->setSizePolicy(sizePolicy);
        gridLayout1 = new QGridLayout(pageCheckBox);
        gridLayout1->setSpacing(0);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        checkBox = new QCheckBox(pageCheckBox);
        checkBox->setObjectName(QString::fromUtf8("checkBox"));
        sizePolicy.setHeightForWidth(checkBox->sizePolicy().hasHeightForWidth());
        checkBox->setSizePolicy(sizePolicy);
        checkBox->setLayoutDirection(Qt::RightToLeft);

        gridLayout1->addWidget(checkBox, 0, 0, 1, 1);

        stackedWidgetRight->addWidget(pageCheckBox);
        pageSpin = new QWidget();
        pageSpin->setObjectName(QString::fromUtf8("pageSpin"));
        QSizePolicy sizePolicy9(QSizePolicy::Ignored, QSizePolicy::Maximum);
        sizePolicy9.setHorizontalStretch(0);
        sizePolicy9.setVerticalStretch(0);
        sizePolicy9.setHeightForWidth(pageSpin->sizePolicy().hasHeightForWidth());
        pageSpin->setSizePolicy(sizePolicy9);
        gridLayout2 = new QGridLayout(pageSpin);
        gridLayout2->setSpacing(0);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        doubleSpinBox = new QDoubleSpinBox(pageSpin);
        doubleSpinBox->setObjectName(QString::fromUtf8("doubleSpinBox"));
        sizePolicy.setHeightForWidth(doubleSpinBox->sizePolicy().hasHeightForWidth());
        doubleSpinBox->setSizePolicy(sizePolicy);
        doubleSpinBox->setFocusPolicy(Qt::StrongFocus);
        doubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBox->setDecimals(0);
        doubleSpinBox->setMaximum(1000.000000000000000);

        gridLayout2->addWidget(doubleSpinBox, 0, 0, 1, 1);

        stackedWidgetRight->addWidget(pageSpin);

        horizontalLayout->addWidget(stackedWidgetRight);


        gridLayout->addLayout(horizontalLayout, 1, 0, 1, 1);

        QWidget::setTabOrder(doubleSpinBox1, doubleSpinBox2);
        QWidget::setTabOrder(doubleSpinBox2, doubleSpinBox3);
        QWidget::setTabOrder(doubleSpinBox3, doubleSpinBox4);
        QWidget::setTabOrder(doubleSpinBox4, slider);
        QWidget::setTabOrder(slider, comboBox);
        QWidget::setTabOrder(comboBox, lineEdit);

        retranslateUi(ParameterEntryWidget);

        stackedWidgetBelow->setCurrentIndex(0);
        stackedWidgetRight->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(ParameterEntryWidget);
    } // setupUi

    void retranslateUi(QWidget *ParameterEntryWidget)
    {
        ParameterEntryWidget->setWindowTitle(q_("Form", nullptr));
        lineEdit->setText(QString());
        labelParameter->setText(q_("Parameter", nullptr));
        labelDescription->setText(q_("Description", nullptr));
        labelInline->setText(q_("Description", nullptr));
        checkBox->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class ParameterEntryWidget: public Ui_ParameterEntryWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PARAMETERENTRYWIDGET_H
