/****************************************************************************
** Meta object code from reading C++ file 'ButtonConfigWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/input/ButtonConfigWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ButtonConfigWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ButtonConfigWidget_t {
    QByteArrayData data[20];
    char stringdata0[514];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ButtonConfigWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ButtonConfigWidget_t qt_meta_stringdata_ButtonConfigWidget = {
    {
QT_MOC_LITERAL(0, 0, 18), // "ButtonConfigWidget"
QT_MOC_LITERAL(1, 19, 19), // "inputMappingChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 28), // "on_comboBoxButton0_activated"
QT_MOC_LITERAL(4, 69, 3), // "val"
QT_MOC_LITERAL(5, 73, 28), // "on_comboBoxButton1_activated"
QT_MOC_LITERAL(6, 102, 28), // "on_comboBoxButton2_activated"
QT_MOC_LITERAL(7, 131, 28), // "on_comboBoxButton3_activated"
QT_MOC_LITERAL(8, 160, 28), // "on_comboBoxButton4_activated"
QT_MOC_LITERAL(9, 189, 28), // "on_comboBoxButton5_activated"
QT_MOC_LITERAL(10, 218, 28), // "on_comboBoxButton6_activated"
QT_MOC_LITERAL(11, 247, 28), // "on_comboBoxButton7_activated"
QT_MOC_LITERAL(12, 276, 28), // "on_comboBoxButton8_activated"
QT_MOC_LITERAL(13, 305, 28), // "on_comboBoxButton9_activated"
QT_MOC_LITERAL(14, 334, 29), // "on_comboBoxButton10_activated"
QT_MOC_LITERAL(15, 364, 29), // "on_comboBoxButton11_activated"
QT_MOC_LITERAL(16, 394, 29), // "on_comboBoxButton12_activated"
QT_MOC_LITERAL(17, 424, 29), // "on_comboBoxButton13_activated"
QT_MOC_LITERAL(18, 454, 29), // "on_comboBoxButton14_activated"
QT_MOC_LITERAL(19, 484, 29) // "on_comboBoxButton15_activated"

    },
    "ButtonConfigWidget\0inputMappingChanged\0"
    "\0on_comboBoxButton0_activated\0val\0"
    "on_comboBoxButton1_activated\0"
    "on_comboBoxButton2_activated\0"
    "on_comboBoxButton3_activated\0"
    "on_comboBoxButton4_activated\0"
    "on_comboBoxButton5_activated\0"
    "on_comboBoxButton6_activated\0"
    "on_comboBoxButton7_activated\0"
    "on_comboBoxButton8_activated\0"
    "on_comboBoxButton9_activated\0"
    "on_comboBoxButton10_activated\0"
    "on_comboBoxButton11_activated\0"
    "on_comboBoxButton12_activated\0"
    "on_comboBoxButton13_activated\0"
    "on_comboBoxButton14_activated\0"
    "on_comboBoxButton15_activated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ButtonConfigWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  100,    2, 0x0a /* Public */,
       5,    1,  103,    2, 0x0a /* Public */,
       6,    1,  106,    2, 0x0a /* Public */,
       7,    1,  109,    2, 0x0a /* Public */,
       8,    1,  112,    2, 0x0a /* Public */,
       9,    1,  115,    2, 0x0a /* Public */,
      10,    1,  118,    2, 0x0a /* Public */,
      11,    1,  121,    2, 0x0a /* Public */,
      12,    1,  124,    2, 0x0a /* Public */,
      13,    1,  127,    2, 0x0a /* Public */,
      14,    1,  130,    2, 0x0a /* Public */,
      15,    1,  133,    2, 0x0a /* Public */,
      16,    1,  136,    2, 0x0a /* Public */,
      17,    1,  139,    2, 0x0a /* Public */,
      18,    1,  142,    2, 0x0a /* Public */,
      19,    1,  145,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,

       0        // eod
};

void ButtonConfigWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ButtonConfigWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->inputMappingChanged(); break;
        case 1: _t->on_comboBoxButton0_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->on_comboBoxButton1_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->on_comboBoxButton2_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->on_comboBoxButton3_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->on_comboBoxButton4_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->on_comboBoxButton5_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->on_comboBoxButton6_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->on_comboBoxButton7_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->on_comboBoxButton8_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->on_comboBoxButton9_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->on_comboBoxButton10_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->on_comboBoxButton11_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->on_comboBoxButton12_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->on_comboBoxButton13_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->on_comboBoxButton14_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->on_comboBoxButton15_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ButtonConfigWidget::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ButtonConfigWidget::inputMappingChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ButtonConfigWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ButtonConfigWidget.data,
    qt_meta_data_ButtonConfigWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ButtonConfigWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ButtonConfigWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ButtonConfigWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Button"))
        return static_cast< Ui::Button*>(this);
    return QWidget::qt_metacast(_clname);
}

int ButtonConfigWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void ButtonConfigWidget::inputMappingChanged()const
{
    QMetaObject::activate(const_cast< ButtonConfigWidget *>(this), &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
