/****************************************************************************
** Meta object code from reading C++ file 'Preferences.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/Preferences.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Preferences.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Preferences_t {
    QByteArrayData data[88];
    char stringdata0[2576];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Preferences_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Preferences_t qt_meta_stringdata_Preferences = {
    {
QT_MOC_LITERAL(0, 0, 11), // "Preferences"
QT_MOC_LITERAL(1, 12, 13), // "requestRedraw"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 16), // "updateUndockMode"
QT_MOC_LITERAL(4, 44, 10), // "undockMode"
QT_MOC_LITERAL(5, 55, 17), // "updateReorderMode"
QT_MOC_LITERAL(6, 73, 11), // "fontChanged"
QT_MOC_LITERAL(7, 85, 6), // "family"
QT_MOC_LITERAL(8, 92, 4), // "size"
QT_MOC_LITERAL(9, 97, 18), // "colorSchemeChanged"
QT_MOC_LITERAL(10, 116, 6), // "scheme"
QT_MOC_LITERAL(11, 123, 22), // "openCSGSettingsChanged"
QT_MOC_LITERAL(12, 146, 22), // "syntaxHighlightChanged"
QT_MOC_LITERAL(13, 169, 1), // "s"
QT_MOC_LITERAL(14, 171, 19), // "editorConfigChanged"
QT_MOC_LITERAL(15, 191, 19), // "ExperimentalChanged"
QT_MOC_LITERAL(16, 211, 22), // "updateMouseCentricZoom"
QT_MOC_LITERAL(17, 234, 5), // "state"
QT_MOC_LITERAL(18, 240, 19), // "autocompleteChanged"
QT_MOC_LITERAL(19, 260, 6), // "status"
QT_MOC_LITERAL(20, 267, 25), // "characterThresholdChanged"
QT_MOC_LITERAL(21, 293, 3), // "val"
QT_MOC_LITERAL(22, 297, 15), // "stepSizeChanged"
QT_MOC_LITERAL(23, 313, 15), // "actionTriggered"
QT_MOC_LITERAL(24, 329, 8), // "QAction*"
QT_MOC_LITERAL(25, 338, 23), // "featuresCheckBoxToggled"
QT_MOC_LITERAL(26, 362, 31), // "on_stackedWidget_currentChanged"
QT_MOC_LITERAL(27, 394, 42), // "on_colorSchemeChooser_itemSel..."
QT_MOC_LITERAL(28, 437, 24), // "on_fontChooser_activated"
QT_MOC_LITERAL(29, 462, 31), // "on_fontSize_currentIndexChanged"
QT_MOC_LITERAL(30, 494, 28), // "on_syntaxHighlight_activated"
QT_MOC_LITERAL(31, 523, 28), // "on_openCSGWarningBox_toggled"
QT_MOC_LITERAL(32, 552, 27), // "on_enableOpenCSGBox_toggled"
QT_MOC_LITERAL(33, 580, 34), // "on_cgalCacheSizeMBEdit_textCh..."
QT_MOC_LITERAL(34, 615, 37), // "on_polysetCacheSizeMBEdit_tex..."
QT_MOC_LITERAL(35, 653, 31), // "on_opencsgLimitEdit_textChanged"
QT_MOC_LITERAL(36, 685, 30), // "on_forceGoldfeatherBox_toggled"
QT_MOC_LITERAL(37, 716, 28), // "on_mouseWheelZoomBox_toggled"
QT_MOC_LITERAL(38, 745, 31), // "on_localizationCheckBox_toggled"
QT_MOC_LITERAL(39, 777, 34), // "on_autoReloadRaiseCheckBox_to..."
QT_MOC_LITERAL(40, 812, 25), // "on_updateCheckBox_toggled"
QT_MOC_LITERAL(41, 838, 27), // "on_snapshotCheckBox_toggled"
QT_MOC_LITERAL(42, 866, 26), // "on_reorderCheckBox_toggled"
QT_MOC_LITERAL(43, 893, 25), // "on_undockCheckBox_toggled"
QT_MOC_LITERAL(44, 919, 25), // "on_checkNowButton_clicked"
QT_MOC_LITERAL(45, 945, 22), // "on_launcherBox_toggled"
QT_MOC_LITERAL(46, 968, 46), // "on_enableSoundOnRenderComplet..."
QT_MOC_LITERAL(47, 1015, 37), // "on_enableHardwarningsCheckBox..."
QT_MOC_LITERAL(48, 1053, 34), // "on_enableParameterCheckBox_to..."
QT_MOC_LITERAL(49, 1088, 30), // "on_enableRangeCheckBox_toggled"
QT_MOC_LITERAL(50, 1119, 30), // "on_useAsciiSTLCheckBox_toggled"
QT_MOC_LITERAL(51, 1150, 36), // "on_enableHidapiTraceCheckBox_..."
QT_MOC_LITERAL(52, 1187, 39), // "on_checkBoxShowWarningsIn3dVi..."
QT_MOC_LITERAL(53, 1227, 35), // "on_checkBoxMouseCentricZoom_t..."
QT_MOC_LITERAL(54, 1263, 53), // "on_timeThresholdOnRenderCompl..."
QT_MOC_LITERAL(55, 1317, 37), // "on_checkBoxEnableAutocomplete..."
QT_MOC_LITERAL(56, 1355, 41), // "on_lineEditCharacterThreshold..."
QT_MOC_LITERAL(57, 1397, 29), // "on_checkBoxAutoIndent_toggled"
QT_MOC_LITERAL(58, 1427, 37), // "on_checkBoxBackspaceUnindents..."
QT_MOC_LITERAL(59, 1465, 32), // "on_comboBoxIndentUsing_activated"
QT_MOC_LITERAL(60, 1498, 39), // "on_spinBoxIndentationWidth_va..."
QT_MOC_LITERAL(61, 1538, 31), // "on_spinBoxTabWidth_valueChanged"
QT_MOC_LITERAL(62, 1570, 35), // "on_comboBoxTabKeyFunction_act..."
QT_MOC_LITERAL(63, 1606, 35), // "on_comboBoxShowWhitespace_act..."
QT_MOC_LITERAL(64, 1642, 41), // "on_spinBoxShowWhitespaceSize_..."
QT_MOC_LITERAL(65, 1684, 29), // "on_comboBoxLineWrap_activated"
QT_MOC_LITERAL(66, 1714, 45), // "on_comboBoxLineWrapIndentatio..."
QT_MOC_LITERAL(67, 1760, 48), // "on_spinBoxLineWrapIndentation..."
QT_MOC_LITERAL(68, 1809, 47), // "on_comboBoxLineWrapVisualizat..."
QT_MOC_LITERAL(69, 1857, 45), // "on_comboBoxLineWrapVisualizat..."
QT_MOC_LITERAL(70, 1903, 46), // "on_comboBoxModifierNumberScro..."
QT_MOC_LITERAL(71, 1950, 39), // "on_checkBoxHighlightCurrentLi..."
QT_MOC_LITERAL(72, 1990, 38), // "on_checkBoxEnableBraceMatchin..."
QT_MOC_LITERAL(73, 2029, 36), // "on_checkBoxEnableLineNumbers_..."
QT_MOC_LITERAL(74, 2066, 45), // "on_pushButtonOctoPrintCheckCo..."
QT_MOC_LITERAL(75, 2112, 43), // "on_pushButtonOctoPrintSlicing..."
QT_MOC_LITERAL(76, 2156, 43), // "on_comboBoxOctoPrintSlicingEn..."
QT_MOC_LITERAL(77, 2200, 44), // "on_pushButtonOctoPrintSlicing..."
QT_MOC_LITERAL(78, 2245, 44), // "on_comboBoxOctoPrintSlicingPr..."
QT_MOC_LITERAL(79, 2290, 36), // "on_comboBoxOctoPrintAction_ac..."
QT_MOC_LITERAL(80, 2327, 40), // "on_comboBoxOctoPrintFileForma..."
QT_MOC_LITERAL(81, 2368, 39), // "on_lineEditOctoPrintURL_editi..."
QT_MOC_LITERAL(82, 2408, 42), // "on_lineEditOctoPrintApiKey_ed..."
QT_MOC_LITERAL(83, 2451, 36), // "on_pushButtonOctoPrintApiKey_..."
QT_MOC_LITERAL(84, 2488, 31), // "on_lineEditStepSize_textChanged"
QT_MOC_LITERAL(85, 2520, 4), // "arg1"
QT_MOC_LITERAL(86, 2525, 42), // "on_checkBoxEnableNumberScroll..."
QT_MOC_LITERAL(87, 2568, 7) // "checked"

    },
    "Preferences\0requestRedraw\0\0updateUndockMode\0"
    "undockMode\0updateReorderMode\0fontChanged\0"
    "family\0size\0colorSchemeChanged\0scheme\0"
    "openCSGSettingsChanged\0syntaxHighlightChanged\0"
    "s\0editorConfigChanged\0ExperimentalChanged\0"
    "updateMouseCentricZoom\0state\0"
    "autocompleteChanged\0status\0"
    "characterThresholdChanged\0val\0"
    "stepSizeChanged\0actionTriggered\0"
    "QAction*\0featuresCheckBoxToggled\0"
    "on_stackedWidget_currentChanged\0"
    "on_colorSchemeChooser_itemSelectionChanged\0"
    "on_fontChooser_activated\0"
    "on_fontSize_currentIndexChanged\0"
    "on_syntaxHighlight_activated\0"
    "on_openCSGWarningBox_toggled\0"
    "on_enableOpenCSGBox_toggled\0"
    "on_cgalCacheSizeMBEdit_textChanged\0"
    "on_polysetCacheSizeMBEdit_textChanged\0"
    "on_opencsgLimitEdit_textChanged\0"
    "on_forceGoldfeatherBox_toggled\0"
    "on_mouseWheelZoomBox_toggled\0"
    "on_localizationCheckBox_toggled\0"
    "on_autoReloadRaiseCheckBox_toggled\0"
    "on_updateCheckBox_toggled\0"
    "on_snapshotCheckBox_toggled\0"
    "on_reorderCheckBox_toggled\0"
    "on_undockCheckBox_toggled\0"
    "on_checkNowButton_clicked\0"
    "on_launcherBox_toggled\0"
    "on_enableSoundOnRenderCompleteCheckBox_toggled\0"
    "on_enableHardwarningsCheckBox_toggled\0"
    "on_enableParameterCheckBox_toggled\0"
    "on_enableRangeCheckBox_toggled\0"
    "on_useAsciiSTLCheckBox_toggled\0"
    "on_enableHidapiTraceCheckBox_toggled\0"
    "on_checkBoxShowWarningsIn3dView_toggled\0"
    "on_checkBoxMouseCentricZoom_toggled\0"
    "on_timeThresholdOnRenderCompleteSoundEdit_textChanged\0"
    "on_checkBoxEnableAutocomplete_toggled\0"
    "on_lineEditCharacterThreshold_textChanged\0"
    "on_checkBoxAutoIndent_toggled\0"
    "on_checkBoxBackspaceUnindents_toggled\0"
    "on_comboBoxIndentUsing_activated\0"
    "on_spinBoxIndentationWidth_valueChanged\0"
    "on_spinBoxTabWidth_valueChanged\0"
    "on_comboBoxTabKeyFunction_activated\0"
    "on_comboBoxShowWhitespace_activated\0"
    "on_spinBoxShowWhitespaceSize_valueChanged\0"
    "on_comboBoxLineWrap_activated\0"
    "on_comboBoxLineWrapIndentationStyle_activated\0"
    "on_spinBoxLineWrapIndentationIndent_valueChanged\0"
    "on_comboBoxLineWrapVisualizationStart_activated\0"
    "on_comboBoxLineWrapVisualizationEnd_activated\0"
    "on_comboBoxModifierNumberScrollWheel_activated\0"
    "on_checkBoxHighlightCurrentLine_toggled\0"
    "on_checkBoxEnableBraceMatching_toggled\0"
    "on_checkBoxEnableLineNumbers_toggled\0"
    "on_pushButtonOctoPrintCheckConnection_clicked\0"
    "on_pushButtonOctoPrintSlicingEngine_clicked\0"
    "on_comboBoxOctoPrintSlicingEngine_activated\0"
    "on_pushButtonOctoPrintSlicingProfile_clicked\0"
    "on_comboBoxOctoPrintSlicingProfile_activated\0"
    "on_comboBoxOctoPrintAction_activated\0"
    "on_comboBoxOctoPrintFileFormat_activated\0"
    "on_lineEditOctoPrintURL_editingFinished\0"
    "on_lineEditOctoPrintApiKey_editingFinished\0"
    "on_pushButtonOctoPrintApiKey_clicked\0"
    "on_lineEditStepSize_textChanged\0arg1\0"
    "on_checkBoxEnableNumberScrollWheel_toggled\0"
    "checked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Preferences[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      75,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      13,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  389,    2, 0x06 /* Public */,
       3,    1,  390,    2, 0x06 /* Public */,
       5,    1,  393,    2, 0x06 /* Public */,
       6,    2,  396,    2, 0x06 /* Public */,
       9,    1,  401,    2, 0x06 /* Public */,
      11,    0,  404,    2, 0x06 /* Public */,
      12,    1,  405,    2, 0x06 /* Public */,
      14,    0,  408,    2, 0x06 /* Public */,
      15,    0,  409,    2, 0x06 /* Public */,
      16,    1,  410,    2, 0x06 /* Public */,
      18,    1,  413,    2, 0x06 /* Public */,
      20,    1,  416,    2, 0x06 /* Public */,
      22,    1,  419,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      23,    1,  422,    2, 0x0a /* Public */,
      25,    1,  425,    2, 0x0a /* Public */,
      26,    1,  428,    2, 0x0a /* Public */,
      27,    0,  431,    2, 0x0a /* Public */,
      28,    1,  432,    2, 0x0a /* Public */,
      29,    1,  435,    2, 0x0a /* Public */,
      30,    1,  438,    2, 0x0a /* Public */,
      31,    1,  441,    2, 0x0a /* Public */,
      32,    1,  444,    2, 0x0a /* Public */,
      33,    1,  447,    2, 0x0a /* Public */,
      34,    1,  450,    2, 0x0a /* Public */,
      35,    1,  453,    2, 0x0a /* Public */,
      36,    1,  456,    2, 0x0a /* Public */,
      37,    1,  459,    2, 0x0a /* Public */,
      38,    1,  462,    2, 0x0a /* Public */,
      39,    1,  465,    2, 0x0a /* Public */,
      40,    1,  468,    2, 0x0a /* Public */,
      41,    1,  471,    2, 0x0a /* Public */,
      42,    1,  474,    2, 0x0a /* Public */,
      43,    1,  477,    2, 0x0a /* Public */,
      44,    0,  480,    2, 0x0a /* Public */,
      45,    1,  481,    2, 0x0a /* Public */,
      46,    1,  484,    2, 0x0a /* Public */,
      47,    1,  487,    2, 0x0a /* Public */,
      48,    1,  490,    2, 0x0a /* Public */,
      49,    1,  493,    2, 0x0a /* Public */,
      50,    1,  496,    2, 0x0a /* Public */,
      51,    1,  499,    2, 0x0a /* Public */,
      52,    1,  502,    2, 0x0a /* Public */,
      53,    1,  505,    2, 0x0a /* Public */,
      54,    1,  508,    2, 0x0a /* Public */,
      55,    1,  511,    2, 0x0a /* Public */,
      56,    1,  514,    2, 0x0a /* Public */,
      57,    1,  517,    2, 0x0a /* Public */,
      58,    1,  520,    2, 0x0a /* Public */,
      59,    1,  523,    2, 0x0a /* Public */,
      60,    1,  526,    2, 0x0a /* Public */,
      61,    1,  529,    2, 0x0a /* Public */,
      62,    1,  532,    2, 0x0a /* Public */,
      63,    1,  535,    2, 0x0a /* Public */,
      64,    1,  538,    2, 0x0a /* Public */,
      65,    1,  541,    2, 0x0a /* Public */,
      66,    1,  544,    2, 0x0a /* Public */,
      67,    1,  547,    2, 0x0a /* Public */,
      68,    1,  550,    2, 0x0a /* Public */,
      69,    1,  553,    2, 0x0a /* Public */,
      70,    1,  556,    2, 0x0a /* Public */,
      71,    1,  559,    2, 0x0a /* Public */,
      72,    1,  562,    2, 0x0a /* Public */,
      73,    1,  565,    2, 0x0a /* Public */,
      74,    0,  568,    2, 0x0a /* Public */,
      75,    0,  569,    2, 0x0a /* Public */,
      76,    1,  570,    2, 0x0a /* Public */,
      77,    0,  573,    2, 0x0a /* Public */,
      78,    1,  574,    2, 0x0a /* Public */,
      79,    1,  577,    2, 0x0a /* Public */,
      80,    1,  580,    2, 0x0a /* Public */,
      81,    0,  583,    2, 0x0a /* Public */,
      82,    0,  584,    2, 0x0a /* Public */,
      83,    0,  585,    2, 0x0a /* Public */,
      84,    1,  586,    2, 0x08 /* Private */,
      86,    1,  589,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::UInt,    7,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   21,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 24,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   85,
    QMetaType::Void, QMetaType::Bool,   87,

       0        // eod
};

void Preferences::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Preferences *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestRedraw(); break;
        case 1: _t->updateUndockMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->updateReorderMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->fontChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 4: _t->colorSchemeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->openCSGSettingsChanged(); break;
        case 6: _t->syntaxHighlightChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->editorConfigChanged(); break;
        case 8: _t->ExperimentalChanged(); break;
        case 9: _t->updateMouseCentricZoom((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->autocompleteChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->characterThresholdChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->stepSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->actionTriggered((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 14: _t->featuresCheckBoxToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->on_stackedWidget_currentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->on_colorSchemeChooser_itemSelectionChanged(); break;
        case 17: _t->on_fontChooser_activated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->on_fontSize_currentIndexChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->on_syntaxHighlight_activated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->on_openCSGWarningBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->on_enableOpenCSGBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->on_cgalCacheSizeMBEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 23: _t->on_polysetCacheSizeMBEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->on_opencsgLimitEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 25: _t->on_forceGoldfeatherBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->on_mouseWheelZoomBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->on_localizationCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->on_autoReloadRaiseCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 29: _t->on_updateCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->on_snapshotCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->on_reorderCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 32: _t->on_undockCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 33: _t->on_checkNowButton_clicked(); break;
        case 34: _t->on_launcherBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 35: _t->on_enableSoundOnRenderCompleteCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 36: _t->on_enableHardwarningsCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 37: _t->on_enableParameterCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 38: _t->on_enableRangeCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 39: _t->on_useAsciiSTLCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 40: _t->on_enableHidapiTraceCheckBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 41: _t->on_checkBoxShowWarningsIn3dView_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 42: _t->on_checkBoxMouseCentricZoom_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 43: _t->on_timeThresholdOnRenderCompleteSoundEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 44: _t->on_checkBoxEnableAutocomplete_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 45: _t->on_lineEditCharacterThreshold_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 46: _t->on_checkBoxAutoIndent_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 47: _t->on_checkBoxBackspaceUnindents_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 48: _t->on_comboBoxIndentUsing_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 49: _t->on_spinBoxIndentationWidth_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 50: _t->on_spinBoxTabWidth_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 51: _t->on_comboBoxTabKeyFunction_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 52: _t->on_comboBoxShowWhitespace_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 53: _t->on_spinBoxShowWhitespaceSize_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 54: _t->on_comboBoxLineWrap_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 55: _t->on_comboBoxLineWrapIndentationStyle_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 56: _t->on_spinBoxLineWrapIndentationIndent_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 57: _t->on_comboBoxLineWrapVisualizationStart_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 58: _t->on_comboBoxLineWrapVisualizationEnd_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 59: _t->on_comboBoxModifierNumberScrollWheel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 60: _t->on_checkBoxHighlightCurrentLine_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 61: _t->on_checkBoxEnableBraceMatching_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: _t->on_checkBoxEnableLineNumbers_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 63: _t->on_pushButtonOctoPrintCheckConnection_clicked(); break;
        case 64: _t->on_pushButtonOctoPrintSlicingEngine_clicked(); break;
        case 65: _t->on_comboBoxOctoPrintSlicingEngine_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 66: _t->on_pushButtonOctoPrintSlicingProfile_clicked(); break;
        case 67: _t->on_comboBoxOctoPrintSlicingProfile_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 68: _t->on_comboBoxOctoPrintAction_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 69: _t->on_comboBoxOctoPrintFileFormat_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 70: _t->on_lineEditOctoPrintURL_editingFinished(); break;
        case 71: _t->on_lineEditOctoPrintApiKey_editingFinished(); break;
        case 72: _t->on_pushButtonOctoPrintApiKey_clicked(); break;
        case 73: _t->on_lineEditStepSize_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 74: _t->on_checkBoxEnableNumberScrollWheel_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Preferences::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::requestRedraw)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::updateUndockMode)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::updateReorderMode)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(const QString & , uint ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::fontChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::colorSchemeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Preferences::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::openCSGSettingsChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(const QString & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::syntaxHighlightChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Preferences::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::editorConfigChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Preferences::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::ExperimentalChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::updateMouseCentricZoom)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(bool ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::autocompleteChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::characterThresholdChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Preferences::*)(int ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Preferences::stepSizeChanged)) {
                *result = 12;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Preferences::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Preferences.data,
    qt_meta_data_Preferences,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Preferences::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Preferences::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Preferences.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Preferences"))
        return static_cast< Ui::Preferences*>(this);
    if (!strcmp(_clname, "InitConfigurator"))
        return static_cast< InitConfigurator*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Preferences::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 75)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 75;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 75)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 75;
    }
    return _id;
}

// SIGNAL 0
void Preferences::requestRedraw()const
{
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Preferences::updateUndockMode(bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Preferences::updateReorderMode(bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Preferences::fontChanged(const QString & _t1, uint _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Preferences::colorSchemeChanged(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Preferences::openCSGSettingsChanged()const
{
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Preferences::syntaxHighlightChanged(const QString & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Preferences::editorConfigChanged()const
{
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Preferences::ExperimentalChanged()const
{
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Preferences::updateMouseCentricZoom(bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 9, _a);
}

// SIGNAL 10
void Preferences::autocompleteChanged(bool _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 10, _a);
}

// SIGNAL 11
void Preferences::characterThresholdChanged(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 11, _a);
}

// SIGNAL 12
void Preferences::stepSizeChanged(int _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< Preferences *>(this), &staticMetaObject, 12, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
