/****************************************************************************
** Meta object code from reading C++ file 'DBusInputDriver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/input/DBusInputDriver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DBusInputDriver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DBusInputDriver_t {
    QByteArrayData data[19];
    char stringdata0[163];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DBusInputDriver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DBusInputDriver_t qt_meta_stringdata_DBusInputDriver = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DBusInputDriver"
QT_MOC_LITERAL(1, 16, 4), // "zoom"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 6), // "zoomTo"
QT_MOC_LITERAL(4, 29, 6), // "rotate"
QT_MOC_LITERAL(5, 36, 1), // "x"
QT_MOC_LITERAL(6, 38, 1), // "y"
QT_MOC_LITERAL(7, 40, 1), // "z"
QT_MOC_LITERAL(8, 42, 8), // "rotateTo"
QT_MOC_LITERAL(9, 51, 14), // "rotateByVector"
QT_MOC_LITERAL(10, 66, 9), // "translate"
QT_MOC_LITERAL(11, 76, 11), // "translateTo"
QT_MOC_LITERAL(12, 88, 6), // "action"
QT_MOC_LITERAL(13, 95, 11), // "buttonPress"
QT_MOC_LITERAL(14, 107, 3), // "idx"
QT_MOC_LITERAL(15, 111, 11), // "getRotation"
QT_MOC_LITERAL(16, 123, 13), // "QList<double>"
QT_MOC_LITERAL(17, 137, 14), // "getTranslation"
QT_MOC_LITERAL(18, 152, 10) // "getActions"

    },
    "DBusInputDriver\0zoom\0\0zoomTo\0rotate\0"
    "x\0y\0z\0rotateTo\0rotateByVector\0translate\0"
    "translateTo\0action\0buttonPress\0idx\0"
    "getRotation\0QList<double>\0getTranslation\0"
    "getActions"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DBusInputDriver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x0a /* Public */,
       3,    1,   77,    2, 0x0a /* Public */,
       4,    3,   80,    2, 0x0a /* Public */,
       8,    3,   87,    2, 0x0a /* Public */,
       9,    3,   94,    2, 0x0a /* Public */,
      10,    3,  101,    2, 0x0a /* Public */,
      11,    3,  108,    2, 0x0a /* Public */,
      12,    1,  115,    2, 0x0a /* Public */,
      13,    1,  118,    2, 0x0a /* Public */,
      15,    0,  121,    2, 0x0a /* Public */,
      17,    0,  122,    2, 0x0a /* Public */,
      18,    0,  123,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Double,    1,
    QMetaType::Void, QMetaType::Double,    1,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double,    5,    6,    7,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double,    5,    6,    7,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double,    5,    6,    7,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double,    5,    6,    7,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double,    5,    6,    7,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::UInt,   14,
    0x80000000 | 16,
    0x80000000 | 16,
    QMetaType::QStringList,

       0        // eod
};

void DBusInputDriver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DBusInputDriver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->zoom((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->zoomTo((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->rotate((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 3: _t->rotateTo((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 4: _t->rotateByVector((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 5: _t->translate((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 6: _t->translateTo((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 7: _t->action((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 8: _t->buttonPress((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 9: { QList<double> _r = _t->getRotation();
            if (_a[0]) *reinterpret_cast< QList<double>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QList<double> _r = _t->getTranslation();
            if (_a[0]) *reinterpret_cast< QList<double>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QStringList _r = _t->getActions();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DBusInputDriver::staticMetaObject = { {
    QMetaObject::SuperData::link<InputDriver::staticMetaObject>(),
    qt_meta_stringdata_DBusInputDriver.data,
    qt_meta_data_DBusInputDriver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DBusInputDriver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusInputDriver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DBusInputDriver.stringdata0))
        return static_cast<void*>(this);
    return InputDriver::qt_metacast(_clname);
}

int DBusInputDriver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = InputDriver::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
