/********************************************************************************
** Form generated from reading UI file 'ButtonConfigWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BUTTONCONFIGWIDGET_H
#define UI_BUTTONCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Button
{
public:
    QGridLayout *gridLayout;
    QScrollArea *scrollArea_InputButton;
    QWidget *scrollAreaWidgetInputButtonContents;
    QGridLayout *gridLayout_8;
    QLabel *labelInputButton2;
    QComboBox *comboBoxButton6;
    QComboBox *comboBoxButton9;
    QLabel *labelInputButton3;
    QComboBox *comboBoxButton7;
    QComboBox *comboBoxButton8;
    QLabel *labelInputButton5;
    QComboBox *comboBoxButton2;
    QLabel *labelInputButton6;
    QComboBox *comboBoxButton1;
    QLabel *labelInputButton4;
    QComboBox *comboBoxButton0;
    QComboBox *comboBoxButton4;
    QLabel *labelInputButton9;
    QLabel *labelInputButton8;
    QComboBox *comboBoxButton3;
    QLabel *labelInputButton7;
    QComboBox *comboBoxButton5;
    QLabel *labelInputButton0;
    QLabel *labelInputButton1;
    QSpacerItem *verticalSpacer_5;
    QLabel *labelInputButton12;
    QLabel *labelInputButton10;
    QLabel *labelInputButton11;
    QLabel *labelInputButton13;
    QLabel *labelInputButton14;
    QLabel *labelInputButton15;
    QComboBox *comboBoxButton10;
    QComboBox *comboBoxButton11;
    QComboBox *comboBoxButton12;
    QComboBox *comboBoxButton13;
    QComboBox *comboBoxButton14;
    QComboBox *comboBoxButton15;
    QSpacerItem *horizontalSpacer_33;

    void setupUi(QWidget *Button)
    {
        if (Button->objectName().isEmpty())
            Button->setObjectName(QString::fromUtf8("Button"));
        Button->resize(410, 231);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Button->sizePolicy().hasHeightForWidth());
        Button->setSizePolicy(sizePolicy);
        Button->setProperty("unifiedTitleAndToolBarOnMac", QVariant(true));
        gridLayout = new QGridLayout(Button);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        scrollArea_InputButton = new QScrollArea(Button);
        scrollArea_InputButton->setObjectName(QString::fromUtf8("scrollArea_InputButton"));
        scrollArea_InputButton->setWidgetResizable(true);
        scrollAreaWidgetInputButtonContents = new QWidget();
        scrollAreaWidgetInputButtonContents->setObjectName(QString::fromUtf8("scrollAreaWidgetInputButtonContents"));
        scrollAreaWidgetInputButtonContents->setGeometry(QRect(0, 0, 376, 534));
        gridLayout_8 = new QGridLayout(scrollAreaWidgetInputButtonContents);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        labelInputButton2 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton2->setObjectName(QString::fromUtf8("labelInputButton2"));

        gridLayout_8->addWidget(labelInputButton2, 18, 0, 1, 1);

        comboBoxButton6 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton6->setObjectName(QString::fromUtf8("comboBoxButton6"));

        gridLayout_8->addWidget(comboBoxButton6, 22, 1, 1, 3);

        comboBoxButton9 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton9->setObjectName(QString::fromUtf8("comboBoxButton9"));

        gridLayout_8->addWidget(comboBoxButton9, 25, 1, 1, 3);

        labelInputButton3 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton3->setObjectName(QString::fromUtf8("labelInputButton3"));

        gridLayout_8->addWidget(labelInputButton3, 19, 0, 1, 1);

        comboBoxButton7 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton7->setObjectName(QString::fromUtf8("comboBoxButton7"));

        gridLayout_8->addWidget(comboBoxButton7, 23, 1, 1, 3);

        comboBoxButton8 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton8->setObjectName(QString::fromUtf8("comboBoxButton8"));

        gridLayout_8->addWidget(comboBoxButton8, 24, 1, 1, 3);

        labelInputButton5 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton5->setObjectName(QString::fromUtf8("labelInputButton5"));

        gridLayout_8->addWidget(labelInputButton5, 21, 0, 1, 1);

        comboBoxButton2 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton2->setObjectName(QString::fromUtf8("comboBoxButton2"));

        gridLayout_8->addWidget(comboBoxButton2, 18, 1, 1, 3);

        labelInputButton6 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton6->setObjectName(QString::fromUtf8("labelInputButton6"));

        gridLayout_8->addWidget(labelInputButton6, 22, 0, 1, 1);

        comboBoxButton1 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton1->setObjectName(QString::fromUtf8("comboBoxButton1"));

        gridLayout_8->addWidget(comboBoxButton1, 17, 1, 1, 3);

        labelInputButton4 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton4->setObjectName(QString::fromUtf8("labelInputButton4"));

        gridLayout_8->addWidget(labelInputButton4, 20, 0, 1, 1);

        comboBoxButton0 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton0->setObjectName(QString::fromUtf8("comboBoxButton0"));

        gridLayout_8->addWidget(comboBoxButton0, 16, 1, 1, 3);

        comboBoxButton4 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton4->setObjectName(QString::fromUtf8("comboBoxButton4"));

        gridLayout_8->addWidget(comboBoxButton4, 20, 1, 1, 3);

        labelInputButton9 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton9->setObjectName(QString::fromUtf8("labelInputButton9"));

        gridLayout_8->addWidget(labelInputButton9, 25, 0, 1, 1);

        labelInputButton8 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton8->setObjectName(QString::fromUtf8("labelInputButton8"));

        gridLayout_8->addWidget(labelInputButton8, 24, 0, 1, 1);

        comboBoxButton3 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton3->setObjectName(QString::fromUtf8("comboBoxButton3"));

        gridLayout_8->addWidget(comboBoxButton3, 19, 1, 1, 3);

        labelInputButton7 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton7->setObjectName(QString::fromUtf8("labelInputButton7"));

        gridLayout_8->addWidget(labelInputButton7, 23, 0, 1, 1);

        comboBoxButton5 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton5->setObjectName(QString::fromUtf8("comboBoxButton5"));

        gridLayout_8->addWidget(comboBoxButton5, 21, 1, 1, 3);

        labelInputButton0 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton0->setObjectName(QString::fromUtf8("labelInputButton0"));

        gridLayout_8->addWidget(labelInputButton0, 16, 0, 1, 1);

        labelInputButton1 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton1->setObjectName(QString::fromUtf8("labelInputButton1"));

        gridLayout_8->addWidget(labelInputButton1, 17, 0, 1, 1);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_8->addItem(verticalSpacer_5, 32, 0, 1, 1);

        labelInputButton12 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton12->setObjectName(QString::fromUtf8("labelInputButton12"));

        gridLayout_8->addWidget(labelInputButton12, 28, 0, 1, 1);

        labelInputButton10 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton10->setObjectName(QString::fromUtf8("labelInputButton10"));

        gridLayout_8->addWidget(labelInputButton10, 26, 0, 1, 1);

        labelInputButton11 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton11->setObjectName(QString::fromUtf8("labelInputButton11"));

        gridLayout_8->addWidget(labelInputButton11, 27, 0, 1, 1);

        labelInputButton13 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton13->setObjectName(QString::fromUtf8("labelInputButton13"));

        gridLayout_8->addWidget(labelInputButton13, 29, 0, 1, 1);

        labelInputButton14 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton14->setObjectName(QString::fromUtf8("labelInputButton14"));

        gridLayout_8->addWidget(labelInputButton14, 30, 0, 1, 1);

        labelInputButton15 = new QLabel(scrollAreaWidgetInputButtonContents);
        labelInputButton15->setObjectName(QString::fromUtf8("labelInputButton15"));

        gridLayout_8->addWidget(labelInputButton15, 31, 0, 1, 1);

        comboBoxButton10 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton10->setObjectName(QString::fromUtf8("comboBoxButton10"));

        gridLayout_8->addWidget(comboBoxButton10, 26, 1, 1, 3);

        comboBoxButton11 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton11->setObjectName(QString::fromUtf8("comboBoxButton11"));

        gridLayout_8->addWidget(comboBoxButton11, 27, 1, 1, 3);

        comboBoxButton12 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton12->setObjectName(QString::fromUtf8("comboBoxButton12"));

        gridLayout_8->addWidget(comboBoxButton12, 28, 1, 1, 3);

        comboBoxButton13 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton13->setObjectName(QString::fromUtf8("comboBoxButton13"));

        gridLayout_8->addWidget(comboBoxButton13, 29, 1, 1, 3);

        comboBoxButton14 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton14->setObjectName(QString::fromUtf8("comboBoxButton14"));

        gridLayout_8->addWidget(comboBoxButton14, 30, 1, 1, 3);

        comboBoxButton15 = new QComboBox(scrollAreaWidgetInputButtonContents);
        comboBoxButton15->setObjectName(QString::fromUtf8("comboBoxButton15"));

        gridLayout_8->addWidget(comboBoxButton15, 31, 1, 1, 3);

        horizontalSpacer_33 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_33, 32, 1, 1, 1);

        scrollArea_InputButton->setWidget(scrollAreaWidgetInputButtonContents);

        gridLayout->addWidget(scrollArea_InputButton, 0, 0, 1, 1);


        retranslateUi(Button);

        QMetaObject::connectSlotsByName(Button);
    } // setupUi

    void retranslateUi(QWidget *Button)
    {
        Button->setWindowTitle(q_("Preferences", nullptr));
        labelInputButton2->setText(q_("Button 2", nullptr));
        labelInputButton3->setText(q_("Button 3", nullptr));
        labelInputButton5->setText(q_("Button 5", nullptr));
        labelInputButton6->setText(q_("Button 6", nullptr));
        labelInputButton4->setText(q_("Button 4", nullptr));
        labelInputButton9->setText(q_("Button 9", nullptr));
        labelInputButton8->setText(q_("Button 8", nullptr));
        labelInputButton7->setText(q_("Button 7", nullptr));
        labelInputButton0->setText(q_("Button 0", nullptr));
        labelInputButton1->setText(q_("Button 1", nullptr));
        labelInputButton12->setText(q_("Button 12", nullptr));
        labelInputButton10->setText(q_("Button 10", nullptr));
        labelInputButton11->setText(q_("Button 11", nullptr));
        labelInputButton13->setText(q_("Button 13", nullptr));
        labelInputButton14->setText(q_("Button 14", nullptr));
        labelInputButton15->setText(q_("Button 15", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Button: public Ui_Button {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BUTTONCONFIGWIDGET_H
