/********************************************************************************
** Form generated from reading UI file 'AboutDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout_2;
    QLabel *logoLabel;
    QLabel *titleLabel;
    QSpacerItem *horizontalSpacer;
    QTextBrowser *aboutText;
    QPushButton *okPushButton;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName(QString::fromUtf8("AboutDialog"));
        AboutDialog->resize(628, 419);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(AboutDialog->sizePolicy().hasHeightForWidth());
        AboutDialog->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(AboutDialog);
        gridLayout->setContentsMargins(16, 16, 16, 16);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setHorizontalSpacing(6);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setHorizontalSpacing(6);
        gridLayout_2->setContentsMargins(0, -1, -1, -1);
        logoLabel = new QLabel(AboutDialog);
        logoLabel->setObjectName(QString::fromUtf8("logoLabel"));
        logoLabel->setMinimumSize(QSize(96, 96));
        logoLabel->setMaximumSize(QSize(96, 96));
        logoLabel->setPixmap(QPixmap(QString::fromUtf8(":/icons/openscad.png")));
        logoLabel->setScaledContents(true);

        gridLayout_2->addWidget(logoLabel, 0, 0, 1, 1);

        titleLabel = new QLabel(AboutDialog);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));

        gridLayout_2->addWidget(titleLabel, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(20, 2, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 1, 1, 1);


        gridLayout->addLayout(gridLayout_2, 0, 0, 1, 3);

        aboutText = new QTextBrowser(AboutDialog);
        aboutText->setObjectName(QString::fromUtf8("aboutText"));
        aboutText->setMinimumSize(QSize(400, 200));
        aboutText->setReadOnly(true);
        aboutText->setSource(QUrl(QString::fromUtf8("qrc:/src/AboutDialog.html")));
        aboutText->setOpenExternalLinks(true);

        gridLayout->addWidget(aboutText, 2, 0, 1, 3);

        okPushButton = new QPushButton(AboutDialog);
        okPushButton->setObjectName(QString::fromUtf8("okPushButton"));

        gridLayout->addWidget(okPushButton, 4, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 4, 1, 1, 1);


        retranslateUi(AboutDialog);

        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(q_("About OpenSCAD", nullptr));
        logoLabel->setText(QString());
        titleLabel->setText(q_("<html><head/><body>\n"
"<p style=\"font-family: 'Open Sans', 'Droid Sans', 'sans-serif'; font-weight: bold; padding-bottom: 0; margin-bottom: 0; font-size: 22pt;\"><span style=\"color: green;\">Open</span>SCAD</p>\n"
"<p style=\"font-family: 'Open Sans', 'Droid Sans', 'sans-serif'; font-weight: normal; font-size: 14pt; padding-top: 0; margin-top: 0; margin-left: 2em;\">The Programmers Solid 3D CAD Modeller</p>\n"
"</body></html>\n"
"\n"
"\n"
"", nullptr));
        okPushButton->setText(q_("OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
