/****************************************************************************
** Meta object code from reading C++ file 'scintillaeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/scintillaeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scintillaeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScintillaEditor_t {
    QByteArrayData data[44];
    char stringdata0[571];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScintillaEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScintillaEditor_t qt_meta_stringdata_ScintillaEditor = {
    {
QT_MOC_LITERAL(0, 0, 15), // "ScintillaEditor"
QT_MOC_LITERAL(1, 16, 14), // "previewRequest"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 25), // "hyperlinkIndicatorClicked"
QT_MOC_LITERAL(4, 58, 3), // "val"
QT_MOC_LITERAL(5, 62, 10), // "uriDropped"
QT_MOC_LITERAL(6, 73, 6), // "zoomIn"
QT_MOC_LITERAL(7, 80, 7), // "zoomOut"
QT_MOC_LITERAL(8, 88, 12), // "setPlainText"
QT_MOC_LITERAL(9, 101, 18), // "setContentModified"
QT_MOC_LITERAL(10, 120, 17), // "isContentModified"
QT_MOC_LITERAL(11, 138, 14), // "highlightError"
QT_MOC_LITERAL(12, 153, 20), // "unhighlightLastError"
QT_MOC_LITERAL(13, 174, 18), // "setHighlightScheme"
QT_MOC_LITERAL(14, 193, 15), // "indentSelection"
QT_MOC_LITERAL(15, 209, 17), // "unindentSelection"
QT_MOC_LITERAL(16, 227, 16), // "commentSelection"
QT_MOC_LITERAL(17, 244, 18), // "uncommentSelection"
QT_MOC_LITERAL(18, 263, 6), // "insert"
QT_MOC_LITERAL(19, 270, 7), // "setText"
QT_MOC_LITERAL(20, 278, 4), // "undo"
QT_MOC_LITERAL(21, 283, 4), // "redo"
QT_MOC_LITERAL(22, 288, 3), // "cut"
QT_MOC_LITERAL(23, 292, 4), // "copy"
QT_MOC_LITERAL(24, 297, 5), // "paste"
QT_MOC_LITERAL(25, 303, 8), // "initFont"
QT_MOC_LITERAL(26, 312, 16), // "displayTemplates"
QT_MOC_LITERAL(27, 329, 14), // "toggleBookmark"
QT_MOC_LITERAL(28, 344, 12), // "nextBookmark"
QT_MOC_LITERAL(29, 357, 12), // "prevBookmark"
QT_MOC_LITERAL(30, 370, 15), // "jumpToNextError"
QT_MOC_LITERAL(31, 386, 13), // "onTextChanged"
QT_MOC_LITERAL(32, 400, 18), // "onUserListSelected"
QT_MOC_LITERAL(33, 419, 2), // "id"
QT_MOC_LITERAL(34, 422, 4), // "text"
QT_MOC_LITERAL(35, 427, 13), // "applySettings"
QT_MOC_LITERAL(36, 441, 21), // "onAutocompleteChanged"
QT_MOC_LITERAL(37, 463, 5), // "state"
QT_MOC_LITERAL(38, 469, 27), // "onCharacterThresholdChanged"
QT_MOC_LITERAL(39, 497, 23), // "fireModificationChanged"
QT_MOC_LITERAL(40, 521, 18), // "onIndicatorClicked"
QT_MOC_LITERAL(41, 540, 4), // "line"
QT_MOC_LITERAL(42, 545, 3), // "col"
QT_MOC_LITERAL(43, 549, 21) // "Qt::KeyboardModifiers"

    },
    "ScintillaEditor\0previewRequest\0\0"
    "hyperlinkIndicatorClicked\0val\0uriDropped\0"
    "zoomIn\0zoomOut\0setPlainText\0"
    "setContentModified\0isContentModified\0"
    "highlightError\0unhighlightLastError\0"
    "setHighlightScheme\0indentSelection\0"
    "unindentSelection\0commentSelection\0"
    "uncommentSelection\0insert\0setText\0"
    "undo\0redo\0cut\0copy\0paste\0initFont\0"
    "displayTemplates\0toggleBookmark\0"
    "nextBookmark\0prevBookmark\0jumpToNextError\0"
    "onTextChanged\0onUserListSelected\0id\0"
    "text\0applySettings\0onAutocompleteChanged\0"
    "state\0onCharacterThresholdChanged\0"
    "fireModificationChanged\0onIndicatorClicked\0"
    "line\0col\0Qt::KeyboardModifiers"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScintillaEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      35,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  189,    2, 0x06 /* Public */,
       3,    1,  190,    2, 0x06 /* Public */,
       5,    1,  193,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,  196,    2, 0x0a /* Public */,
       7,    0,  197,    2, 0x0a /* Public */,
       8,    1,  198,    2, 0x0a /* Public */,
       9,    1,  201,    2, 0x0a /* Public */,
      10,    0,  204,    2, 0x0a /* Public */,
      11,    1,  205,    2, 0x0a /* Public */,
      12,    0,  208,    2, 0x0a /* Public */,
      13,    1,  209,    2, 0x0a /* Public */,
      14,    0,  212,    2, 0x0a /* Public */,
      15,    0,  213,    2, 0x0a /* Public */,
      16,    0,  214,    2, 0x0a /* Public */,
      17,    0,  215,    2, 0x0a /* Public */,
      18,    1,  216,    2, 0x0a /* Public */,
      19,    1,  219,    2, 0x0a /* Public */,
      20,    0,  222,    2, 0x0a /* Public */,
      21,    0,  223,    2, 0x0a /* Public */,
      22,    0,  224,    2, 0x0a /* Public */,
      23,    0,  225,    2, 0x0a /* Public */,
      24,    0,  226,    2, 0x0a /* Public */,
      25,    2,  227,    2, 0x0a /* Public */,
      26,    0,  232,    2, 0x0a /* Public */,
      27,    0,  233,    2, 0x0a /* Public */,
      28,    0,  234,    2, 0x0a /* Public */,
      29,    0,  235,    2, 0x0a /* Public */,
      30,    0,  236,    2, 0x0a /* Public */,
      31,    0,  237,    2, 0x08 /* Private */,
      32,    2,  238,    2, 0x08 /* Private */,
      35,    0,  243,    2, 0x08 /* Private */,
      36,    1,  244,    2, 0x08 /* Private */,
      38,    1,  247,    2, 0x08 /* Private */,
      39,    1,  250,    2, 0x08 /* Private */,
      40,    3,  253,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::QUrl,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::UInt,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   33,   34,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   37,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, 0x80000000 | 43,   41,   42,   37,

       0        // eod
};

void ScintillaEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScintillaEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->previewRequest(); break;
        case 1: _t->hyperlinkIndicatorClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->uriDropped((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 3: _t->zoomIn(); break;
        case 4: _t->zoomOut(); break;
        case 5: _t->setPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->setContentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: { bool _r = _t->isContentModified();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->highlightError((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->unhighlightLastError(); break;
        case 10: _t->setHighlightScheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->indentSelection(); break;
        case 12: _t->unindentSelection(); break;
        case 13: _t->commentSelection(); break;
        case 14: _t->uncommentSelection(); break;
        case 15: _t->insert((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->setText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->undo(); break;
        case 18: _t->redo(); break;
        case 19: _t->cut(); break;
        case 20: _t->copy(); break;
        case 21: _t->paste(); break;
        case 22: _t->initFont((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 23: _t->displayTemplates(); break;
        case 24: _t->toggleBookmark(); break;
        case 25: _t->nextBookmark(); break;
        case 26: _t->prevBookmark(); break;
        case 27: _t->jumpToNextError(); break;
        case 28: _t->onTextChanged(); break;
        case 29: _t->onUserListSelected((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 30: _t->applySettings(); break;
        case 31: _t->onAutocompleteChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 32: _t->onCharacterThresholdChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->fireModificationChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 34: _t->onIndicatorClicked((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< Qt::KeyboardModifiers(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScintillaEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScintillaEditor::previewRequest)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ScintillaEditor::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScintillaEditor::hyperlinkIndicatorClicked)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ScintillaEditor::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScintillaEditor::uriDropped)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScintillaEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<EditorInterface::staticMetaObject>(),
    qt_meta_stringdata_ScintillaEditor.data,
    qt_meta_data_ScintillaEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScintillaEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScintillaEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScintillaEditor.stringdata0))
        return static_cast<void*>(this);
    return EditorInterface::qt_metacast(_clname);
}

int ScintillaEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EditorInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void ScintillaEditor::previewRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ScintillaEditor::hyperlinkIndicatorClicked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ScintillaEditor::uriDropped(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
