/********************************************************************************
** Form generated from reading UI file 'ErrorLog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ERRORLOG_H
#define UI_ERRORLOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTableView>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_errorLogWidget
{
public:
    QAction *actionRowSelected;
    QGridLayout *gridLayout;
    QTableView *logTable;
    QLabel *errorLogShow;
    QComboBox *errorLogComboBox;

    void setupUi(QWidget *errorLogWidget)
    {
        if (errorLogWidget->objectName().isEmpty())
            errorLogWidget->setObjectName(QString::fromUtf8("errorLogWidget"));
        errorLogWidget->resize(659, 506);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(errorLogWidget->sizePolicy().hasHeightForWidth());
        errorLogWidget->setSizePolicy(sizePolicy);
        actionRowSelected = new QAction(errorLogWidget);
        actionRowSelected->setObjectName(QString::fromUtf8("actionRowSelected"));
        gridLayout = new QGridLayout(errorLogWidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        logTable = new QTableView(errorLogWidget);
        logTable->setObjectName(QString::fromUtf8("logTable"));
        logTable->setFocusPolicy(Qt::StrongFocus);
        logTable->setDragEnabled(true);
        logTable->setDragDropMode(QAbstractItemView::DragOnly);
        logTable->setSelectionMode(QAbstractItemView::SingleSelection);
        logTable->setSelectionBehavior(QAbstractItemView::SelectRows);
        logTable->horizontalHeader()->setCascadingSectionResizes(true);
        logTable->horizontalHeader()->setDefaultSectionSize(140);
        logTable->horizontalHeader()->setHighlightSections(false);
        logTable->horizontalHeader()->setStretchLastSection(true);

        gridLayout->addWidget(logTable, 1, 0, 1, 11);

        errorLogShow = new QLabel(errorLogWidget);
        errorLogShow->setObjectName(QString::fromUtf8("errorLogShow"));

        gridLayout->addWidget(errorLogShow, 0, 0, 1, 1);

        errorLogComboBox = new QComboBox(errorLogWidget);
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->addItem(QString());
        errorLogComboBox->setObjectName(QString::fromUtf8("errorLogComboBox"));
        errorLogComboBox->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(errorLogComboBox, 0, 1, 1, 1);

#if QT_CONFIG(shortcut)
        errorLogShow->setBuddy(errorLogComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(errorLogComboBox, logTable);

        retranslateUi(errorLogWidget);

        QMetaObject::connectSlotsByName(errorLogWidget);
    } // setupUi

    void retranslateUi(QWidget *errorLogWidget)
    {
        errorLogWidget->setWindowTitle(q_("Error Log", nullptr));
        actionRowSelected->setText(q_("RowSelected", nullptr));
#if QT_CONFIG(shortcut)
        actionRowSelected->setShortcut(q_("Return", nullptr));
#endif // QT_CONFIG(shortcut)
        errorLogShow->setText(q_("&Show", nullptr));
        errorLogComboBox->setItemText(0, q_("All", nullptr));
        errorLogComboBox->setItemText(1, q_("ERROR", nullptr));
        errorLogComboBox->setItemText(2, q_("WARNING", nullptr));
        errorLogComboBox->setItemText(3, q_("UI-WARNING", nullptr));
        errorLogComboBox->setItemText(4, q_("FONT-WARNING", nullptr));
        errorLogComboBox->setItemText(5, q_("EXPORT-WARNING", nullptr));
        errorLogComboBox->setItemText(6, q_("EXPORT-ERROR", nullptr));
        errorLogComboBox->setItemText(7, q_("TRACE", nullptr));
        errorLogComboBox->setItemText(8, q_("DEPRECATED", nullptr));

    } // retranslateUi

};

namespace Ui {
    class errorLogWidget: public Ui_errorLogWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ERRORLOG_H
