/********************************************************************************
** Form generated from reading UI file 'Preferences.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFERENCES_H
#define UI_PREFERENCES_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "input/AxisConfigWidget.h"
#include "input/ButtonConfigWidget.h"

QT_BEGIN_NAMESPACE

class Ui_Preferences
{
public:
    QAction *prefsAction3DView;
    QAction *prefsActionAdvanced;
    QAction *prefsActionEditor;
    QAction *prefsActionUpdate;
    QAction *prefsActionFeatures;
    QAction *prefsActionInput;
    QAction *prefsActionInputButton;
    QAction *prefsAction3DPrint;
    QWidget *centralwidget;
    QGridLayout *gridLayout_6;
    QStackedWidget *stackedWidget;
    QWidget *page3DView;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QListWidget *colorSchemeChooser;
    QSpacerItem *horizontalSpacer;
    QCheckBox *checkBoxShowWarningsIn3dView;
    QCheckBox *checkBoxMouseCentricZoom;
    QWidget *pageEditor;
    QVBoxLayout *verticalLayout;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_2;
    QGridLayout *gridLayout_5;
    QSpacerItem *horizontalSpacer_4;
    QGroupBox *groupBoxNumberScroll;
    QGridLayout *gridLayout_8;
    QSpacerItem *horizontalSpacer_7;
    QSpacerItem *horizontalSpacer_28;
    QComboBox *comboBoxModifierNumberScrollWheel;
    QSpacerItem *horizontalSpacer_34;
    QLineEdit *lineEditStepSize;
    QLabel *labelStepSize;
    QSpacerItem *horizontalSpacer_33;
    QCheckBox *checkBoxEnableNumberScrollWheel;
    QLabel *labelModifierNumberScrollWheel;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *groupBoxAutocomplete;
    QGridLayout *gridLayout1;
    QCheckBox *checkBoxEnableAutocomplete;
    QHBoxLayout *horizontalLayout_30;
    QLabel *labelCharacterThreshold;
    QLineEdit *lineEditCharacterThreshold;
    QGroupBox *groupBoxLineWrap;
    QGridLayout *gridLayout_4;
    QHBoxLayout *horizontalLayout_25;
    QComboBox *comboBoxLineWrap;
    QSpacerItem *horizontalSpacer_25;
    QHBoxLayout *horizontalLayout_21;
    QSpacerItem *horizontalSpacer_23;
    QLabel *labelLineWrapIndentation;
    QHBoxLayout *horizontalLayout_22;
    QSpacerItem *horizontalSpacer_24;
    QLabel *labelLineWrapVisualization;
    QHBoxLayout *horizontalLayout_24;
    QLabel *labelLineWrapIndentationStyle;
    QComboBox *comboBoxLineWrapIndentationStyle;
    QLabel *labelLineWrapIndentationIndent;
    QSpinBox *spinBoxLineWrapIndentationIndent;
    QSpacerItem *horizontalSpacer_26;
    QHBoxLayout *horizontalLayout_23;
    QLabel *labelLineWrapVisualizationStart;
    QComboBox *comboBoxLineWrapVisualizationStart;
    QLabel *labelLineWrapVisualizationEnd;
    QComboBox *comboBoxLineWrapVisualizationEnd;
    QSpacerItem *horizontalSpacer_27;
    QHBoxLayout *horizontalLayout_20;
    QSpacerItem *horizontalSpacer_22;
    QLabel *labelLineWrap;
    QGroupBox *groupBoxDisplay;
    QGridLayout *gridLayout;
    QCheckBox *checkBoxHighlightCurrentLine;
    QCheckBox *checkBoxEnableLineNumbers;
    QCheckBox *checkBoxEnableBraceMatching;
    QGridLayout *gridLayout_9;
    QHBoxLayout *horizontalLayout_28;
    QSpacerItem *horizontalSpacer_29;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_5;
    QFontComboBox *fontChooser;
    QComboBox *fontSize;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *horizontalLayout_29;
    QSpacerItem *horizontalSpacer_30;
    QLabel *labelColorSyntaxHighlighting;
    QSpacerItem *horizontalSpacer_6;
    QCheckBox *mouseWheelZoomBox;
    QHBoxLayout *horizontalLayout_26;
    QComboBox *syntaxHighlight;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *groupBoxIndentation;
    QGridLayout *gridLayout_3;
    QHBoxLayout *horizontalLayout_16;
    QSpinBox *spinBoxIndentationWidth;
    QSpacerItem *horizontalSpacer_11;
    QSpacerItem *horizontalSpacer_13;
    QCheckBox *checkBoxAutoIndent;
    QSpacerItem *horizontalSpacer_32;
    QCheckBox *checkBoxBackspaceUnindents;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer_8;
    QLabel *label_10;
    QHBoxLayout *horizontalLayout_15;
    QComboBox *comboBoxIndentUsing;
    QSpacerItem *horizontalSpacer_14;
    QHBoxLayout *horizontalLayout_8;
    QSpacerItem *horizontalSpacer_15;
    QLabel *labelIndentationWidth;
    QHBoxLayout *horizontalLayout_12;
    QSpacerItem *horizontalSpacer_16;
    QLabel *labelTabWidth;
    QHBoxLayout *horizontalLayout_17;
    QSpinBox *spinBoxTabWidth;
    QSpacerItem *horizontalSpacer_17;
    QHBoxLayout *horizontalLayout_13;
    QSpacerItem *horizontalSpacer_19;
    QLabel *label_9;
    QHBoxLayout *horizontalLayout_18;
    QComboBox *comboBoxTabKeyFunction;
    QSpacerItem *horizontalSpacer_18;
    QHBoxLayout *horizontalLayout_14;
    QSpacerItem *horizontalSpacer_20;
    QLabel *labelWhitespaces;
    QHBoxLayout *horizontalLayout_19;
    QComboBox *comboBoxShowWhitespace;
    QLabel *labelShowWhitespacesSize;
    QSpinBox *spinBoxShowWhitespaceSize;
    QSpacerItem *horizontalSpacer_21;
    QWidget *pageUpdate;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_9;
    QVBoxLayout *verticalLayout_8;
    QCheckBox *updateCheckBox;
    QCheckBox *snapshotCheckBox;
    QSpacerItem *horizontalSpacer_2;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_11;
    QSpacerItem *horizontalSpacer_10;
    QPushButton *checkNowButton;
    QSpacerItem *horizontalSpacer_9;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer_31;
    QLabel *lastCheckedLabel;
    QSpacerItem *horizontalSpacer_12;
    QSpacerItem *verticalSpacer_7;
    QWidget *pageFeatures;
    QGridLayout *gridLayout_2;
    QVBoxLayout *verticalLayout_9;
    QLabel *labelFeatures;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_10;
    QGridLayout *gridLayoutExperimentalFeatures;
    QSpacerItem *verticalSpacer_3;
    QWidget *page3DPrint;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBoxOctoPrint;
    QGridLayout *gridLayout_7;
    QLineEdit *lineEditOctoPrintURL;
    QLabel *labelOctoPrintURL;
    QLabel *labelOctoPrintApiKey;
    QHBoxLayout *horizontalLayoutOctoPrintSlicingEngine;
    QComboBox *comboBoxOctoPrintSlicingEngine;
    QSpacerItem *horizontalSpacerOctoPrintSlicingEngine;
    QPushButton *pushButtonOctoPrintSlicingEngine;
    QHBoxLayout *horizontalLayoutOctoPrintSlicingProfile;
    QComboBox *comboBoxOctoPrintSlicingProfile;
    QSpacerItem *horizontalSpacerOctoPrintSlicingProfile;
    QPushButton *pushButtonOctoPrintSlicingProfile;
    QHBoxLayout *horizontalLayoutOctoPrintCheckConnection;
    QSpacerItem *horizontalSpacerOctoPrintCheckConnection;
    QLabel *labelOctoPrintCheckConnection;
    QPushButton *pushButtonOctoPrintCheckConnection;
    QLabel *labelOctoPrintSlicingProfile;
    QLabel *labelOctoPrintSlicingEngine;
    QLabel *labelOctoPrintFileFormat;
    QHBoxLayout *horizontalLayoutOctoPrintFileFormat;
    QComboBox *comboBoxOctoPrintFileFormat;
    QSpacerItem *horizontalSpacerOctoPrintFileFormat;
    QLabel *labelOctoPrintAction;
    QHBoxLayout *horizontalLayoutOctoPrintSliceAction;
    QComboBox *comboBoxOctoPrintAction;
    QSpacerItem *horizontalSpacerOctoPrintSliceAction;
    QHBoxLayout *horizontalLayoutApiKey;
    QLineEdit *lineEditOctoPrintApiKey;
    QPushButton *pushButtonOctoPrintApiKey;
    QLabel *labelOctoPrintApiKeyNote;
    QSpacerItem *verticalSpacer3DPrint;
    QWidget *pageAdvanced;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollAreaAdvanced;
    QWidget *scrollAreaAdvancedWidgetContents;
    QVBoxLayout *verticalLayout_12;
    QGroupBox *groupBox_CSG;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *openCSGWarningBox;
    QCheckBox *enableOpenCSGBox;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_7;
    QLineEdit *opencsgLimitEdit;
    QLabel *label_8;
    QCheckBox *forceGoldfeatherBox;
    QGroupBox *groupBox_CGAL;
    QVBoxLayout *verticalLayout_14;
    QHBoxLayout *horizontalLayout_cgalCacheSizeMB;
    QLabel *label_4;
    QLineEdit *cgalCacheSizeMBEdit;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout_polysetCacheSizeMB;
    QLabel *label_5;
    QLineEdit *polysetCacheSizeMBEdit;
    QLabel *label_6;
    QGroupBox *groupBox_UI;
    QVBoxLayout *verticalLayout_13;
    QCheckBox *reorderCheckBox;
    QCheckBox *undockCheckBox;
    QCheckBox *launcherBox;
    QCheckBox *localizationCheckBox;
    QCheckBox *autoReloadRaiseCheckBox;
    QCheckBox *enableSoundOnRenderCompleteCheckBox;
    QHBoxLayout *horizontalLayout_timeThresholdOnRenderCompleteSound;
    QLabel *timeThresholdOnRenderCompleteSoundLabel;
    QLineEdit *timeThresholdOnRenderCompleteSoundEdit;
    QLabel *secLabel;
    QGroupBox *groupBox_lang;
    QVBoxLayout *verticalLayout_15;
    QCheckBox *enableParameterCheckBox;
    QCheckBox *enableHardwarningsCheckBox;
    QCheckBox *enableRangeCheckBox;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_Export;
    QVBoxLayout *verticalLayout_16;
    QCheckBox *useAsciiSTLCheckBox;
    QSpacerItem *verticalSpacer1;
    QGroupBox *groupBox_Debug;
    QVBoxLayout *verticalLayout_161;
    QCheckBox *enableHidapiTraceCheckBox;
    QWidget *pageInput;
    QGridLayout *gridLayout_13;
    QScrollArea *scrollArea_InputAxis;
    AxisConfigWidget *AxisConfig;
    QWidget *pageInputButton;
    QGridLayout *gridLayout_14;
    QScrollArea *scrollArea_InputButton;
    ButtonConfigWidget *ButtonConfig;
    QToolBar *toolBar;

    void setupUi(QMainWindow *Preferences)
    {
        if (Preferences->objectName().isEmpty())
            Preferences->setObjectName(QString::fromUtf8("Preferences"));
        Preferences->resize(636, 539);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Preferences->sizePolicy().hasHeightForWidth());
        Preferences->setSizePolicy(sizePolicy);
        Preferences->setUnifiedTitleAndToolBarOnMac(true);
        prefsAction3DView = new QAction(Preferences);
        prefsAction3DView->setObjectName(QString::fromUtf8("prefsAction3DView"));
        prefsAction3DView->setCheckable(true);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/prefs3DView.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsAction3DView->setIcon(icon);
        prefsActionAdvanced = new QAction(Preferences);
        prefsActionAdvanced->setObjectName(QString::fromUtf8("prefsActionAdvanced"));
        prefsActionAdvanced->setCheckable(true);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/prefsAdvanced.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsActionAdvanced->setIcon(icon1);
        prefsActionEditor = new QAction(Preferences);
        prefsActionEditor->setObjectName(QString::fromUtf8("prefsActionEditor"));
        prefsActionEditor->setCheckable(true);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/prefsEditor.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsActionEditor->setIcon(icon2);
        prefsActionUpdate = new QAction(Preferences);
        prefsActionUpdate->setObjectName(QString::fromUtf8("prefsActionUpdate"));
        prefsActionUpdate->setCheckable(true);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/prefsUpdate.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsActionUpdate->setIcon(icon3);
        prefsActionFeatures = new QAction(Preferences);
        prefsActionFeatures->setObjectName(QString::fromUtf8("prefsActionFeatures"));
        prefsActionFeatures->setCheckable(true);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/prefsFeatures.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsActionFeatures->setIcon(icon4);
        prefsActionInput = new QAction(Preferences);
        prefsActionInput->setObjectName(QString::fromUtf8("prefsActionInput"));
        prefsActionInput->setCheckable(true);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/prefsInputAxis.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsActionInput->setIcon(icon5);
        prefsActionInputButton = new QAction(Preferences);
        prefsActionInputButton->setObjectName(QString::fromUtf8("prefsActionInputButton"));
        prefsActionInputButton->setCheckable(true);
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/prefsInputButtons.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsActionInputButton->setIcon(icon6);
        prefsAction3DPrint = new QAction(Preferences);
        prefsAction3DPrint->setObjectName(QString::fromUtf8("prefsAction3DPrint"));
        prefsAction3DPrint->setCheckable(true);
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/prefs3DPrint.png"), QSize(), QIcon::Normal, QIcon::Off);
        prefsAction3DPrint->setIcon(icon7);
        centralwidget = new QWidget(Preferences);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        gridLayout_6 = new QGridLayout(centralwidget);
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        stackedWidget = new QStackedWidget(centralwidget);
        stackedWidget->setObjectName(QString::fromUtf8("stackedWidget"));
        page3DView = new QWidget();
        page3DView->setObjectName(QString::fromUtf8("page3DView"));
        verticalLayout_4 = new QVBoxLayout(page3DView);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        label = new QLabel(page3DView);
        label->setObjectName(QString::fromUtf8("label"));

        verticalLayout_2->addWidget(label);

        colorSchemeChooser = new QListWidget(page3DView);
        colorSchemeChooser->setObjectName(QString::fromUtf8("colorSchemeChooser"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(colorSchemeChooser->sizePolicy().hasHeightForWidth());
        colorSchemeChooser->setSizePolicy(sizePolicy1);
        colorSchemeChooser->setAlternatingRowColors(false);

        verticalLayout_2->addWidget(colorSchemeChooser);


        horizontalLayout->addLayout(verticalLayout_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 3);

        verticalLayout_4->addLayout(horizontalLayout);

        checkBoxShowWarningsIn3dView = new QCheckBox(page3DView);
        checkBoxShowWarningsIn3dView->setObjectName(QString::fromUtf8("checkBoxShowWarningsIn3dView"));
        checkBoxShowWarningsIn3dView->setChecked(false);

        verticalLayout_4->addWidget(checkBoxShowWarningsIn3dView);

        checkBoxMouseCentricZoom = new QCheckBox(page3DView);
        checkBoxMouseCentricZoom->setObjectName(QString::fromUtf8("checkBoxMouseCentricZoom"));
        checkBoxMouseCentricZoom->setChecked(false);

        verticalLayout_4->addWidget(checkBoxMouseCentricZoom);

        stackedWidget->addWidget(page3DView);
        pageEditor = new QWidget();
        pageEditor->setObjectName(QString::fromUtf8("pageEditor"));
        verticalLayout = new QVBoxLayout(pageEditor);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        scrollArea_2 = new QScrollArea(pageEditor);
        scrollArea_2->setObjectName(QString::fromUtf8("scrollArea_2"));
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_2"));
        scrollAreaWidgetContents_2->setGeometry(QRect(0, -450, 602, 901));
        gridLayout_5 = new QGridLayout(scrollAreaWidgetContents_2);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_4, 0, 1, 1, 1);

        groupBoxNumberScroll = new QGroupBox(scrollAreaWidgetContents_2);
        groupBoxNumberScroll->setObjectName(QString::fromUtf8("groupBoxNumberScroll"));
        gridLayout_8 = new QGridLayout(groupBoxNumberScroll);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_7, 1, 1, 1, 1);

        horizontalSpacer_28 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_28, 1, 5, 1, 1);

        comboBoxModifierNumberScrollWheel = new QComboBox(groupBoxNumberScroll);
        comboBoxModifierNumberScrollWheel->addItem(QString());
        comboBoxModifierNumberScrollWheel->addItem(QString());
        comboBoxModifierNumberScrollWheel->addItem(QString());
        comboBoxModifierNumberScrollWheel->setObjectName(QString::fromUtf8("comboBoxModifierNumberScrollWheel"));

        gridLayout_8->addWidget(comboBoxModifierNumberScrollWheel, 3, 2, 1, 1);

        horizontalSpacer_34 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_34, 1, 3, 1, 1);

        lineEditStepSize = new QLineEdit(groupBoxNumberScroll);
        lineEditStepSize->setObjectName(QString::fromUtf8("lineEditStepSize"));

        gridLayout_8->addWidget(lineEditStepSize, 1, 2, 1, 1);

        labelStepSize = new QLabel(groupBoxNumberScroll);
        labelStepSize->setObjectName(QString::fromUtf8("labelStepSize"));

        gridLayout_8->addWidget(labelStepSize, 1, 0, 1, 1);

        horizontalSpacer_33 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_33, 1, 4, 1, 1);

        checkBoxEnableNumberScrollWheel = new QCheckBox(groupBoxNumberScroll);
        checkBoxEnableNumberScrollWheel->setObjectName(QString::fromUtf8("checkBoxEnableNumberScrollWheel"));

        gridLayout_8->addWidget(checkBoxEnableNumberScrollWheel, 2, 0, 1, 1);

        labelModifierNumberScrollWheel = new QLabel(groupBoxNumberScroll);
        labelModifierNumberScrollWheel->setObjectName(QString::fromUtf8("labelModifierNumberScrollWheel"));

        gridLayout_8->addWidget(labelModifierNumberScrollWheel, 3, 0, 1, 1);


        gridLayout_5->addWidget(groupBoxNumberScroll, 4, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_5->addItem(verticalSpacer_2, 6, 0, 1, 1);

        groupBoxAutocomplete = new QGroupBox(scrollAreaWidgetContents_2);
        groupBoxAutocomplete->setObjectName(QString::fromUtf8("groupBoxAutocomplete"));
        gridLayout1 = new QGridLayout(groupBoxAutocomplete);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setContentsMargins(-1, -1, 9, -1);
        checkBoxEnableAutocomplete = new QCheckBox(groupBoxAutocomplete);
        checkBoxEnableAutocomplete->setObjectName(QString::fromUtf8("checkBoxEnableAutocomplete"));

        gridLayout1->addWidget(checkBoxEnableAutocomplete, 0, 0, 1, 1);

        horizontalLayout_30 = new QHBoxLayout();
        horizontalLayout_30->setObjectName(QString::fromUtf8("horizontalLayout_30"));
        labelCharacterThreshold = new QLabel(groupBoxAutocomplete);
        labelCharacterThreshold->setObjectName(QString::fromUtf8("labelCharacterThreshold"));

        horizontalLayout_30->addWidget(labelCharacterThreshold);

        lineEditCharacterThreshold = new QLineEdit(groupBoxAutocomplete);
        lineEditCharacterThreshold->setObjectName(QString::fromUtf8("lineEditCharacterThreshold"));

        horizontalLayout_30->addWidget(lineEditCharacterThreshold);


        gridLayout1->addLayout(horizontalLayout_30, 1, 0, 1, 1);


        gridLayout_5->addWidget(groupBoxAutocomplete, 5, 0, 1, 1);

        groupBoxLineWrap = new QGroupBox(scrollAreaWidgetContents_2);
        groupBoxLineWrap->setObjectName(QString::fromUtf8("groupBoxLineWrap"));
        gridLayout_4 = new QGridLayout(groupBoxLineWrap);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        horizontalLayout_25 = new QHBoxLayout();
        horizontalLayout_25->setObjectName(QString::fromUtf8("horizontalLayout_25"));
        comboBoxLineWrap = new QComboBox(groupBoxLineWrap);
        comboBoxLineWrap->addItem(QString());
        comboBoxLineWrap->addItem(QString());
        comboBoxLineWrap->addItem(QString());
        comboBoxLineWrap->setObjectName(QString::fromUtf8("comboBoxLineWrap"));

        horizontalLayout_25->addWidget(comboBoxLineWrap);

        horizontalSpacer_25 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_25->addItem(horizontalSpacer_25);


        gridLayout_4->addLayout(horizontalLayout_25, 0, 1, 1, 1);

        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setObjectName(QString::fromUtf8("horizontalLayout_21"));
        horizontalSpacer_23 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_23);

        labelLineWrapIndentation = new QLabel(groupBoxLineWrap);
        labelLineWrapIndentation->setObjectName(QString::fromUtf8("labelLineWrapIndentation"));

        horizontalLayout_21->addWidget(labelLineWrapIndentation);


        gridLayout_4->addLayout(horizontalLayout_21, 1, 0, 1, 1);

        horizontalLayout_22 = new QHBoxLayout();
        horizontalLayout_22->setObjectName(QString::fromUtf8("horizontalLayout_22"));
        horizontalSpacer_24 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_22->addItem(horizontalSpacer_24);

        labelLineWrapVisualization = new QLabel(groupBoxLineWrap);
        labelLineWrapVisualization->setObjectName(QString::fromUtf8("labelLineWrapVisualization"));

        horizontalLayout_22->addWidget(labelLineWrapVisualization);


        gridLayout_4->addLayout(horizontalLayout_22, 2, 0, 1, 1);

        horizontalLayout_24 = new QHBoxLayout();
        horizontalLayout_24->setObjectName(QString::fromUtf8("horizontalLayout_24"));
        labelLineWrapIndentationStyle = new QLabel(groupBoxLineWrap);
        labelLineWrapIndentationStyle->setObjectName(QString::fromUtf8("labelLineWrapIndentationStyle"));

        horizontalLayout_24->addWidget(labelLineWrapIndentationStyle);

        comboBoxLineWrapIndentationStyle = new QComboBox(groupBoxLineWrap);
        comboBoxLineWrapIndentationStyle->addItem(QString());
        comboBoxLineWrapIndentationStyle->addItem(QString());
        comboBoxLineWrapIndentationStyle->addItem(QString());
        comboBoxLineWrapIndentationStyle->setObjectName(QString::fromUtf8("comboBoxLineWrapIndentationStyle"));

        horizontalLayout_24->addWidget(comboBoxLineWrapIndentationStyle);

        labelLineWrapIndentationIndent = new QLabel(groupBoxLineWrap);
        labelLineWrapIndentationIndent->setObjectName(QString::fromUtf8("labelLineWrapIndentationIndent"));

        horizontalLayout_24->addWidget(labelLineWrapIndentationIndent);

        spinBoxLineWrapIndentationIndent = new QSpinBox(groupBoxLineWrap);
        spinBoxLineWrapIndentationIndent->setObjectName(QString::fromUtf8("spinBoxLineWrapIndentationIndent"));
        spinBoxLineWrapIndentationIndent->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_24->addWidget(spinBoxLineWrapIndentationIndent);

        horizontalSpacer_26 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_24->addItem(horizontalSpacer_26);


        gridLayout_4->addLayout(horizontalLayout_24, 1, 1, 1, 1);

        horizontalLayout_23 = new QHBoxLayout();
        horizontalLayout_23->setObjectName(QString::fromUtf8("horizontalLayout_23"));
        labelLineWrapVisualizationStart = new QLabel(groupBoxLineWrap);
        labelLineWrapVisualizationStart->setObjectName(QString::fromUtf8("labelLineWrapVisualizationStart"));

        horizontalLayout_23->addWidget(labelLineWrapVisualizationStart);

        comboBoxLineWrapVisualizationStart = new QComboBox(groupBoxLineWrap);
        comboBoxLineWrapVisualizationStart->addItem(QString());
        comboBoxLineWrapVisualizationStart->addItem(QString());
        comboBoxLineWrapVisualizationStart->addItem(QString());
        comboBoxLineWrapVisualizationStart->addItem(QString());
        comboBoxLineWrapVisualizationStart->setObjectName(QString::fromUtf8("comboBoxLineWrapVisualizationStart"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(comboBoxLineWrapVisualizationStart->sizePolicy().hasHeightForWidth());
        comboBoxLineWrapVisualizationStart->setSizePolicy(sizePolicy2);

        horizontalLayout_23->addWidget(comboBoxLineWrapVisualizationStart);

        labelLineWrapVisualizationEnd = new QLabel(groupBoxLineWrap);
        labelLineWrapVisualizationEnd->setObjectName(QString::fromUtf8("labelLineWrapVisualizationEnd"));

        horizontalLayout_23->addWidget(labelLineWrapVisualizationEnd);

        comboBoxLineWrapVisualizationEnd = new QComboBox(groupBoxLineWrap);
        comboBoxLineWrapVisualizationEnd->addItem(QString());
        comboBoxLineWrapVisualizationEnd->addItem(QString());
        comboBoxLineWrapVisualizationEnd->addItem(QString());
        comboBoxLineWrapVisualizationEnd->addItem(QString());
        comboBoxLineWrapVisualizationEnd->setObjectName(QString::fromUtf8("comboBoxLineWrapVisualizationEnd"));
        sizePolicy2.setHeightForWidth(comboBoxLineWrapVisualizationEnd->sizePolicy().hasHeightForWidth());
        comboBoxLineWrapVisualizationEnd->setSizePolicy(sizePolicy2);

        horizontalLayout_23->addWidget(comboBoxLineWrapVisualizationEnd);

        horizontalSpacer_27 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_23->addItem(horizontalSpacer_27);


        gridLayout_4->addLayout(horizontalLayout_23, 2, 1, 1, 1);

        horizontalLayout_20 = new QHBoxLayout();
        horizontalLayout_20->setObjectName(QString::fromUtf8("horizontalLayout_20"));
        horizontalSpacer_22 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_22);

        labelLineWrap = new QLabel(groupBoxLineWrap);
        labelLineWrap->setObjectName(QString::fromUtf8("labelLineWrap"));

        horizontalLayout_20->addWidget(labelLineWrap);


        gridLayout_4->addLayout(horizontalLayout_20, 0, 0, 1, 1);


        gridLayout_5->addWidget(groupBoxLineWrap, 2, 0, 1, 1);

        groupBoxDisplay = new QGroupBox(scrollAreaWidgetContents_2);
        groupBoxDisplay->setObjectName(QString::fromUtf8("groupBoxDisplay"));
        gridLayout = new QGridLayout(groupBoxDisplay);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        checkBoxHighlightCurrentLine = new QCheckBox(groupBoxDisplay);
        checkBoxHighlightCurrentLine->setObjectName(QString::fromUtf8("checkBoxHighlightCurrentLine"));

        gridLayout->addWidget(checkBoxHighlightCurrentLine, 1, 0, 1, 1);

        checkBoxEnableLineNumbers = new QCheckBox(groupBoxDisplay);
        checkBoxEnableLineNumbers->setObjectName(QString::fromUtf8("checkBoxEnableLineNumbers"));

        gridLayout->addWidget(checkBoxEnableLineNumbers, 3, 0, 1, 1);

        checkBoxEnableBraceMatching = new QCheckBox(groupBoxDisplay);
        checkBoxEnableBraceMatching->setObjectName(QString::fromUtf8("checkBoxEnableBraceMatching"));

        gridLayout->addWidget(checkBoxEnableBraceMatching, 2, 0, 1, 1);


        gridLayout_5->addWidget(groupBoxDisplay, 3, 0, 1, 1);

        gridLayout_9 = new QGridLayout();
        gridLayout_9->setObjectName(QString::fromUtf8("gridLayout_9"));
        horizontalLayout_28 = new QHBoxLayout();
        horizontalLayout_28->setObjectName(QString::fromUtf8("horizontalLayout_28"));
        horizontalSpacer_29 = new QSpacerItem(30, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_28->addItem(horizontalSpacer_29);

        label_3 = new QLabel(scrollAreaWidgetContents_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setScaledContents(false);

        horizontalLayout_28->addWidget(label_3);


        gridLayout_9->addLayout(horizontalLayout_28, 0, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        fontChooser = new QFontComboBox(scrollAreaWidgetContents_2);
        fontChooser->setObjectName(QString::fromUtf8("fontChooser"));
        QFont font;
        font.setFamily(QString::fromUtf8("DejaVu Sans"));
        font.setPointSize(12);
        fontChooser->setCurrentFont(font);

        horizontalLayout_5->addWidget(fontChooser);

        fontSize = new QComboBox(scrollAreaWidgetContents_2);
        fontSize->setObjectName(QString::fromUtf8("fontSize"));
        fontSize->setEditable(true);

        horizontalLayout_5->addWidget(fontSize);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);


        gridLayout_9->addLayout(horizontalLayout_5, 0, 1, 1, 1);

        horizontalLayout_29 = new QHBoxLayout();
        horizontalLayout_29->setObjectName(QString::fromUtf8("horizontalLayout_29"));
        horizontalSpacer_30 = new QSpacerItem(30, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_29->addItem(horizontalSpacer_30);

        labelColorSyntaxHighlighting = new QLabel(scrollAreaWidgetContents_2);
        labelColorSyntaxHighlighting->setObjectName(QString::fromUtf8("labelColorSyntaxHighlighting"));

        horizontalLayout_29->addWidget(labelColorSyntaxHighlighting);


        gridLayout_9->addLayout(horizontalLayout_29, 1, 0, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(198, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_9->addItem(horizontalSpacer_6, 2, 0, 1, 1);

        mouseWheelZoomBox = new QCheckBox(scrollAreaWidgetContents_2);
        mouseWheelZoomBox->setObjectName(QString::fromUtf8("mouseWheelZoomBox"));

        gridLayout_9->addWidget(mouseWheelZoomBox, 2, 1, 1, 1);

        horizontalLayout_26 = new QHBoxLayout();
        horizontalLayout_26->setObjectName(QString::fromUtf8("horizontalLayout_26"));
        syntaxHighlight = new QComboBox(scrollAreaWidgetContents_2);
        syntaxHighlight->setObjectName(QString::fromUtf8("syntaxHighlight"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(syntaxHighlight->sizePolicy().hasHeightForWidth());
        syntaxHighlight->setSizePolicy(sizePolicy3);

        horizontalLayout_26->addWidget(syntaxHighlight);

        horizontalSpacer_3 = new QSpacerItem(30, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_26->addItem(horizontalSpacer_3);


        gridLayout_9->addLayout(horizontalLayout_26, 1, 1, 1, 1);


        gridLayout_5->addLayout(gridLayout_9, 0, 0, 1, 1);

        groupBoxIndentation = new QGroupBox(scrollAreaWidgetContents_2);
        groupBoxIndentation->setObjectName(QString::fromUtf8("groupBoxIndentation"));
        gridLayout_3 = new QGridLayout(groupBoxIndentation);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName(QString::fromUtf8("horizontalLayout_16"));
        spinBoxIndentationWidth = new QSpinBox(groupBoxIndentation);
        spinBoxIndentationWidth->setObjectName(QString::fromUtf8("spinBoxIndentationWidth"));
        spinBoxIndentationWidth->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxIndentationWidth->setMaximum(99);
        spinBoxIndentationWidth->setValue(4);

        horizontalLayout_16->addWidget(spinBoxIndentationWidth);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_11);


        gridLayout_3->addLayout(horizontalLayout_16, 3, 1, 1, 1);

        horizontalSpacer_13 = new QSpacerItem(223, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_13, 0, 0, 1, 1);

        checkBoxAutoIndent = new QCheckBox(groupBoxIndentation);
        checkBoxAutoIndent->setObjectName(QString::fromUtf8("checkBoxAutoIndent"));
        checkBoxAutoIndent->setChecked(true);

        gridLayout_3->addWidget(checkBoxAutoIndent, 0, 1, 1, 1);

        horizontalSpacer_32 = new QSpacerItem(223, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_32, 1, 0, 1, 1);

        checkBoxBackspaceUnindents = new QCheckBox(groupBoxIndentation);
        checkBoxBackspaceUnindents->setObjectName(QString::fromUtf8("checkBoxBackspaceUnindents"));
        checkBoxBackspaceUnindents->setChecked(false);

        gridLayout_3->addWidget(checkBoxBackspaceUnindents, 1, 1, 1, 1);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_8);

        label_10 = new QLabel(groupBoxIndentation);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        horizontalLayout_7->addWidget(label_10);


        gridLayout_3->addLayout(horizontalLayout_7, 2, 0, 1, 1);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName(QString::fromUtf8("horizontalLayout_15"));
        comboBoxIndentUsing = new QComboBox(groupBoxIndentation);
        comboBoxIndentUsing->addItem(QString());
        comboBoxIndentUsing->addItem(QString());
        comboBoxIndentUsing->setObjectName(QString::fromUtf8("comboBoxIndentUsing"));

        horizontalLayout_15->addWidget(comboBoxIndentUsing);

        horizontalSpacer_14 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_14);


        gridLayout_3->addLayout(horizontalLayout_15, 2, 1, 1, 1);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_15);

        labelIndentationWidth = new QLabel(groupBoxIndentation);
        labelIndentationWidth->setObjectName(QString::fromUtf8("labelIndentationWidth"));

        horizontalLayout_8->addWidget(labelIndentationWidth);


        gridLayout_3->addLayout(horizontalLayout_8, 3, 0, 1, 1);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        horizontalSpacer_16 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_16);

        labelTabWidth = new QLabel(groupBoxIndentation);
        labelTabWidth->setObjectName(QString::fromUtf8("labelTabWidth"));

        horizontalLayout_12->addWidget(labelTabWidth);


        gridLayout_3->addLayout(horizontalLayout_12, 4, 0, 1, 1);

        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName(QString::fromUtf8("horizontalLayout_17"));
        spinBoxTabWidth = new QSpinBox(groupBoxIndentation);
        spinBoxTabWidth->setObjectName(QString::fromUtf8("spinBoxTabWidth"));
        spinBoxTabWidth->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxTabWidth->setValue(4);

        horizontalLayout_17->addWidget(spinBoxTabWidth);

        horizontalSpacer_17 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_17);


        gridLayout_3->addLayout(horizontalLayout_17, 4, 1, 1, 1);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName(QString::fromUtf8("horizontalLayout_13"));
        horizontalSpacer_19 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_19);

        label_9 = new QLabel(groupBoxIndentation);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        horizontalLayout_13->addWidget(label_9);


        gridLayout_3->addLayout(horizontalLayout_13, 5, 0, 1, 1);

        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName(QString::fromUtf8("horizontalLayout_18"));
        comboBoxTabKeyFunction = new QComboBox(groupBoxIndentation);
        comboBoxTabKeyFunction->addItem(QString());
        comboBoxTabKeyFunction->addItem(QString());
        comboBoxTabKeyFunction->setObjectName(QString::fromUtf8("comboBoxTabKeyFunction"));

        horizontalLayout_18->addWidget(comboBoxTabKeyFunction);

        horizontalSpacer_18 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_18);


        gridLayout_3->addLayout(horizontalLayout_18, 5, 1, 1, 1);

        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName(QString::fromUtf8("horizontalLayout_14"));
        horizontalSpacer_20 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_20);

        labelWhitespaces = new QLabel(groupBoxIndentation);
        labelWhitespaces->setObjectName(QString::fromUtf8("labelWhitespaces"));

        horizontalLayout_14->addWidget(labelWhitespaces);


        gridLayout_3->addLayout(horizontalLayout_14, 6, 0, 1, 1);

        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setObjectName(QString::fromUtf8("horizontalLayout_19"));
        comboBoxShowWhitespace = new QComboBox(groupBoxIndentation);
        comboBoxShowWhitespace->addItem(QString());
        comboBoxShowWhitespace->addItem(QString());
        comboBoxShowWhitespace->addItem(QString());
        comboBoxShowWhitespace->setObjectName(QString::fromUtf8("comboBoxShowWhitespace"));

        horizontalLayout_19->addWidget(comboBoxShowWhitespace);

        labelShowWhitespacesSize = new QLabel(groupBoxIndentation);
        labelShowWhitespacesSize->setObjectName(QString::fromUtf8("labelShowWhitespacesSize"));

        horizontalLayout_19->addWidget(labelShowWhitespacesSize);

        spinBoxShowWhitespaceSize = new QSpinBox(groupBoxIndentation);
        spinBoxShowWhitespaceSize->setObjectName(QString::fromUtf8("spinBoxShowWhitespaceSize"));
        spinBoxShowWhitespaceSize->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxShowWhitespaceSize->setMinimum(1);
        spinBoxShowWhitespaceSize->setMaximum(9);

        horizontalLayout_19->addWidget(spinBoxShowWhitespaceSize);

        horizontalSpacer_21 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_21);


        gridLayout_3->addLayout(horizontalLayout_19, 6, 1, 1, 1);


        gridLayout_5->addWidget(groupBoxIndentation, 1, 0, 1, 1);

        scrollArea_2->setWidget(scrollAreaWidgetContents_2);

        verticalLayout->addWidget(scrollArea_2);

        stackedWidget->addWidget(pageEditor);
        pageUpdate = new QWidget();
        pageUpdate->setObjectName(QString::fromUtf8("pageUpdate"));
        verticalLayout_11 = new QVBoxLayout(pageUpdate);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        updateCheckBox = new QCheckBox(pageUpdate);
        updateCheckBox->setObjectName(QString::fromUtf8("updateCheckBox"));
        updateCheckBox->setChecked(true);

        verticalLayout_8->addWidget(updateCheckBox);

        snapshotCheckBox = new QCheckBox(pageUpdate);
        snapshotCheckBox->setObjectName(QString::fromUtf8("snapshotCheckBox"));

        verticalLayout_8->addWidget(snapshotCheckBox);


        horizontalLayout_9->addLayout(verticalLayout_8);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_2);


        verticalLayout_11->addLayout(horizontalLayout_9);

        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setSpacing(0);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_10);

        checkNowButton = new QPushButton(pageUpdate);
        checkNowButton->setObjectName(QString::fromUtf8("checkNowButton"));

        horizontalLayout_11->addWidget(checkNowButton);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_9);


        verticalLayout_7->addLayout(horizontalLayout_11);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        horizontalSpacer_31 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_31);

        lastCheckedLabel = new QLabel(pageUpdate);
        lastCheckedLabel->setObjectName(QString::fromUtf8("lastCheckedLabel"));
        lastCheckedLabel->setEnabled(false);
        QFont font1;
        font1.setPointSize(11);
        lastCheckedLabel->setFont(font1);

        horizontalLayout_10->addWidget(lastCheckedLabel);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_12);


        verticalLayout_7->addLayout(horizontalLayout_10);


        verticalLayout_11->addLayout(verticalLayout_7);

        verticalSpacer_7 = new QSpacerItem(20, 89, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_11->addItem(verticalSpacer_7);

        stackedWidget->addWidget(pageUpdate);
        pageFeatures = new QWidget();
        pageFeatures->setObjectName(QString::fromUtf8("pageFeatures"));
        gridLayout_2 = new QGridLayout(pageFeatures);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        labelFeatures = new QLabel(pageFeatures);
        labelFeatures->setObjectName(QString::fromUtf8("labelFeatures"));
        QFont font2;
        font2.setBold(true);
        font2.setWeight(75);
        labelFeatures->setFont(font2);

        verticalLayout_9->addWidget(labelFeatures);

        scrollArea = new QScrollArea(pageFeatures);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 84, 26));
        verticalLayout_10 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        gridLayoutExperimentalFeatures = new QGridLayout();
        gridLayoutExperimentalFeatures->setSpacing(8);
        gridLayoutExperimentalFeatures->setObjectName(QString::fromUtf8("gridLayoutExperimentalFeatures"));

        verticalLayout_10->addLayout(gridLayoutExperimentalFeatures);

        verticalSpacer_3 = new QSpacerItem(20, 282, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_3);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_9->addWidget(scrollArea);


        gridLayout_2->addLayout(verticalLayout_9, 0, 0, 1, 1);

        stackedWidget->addWidget(pageFeatures);
        page3DPrint = new QWidget();
        page3DPrint->setObjectName(QString::fromUtf8("page3DPrint"));
        verticalLayout_5 = new QVBoxLayout(page3DPrint);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        groupBoxOctoPrint = new QGroupBox(page3DPrint);
        groupBoxOctoPrint->setObjectName(QString::fromUtf8("groupBoxOctoPrint"));
        gridLayout_7 = new QGridLayout(groupBoxOctoPrint);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        lineEditOctoPrintURL = new QLineEdit(groupBoxOctoPrint);
        lineEditOctoPrintURL->setObjectName(QString::fromUtf8("lineEditOctoPrintURL"));
        lineEditOctoPrintURL->setPlaceholderText(QString::fromUtf8("http://octopi.local/api"));

        gridLayout_7->addWidget(lineEditOctoPrintURL, 0, 2, 1, 1);

        labelOctoPrintURL = new QLabel(groupBoxOctoPrint);
        labelOctoPrintURL->setObjectName(QString::fromUtf8("labelOctoPrintURL"));
        labelOctoPrintURL->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(labelOctoPrintURL, 0, 0, 1, 1);

        labelOctoPrintApiKey = new QLabel(groupBoxOctoPrint);
        labelOctoPrintApiKey->setObjectName(QString::fromUtf8("labelOctoPrintApiKey"));
        labelOctoPrintApiKey->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(labelOctoPrintApiKey, 1, 0, 1, 1);

        horizontalLayoutOctoPrintSlicingEngine = new QHBoxLayout();
        horizontalLayoutOctoPrintSlicingEngine->setObjectName(QString::fromUtf8("horizontalLayoutOctoPrintSlicingEngine"));
        comboBoxOctoPrintSlicingEngine = new QComboBox(groupBoxOctoPrint);
        comboBoxOctoPrintSlicingEngine->setObjectName(QString::fromUtf8("comboBoxOctoPrintSlicingEngine"));
        comboBoxOctoPrintSlicingEngine->setMinimumSize(QSize(160, 0));

        horizontalLayoutOctoPrintSlicingEngine->addWidget(comboBoxOctoPrintSlicingEngine);

        horizontalSpacerOctoPrintSlicingEngine = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayoutOctoPrintSlicingEngine->addItem(horizontalSpacerOctoPrintSlicingEngine);

        pushButtonOctoPrintSlicingEngine = new QPushButton(groupBoxOctoPrint);
        pushButtonOctoPrintSlicingEngine->setObjectName(QString::fromUtf8("pushButtonOctoPrintSlicingEngine"));

        horizontalLayoutOctoPrintSlicingEngine->addWidget(pushButtonOctoPrintSlicingEngine);


        gridLayout_7->addLayout(horizontalLayoutOctoPrintSlicingEngine, 7, 2, 1, 1);

        horizontalLayoutOctoPrintSlicingProfile = new QHBoxLayout();
        horizontalLayoutOctoPrintSlicingProfile->setObjectName(QString::fromUtf8("horizontalLayoutOctoPrintSlicingProfile"));
        comboBoxOctoPrintSlicingProfile = new QComboBox(groupBoxOctoPrint);
        comboBoxOctoPrintSlicingProfile->setObjectName(QString::fromUtf8("comboBoxOctoPrintSlicingProfile"));
        comboBoxOctoPrintSlicingProfile->setMinimumSize(QSize(160, 0));

        horizontalLayoutOctoPrintSlicingProfile->addWidget(comboBoxOctoPrintSlicingProfile);

        horizontalSpacerOctoPrintSlicingProfile = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayoutOctoPrintSlicingProfile->addItem(horizontalSpacerOctoPrintSlicingProfile);

        pushButtonOctoPrintSlicingProfile = new QPushButton(groupBoxOctoPrint);
        pushButtonOctoPrintSlicingProfile->setObjectName(QString::fromUtf8("pushButtonOctoPrintSlicingProfile"));

        horizontalLayoutOctoPrintSlicingProfile->addWidget(pushButtonOctoPrintSlicingProfile);


        gridLayout_7->addLayout(horizontalLayoutOctoPrintSlicingProfile, 8, 2, 1, 1);

        horizontalLayoutOctoPrintCheckConnection = new QHBoxLayout();
        horizontalLayoutOctoPrintCheckConnection->setObjectName(QString::fromUtf8("horizontalLayoutOctoPrintCheckConnection"));
        horizontalSpacerOctoPrintCheckConnection = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayoutOctoPrintCheckConnection->addItem(horizontalSpacerOctoPrintCheckConnection);

        labelOctoPrintCheckConnection = new QLabel(groupBoxOctoPrint);
        labelOctoPrintCheckConnection->setObjectName(QString::fromUtf8("labelOctoPrintCheckConnection"));

        horizontalLayoutOctoPrintCheckConnection->addWidget(labelOctoPrintCheckConnection);

        pushButtonOctoPrintCheckConnection = new QPushButton(groupBoxOctoPrint);
        pushButtonOctoPrintCheckConnection->setObjectName(QString::fromUtf8("pushButtonOctoPrintCheckConnection"));

        horizontalLayoutOctoPrintCheckConnection->addWidget(pushButtonOctoPrintCheckConnection);


        gridLayout_7->addLayout(horizontalLayoutOctoPrintCheckConnection, 4, 2, 1, 1);

        labelOctoPrintSlicingProfile = new QLabel(groupBoxOctoPrint);
        labelOctoPrintSlicingProfile->setObjectName(QString::fromUtf8("labelOctoPrintSlicingProfile"));
        labelOctoPrintSlicingProfile->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(labelOctoPrintSlicingProfile, 8, 0, 1, 1);

        labelOctoPrintSlicingEngine = new QLabel(groupBoxOctoPrint);
        labelOctoPrintSlicingEngine->setObjectName(QString::fromUtf8("labelOctoPrintSlicingEngine"));
        labelOctoPrintSlicingEngine->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(labelOctoPrintSlicingEngine, 7, 0, 1, 1);

        labelOctoPrintFileFormat = new QLabel(groupBoxOctoPrint);
        labelOctoPrintFileFormat->setObjectName(QString::fromUtf8("labelOctoPrintFileFormat"));
        labelOctoPrintFileFormat->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(labelOctoPrintFileFormat, 5, 0, 1, 1);

        horizontalLayoutOctoPrintFileFormat = new QHBoxLayout();
        horizontalLayoutOctoPrintFileFormat->setObjectName(QString::fromUtf8("horizontalLayoutOctoPrintFileFormat"));
        comboBoxOctoPrintFileFormat = new QComboBox(groupBoxOctoPrint);
        comboBoxOctoPrintFileFormat->setObjectName(QString::fromUtf8("comboBoxOctoPrintFileFormat"));

        horizontalLayoutOctoPrintFileFormat->addWidget(comboBoxOctoPrintFileFormat);

        horizontalSpacerOctoPrintFileFormat = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayoutOctoPrintFileFormat->addItem(horizontalSpacerOctoPrintFileFormat);


        gridLayout_7->addLayout(horizontalLayoutOctoPrintFileFormat, 5, 2, 1, 1);

        labelOctoPrintAction = new QLabel(groupBoxOctoPrint);
        labelOctoPrintAction->setObjectName(QString::fromUtf8("labelOctoPrintAction"));
        labelOctoPrintAction->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(labelOctoPrintAction, 6, 0, 1, 1);

        horizontalLayoutOctoPrintSliceAction = new QHBoxLayout();
        horizontalLayoutOctoPrintSliceAction->setObjectName(QString::fromUtf8("horizontalLayoutOctoPrintSliceAction"));
        comboBoxOctoPrintAction = new QComboBox(groupBoxOctoPrint);
        comboBoxOctoPrintAction->setObjectName(QString::fromUtf8("comboBoxOctoPrintAction"));

        horizontalLayoutOctoPrintSliceAction->addWidget(comboBoxOctoPrintAction);

        horizontalSpacerOctoPrintSliceAction = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayoutOctoPrintSliceAction->addItem(horizontalSpacerOctoPrintSliceAction);


        gridLayout_7->addLayout(horizontalLayoutOctoPrintSliceAction, 6, 2, 1, 1);

        horizontalLayoutApiKey = new QHBoxLayout();
        horizontalLayoutApiKey->setObjectName(QString::fromUtf8("horizontalLayoutApiKey"));
        lineEditOctoPrintApiKey = new QLineEdit(groupBoxOctoPrint);
        lineEditOctoPrintApiKey->setObjectName(QString::fromUtf8("lineEditOctoPrintApiKey"));
        lineEditOctoPrintApiKey->setEchoMode(QLineEdit::PasswordEchoOnEdit);

        horizontalLayoutApiKey->addWidget(lineEditOctoPrintApiKey);

        pushButtonOctoPrintApiKey = new QPushButton(groupBoxOctoPrint);
        pushButtonOctoPrintApiKey->setObjectName(QString::fromUtf8("pushButtonOctoPrintApiKey"));
        pushButtonOctoPrintApiKey->setCheckable(true);

        horizontalLayoutApiKey->addWidget(pushButtonOctoPrintApiKey);


        gridLayout_7->addLayout(horizontalLayoutApiKey, 1, 2, 1, 1);

        labelOctoPrintApiKeyNote = new QLabel(groupBoxOctoPrint);
        labelOctoPrintApiKeyNote->setObjectName(QString::fromUtf8("labelOctoPrintApiKeyNote"));

        gridLayout_7->addWidget(labelOctoPrintApiKeyNote, 2, 2, 1, 1);


        verticalLayout_5->addWidget(groupBoxOctoPrint);

        verticalSpacer3DPrint = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_5->addItem(verticalSpacer3DPrint);

        stackedWidget->addWidget(page3DPrint);
        pageAdvanced = new QWidget();
        pageAdvanced->setObjectName(QString::fromUtf8("pageAdvanced"));
        verticalLayout_3 = new QVBoxLayout(pageAdvanced);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        scrollAreaAdvanced = new QScrollArea(pageAdvanced);
        scrollAreaAdvanced->setObjectName(QString::fromUtf8("scrollAreaAdvanced"));
        scrollAreaAdvanced->setFrameShadow(QFrame::Plain);
        scrollAreaAdvanced->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        scrollAreaAdvanced->setWidgetResizable(true);
        scrollAreaAdvancedWidgetContents = new QWidget();
        scrollAreaAdvancedWidgetContents->setObjectName(QString::fromUtf8("scrollAreaAdvancedWidgetContents"));
        scrollAreaAdvancedWidgetContents->setGeometry(QRect(0, 0, 548, 747));
        verticalLayout_12 = new QVBoxLayout(scrollAreaAdvancedWidgetContents);
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        verticalLayout_12->setContentsMargins(9, 9, 9, -1);
        groupBox_CSG = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupBox_CSG->setObjectName(QString::fromUtf8("groupBox_CSG"));
        verticalLayout_6 = new QVBoxLayout(groupBox_CSG);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        openCSGWarningBox = new QCheckBox(groupBox_CSG);
        openCSGWarningBox->setObjectName(QString::fromUtf8("openCSGWarningBox"));
        openCSGWarningBox->setChecked(true);

        verticalLayout_6->addWidget(openCSGWarningBox);

        enableOpenCSGBox = new QCheckBox(groupBox_CSG);
        enableOpenCSGBox->setObjectName(QString::fromUtf8("enableOpenCSGBox"));

        verticalLayout_6->addWidget(enableOpenCSGBox);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        label_7 = new QLabel(groupBox_CSG);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        horizontalLayout_4->addWidget(label_7);

        opencsgLimitEdit = new QLineEdit(groupBox_CSG);
        opencsgLimitEdit->setObjectName(QString::fromUtf8("opencsgLimitEdit"));

        horizontalLayout_4->addWidget(opencsgLimitEdit);

        label_8 = new QLabel(groupBox_CSG);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        horizontalLayout_4->addWidget(label_8);


        verticalLayout_6->addLayout(horizontalLayout_4);

        forceGoldfeatherBox = new QCheckBox(groupBox_CSG);
        forceGoldfeatherBox->setObjectName(QString::fromUtf8("forceGoldfeatherBox"));

        verticalLayout_6->addWidget(forceGoldfeatherBox);


        verticalLayout_12->addWidget(groupBox_CSG);

        groupBox_CGAL = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupBox_CGAL->setObjectName(QString::fromUtf8("groupBox_CGAL"));
        verticalLayout_14 = new QVBoxLayout(groupBox_CGAL);
        verticalLayout_14->setObjectName(QString::fromUtf8("verticalLayout_14"));
        horizontalLayout_cgalCacheSizeMB = new QHBoxLayout();
        horizontalLayout_cgalCacheSizeMB->setObjectName(QString::fromUtf8("horizontalLayout_cgalCacheSizeMB"));
        label_4 = new QLabel(groupBox_CGAL);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        horizontalLayout_cgalCacheSizeMB->addWidget(label_4);

        cgalCacheSizeMBEdit = new QLineEdit(groupBox_CGAL);
        cgalCacheSizeMBEdit->setObjectName(QString::fromUtf8("cgalCacheSizeMBEdit"));

        horizontalLayout_cgalCacheSizeMB->addWidget(cgalCacheSizeMBEdit);

        label_2 = new QLabel(groupBox_CGAL);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout_cgalCacheSizeMB->addWidget(label_2);


        verticalLayout_14->addLayout(horizontalLayout_cgalCacheSizeMB);

        horizontalLayout_polysetCacheSizeMB = new QHBoxLayout();
        horizontalLayout_polysetCacheSizeMB->setObjectName(QString::fromUtf8("horizontalLayout_polysetCacheSizeMB"));
        horizontalLayout_polysetCacheSizeMB->setContentsMargins(-1, -1, -1, 0);
        label_5 = new QLabel(groupBox_CGAL);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        horizontalLayout_polysetCacheSizeMB->addWidget(label_5);

        polysetCacheSizeMBEdit = new QLineEdit(groupBox_CGAL);
        polysetCacheSizeMBEdit->setObjectName(QString::fromUtf8("polysetCacheSizeMBEdit"));

        horizontalLayout_polysetCacheSizeMB->addWidget(polysetCacheSizeMBEdit);

        label_6 = new QLabel(groupBox_CGAL);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        horizontalLayout_polysetCacheSizeMB->addWidget(label_6);


        verticalLayout_14->addLayout(horizontalLayout_polysetCacheSizeMB);


        verticalLayout_12->addWidget(groupBox_CGAL);

        groupBox_UI = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupBox_UI->setObjectName(QString::fromUtf8("groupBox_UI"));
        verticalLayout_13 = new QVBoxLayout(groupBox_UI);
        verticalLayout_13->setObjectName(QString::fromUtf8("verticalLayout_13"));
        reorderCheckBox = new QCheckBox(groupBox_UI);
        reorderCheckBox->setObjectName(QString::fromUtf8("reorderCheckBox"));

        verticalLayout_13->addWidget(reorderCheckBox);

        undockCheckBox = new QCheckBox(groupBox_UI);
        undockCheckBox->setObjectName(QString::fromUtf8("undockCheckBox"));

        verticalLayout_13->addWidget(undockCheckBox);

        launcherBox = new QCheckBox(groupBox_UI);
        launcherBox->setObjectName(QString::fromUtf8("launcherBox"));

        verticalLayout_13->addWidget(launcherBox);

        localizationCheckBox = new QCheckBox(groupBox_UI);
        localizationCheckBox->setObjectName(QString::fromUtf8("localizationCheckBox"));
        localizationCheckBox->setChecked(true);

        verticalLayout_13->addWidget(localizationCheckBox);

        autoReloadRaiseCheckBox = new QCheckBox(groupBox_UI);
        autoReloadRaiseCheckBox->setObjectName(QString::fromUtf8("autoReloadRaiseCheckBox"));
        autoReloadRaiseCheckBox->setChecked(false);

        verticalLayout_13->addWidget(autoReloadRaiseCheckBox);

        enableSoundOnRenderCompleteCheckBox = new QCheckBox(groupBox_UI);
        enableSoundOnRenderCompleteCheckBox->setObjectName(QString::fromUtf8("enableSoundOnRenderCompleteCheckBox"));
        enableSoundOnRenderCompleteCheckBox->setChecked(false);

        verticalLayout_13->addWidget(enableSoundOnRenderCompleteCheckBox);

        horizontalLayout_timeThresholdOnRenderCompleteSound = new QHBoxLayout();
        horizontalLayout_timeThresholdOnRenderCompleteSound->setObjectName(QString::fromUtf8("horizontalLayout_timeThresholdOnRenderCompleteSound"));
        horizontalLayout_timeThresholdOnRenderCompleteSound->setContentsMargins(40, 0, 30, 0);
        timeThresholdOnRenderCompleteSoundLabel = new QLabel(groupBox_UI);
        timeThresholdOnRenderCompleteSoundLabel->setObjectName(QString::fromUtf8("timeThresholdOnRenderCompleteSoundLabel"));

        horizontalLayout_timeThresholdOnRenderCompleteSound->addWidget(timeThresholdOnRenderCompleteSoundLabel);

        timeThresholdOnRenderCompleteSoundEdit = new QLineEdit(groupBox_UI);
        timeThresholdOnRenderCompleteSoundEdit->setObjectName(QString::fromUtf8("timeThresholdOnRenderCompleteSoundEdit"));

        horizontalLayout_timeThresholdOnRenderCompleteSound->addWidget(timeThresholdOnRenderCompleteSoundEdit);

        secLabel = new QLabel(groupBox_UI);
        secLabel->setObjectName(QString::fromUtf8("secLabel"));

        horizontalLayout_timeThresholdOnRenderCompleteSound->addWidget(secLabel);


        verticalLayout_13->addLayout(horizontalLayout_timeThresholdOnRenderCompleteSound);


        verticalLayout_12->addWidget(groupBox_UI);

        groupBox_lang = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupBox_lang->setObjectName(QString::fromUtf8("groupBox_lang"));
        verticalLayout_15 = new QVBoxLayout(groupBox_lang);
        verticalLayout_15->setObjectName(QString::fromUtf8("verticalLayout_15"));
        enableParameterCheckBox = new QCheckBox(groupBox_lang);
        enableParameterCheckBox->setObjectName(QString::fromUtf8("enableParameterCheckBox"));
        enableParameterCheckBox->setChecked(false);

        verticalLayout_15->addWidget(enableParameterCheckBox);

        enableHardwarningsCheckBox = new QCheckBox(groupBox_lang);
        enableHardwarningsCheckBox->setObjectName(QString::fromUtf8("enableHardwarningsCheckBox"));
        enableHardwarningsCheckBox->setChecked(false);

        verticalLayout_15->addWidget(enableHardwarningsCheckBox);

        enableRangeCheckBox = new QCheckBox(groupBox_lang);
        enableRangeCheckBox->setObjectName(QString::fromUtf8("enableRangeCheckBox"));
        enableRangeCheckBox->setChecked(false);

        verticalLayout_15->addWidget(enableRangeCheckBox);


        verticalLayout_12->addWidget(groupBox_lang);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_12->addItem(verticalSpacer);

        groupBox_Export = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupBox_Export->setObjectName(QString::fromUtf8("groupBox_Export"));
        verticalLayout_16 = new QVBoxLayout(groupBox_Export);
        verticalLayout_16->setObjectName(QString::fromUtf8("verticalLayout_16"));
        useAsciiSTLCheckBox = new QCheckBox(groupBox_Export);
        useAsciiSTLCheckBox->setObjectName(QString::fromUtf8("useAsciiSTLCheckBox"));

        verticalLayout_16->addWidget(useAsciiSTLCheckBox);


        verticalLayout_12->addWidget(groupBox_Export);

        verticalSpacer1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_12->addItem(verticalSpacer1);

        groupBox_Debug = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupBox_Debug->setObjectName(QString::fromUtf8("groupBox_Debug"));
        verticalLayout_161 = new QVBoxLayout(groupBox_Debug);
        verticalLayout_161->setObjectName(QString::fromUtf8("verticalLayout_161"));
        enableHidapiTraceCheckBox = new QCheckBox(groupBox_Debug);
        enableHidapiTraceCheckBox->setObjectName(QString::fromUtf8("enableHidapiTraceCheckBox"));

        verticalLayout_161->addWidget(enableHidapiTraceCheckBox);


        verticalLayout_12->addWidget(groupBox_Debug);

        scrollAreaAdvanced->setWidget(scrollAreaAdvancedWidgetContents);

        verticalLayout_3->addWidget(scrollAreaAdvanced);

        stackedWidget->addWidget(pageAdvanced);
        pageInput = new QWidget();
        pageInput->setObjectName(QString::fromUtf8("pageInput"));
        gridLayout_13 = new QGridLayout(pageInput);
        gridLayout_13->setObjectName(QString::fromUtf8("gridLayout_13"));
        gridLayout_13->setContentsMargins(0, 0, 0, 0);
        scrollArea_InputAxis = new QScrollArea(pageInput);
        scrollArea_InputAxis->setObjectName(QString::fromUtf8("scrollArea_InputAxis"));
        scrollArea_InputAxis->setWidgetResizable(true);
        AxisConfig = new AxisConfigWidget();
        AxisConfig->setObjectName(QString::fromUtf8("AxisConfig"));
        AxisConfig->setGeometry(QRect(0, 0, 98, 28));
        scrollArea_InputAxis->setWidget(AxisConfig);

        gridLayout_13->addWidget(scrollArea_InputAxis, 0, 0, 1, 1);

        stackedWidget->addWidget(pageInput);
        pageInputButton = new QWidget();
        pageInputButton->setObjectName(QString::fromUtf8("pageInputButton"));
        gridLayout_14 = new QGridLayout(pageInputButton);
        gridLayout_14->setObjectName(QString::fromUtf8("gridLayout_14"));
        gridLayout_14->setContentsMargins(0, 0, 0, 0);
        scrollArea_InputButton = new QScrollArea(pageInputButton);
        scrollArea_InputButton->setObjectName(QString::fromUtf8("scrollArea_InputButton"));
        scrollArea_InputButton->setWidgetResizable(true);
        ButtonConfig = new ButtonConfigWidget();
        ButtonConfig->setObjectName(QString::fromUtf8("ButtonConfig"));
        ButtonConfig->setGeometry(QRect(0, 0, 98, 28));
        scrollArea_InputButton->setWidget(ButtonConfig);

        gridLayout_14->addWidget(scrollArea_InputButton, 3, 1, 1, 1);

        stackedWidget->addWidget(pageInputButton);

        gridLayout_6->addWidget(stackedWidget, 0, 0, 1, 1);

        Preferences->setCentralWidget(centralwidget);
        toolBar = new QToolBar(Preferences);
        toolBar->setObjectName(QString::fromUtf8("toolBar"));
        toolBar->setMovable(false);
        toolBar->setAllowedAreas(Qt::TopToolBarArea);
        toolBar->setIconSize(QSize(32, 32));
        toolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolBar->setFloatable(false);
        Preferences->addToolBar(Qt::TopToolBarArea, toolBar);

        toolBar->addAction(prefsAction3DView);
        toolBar->addAction(prefsActionEditor);
        toolBar->addAction(prefsActionUpdate);
        toolBar->addAction(prefsActionFeatures);
        toolBar->addAction(prefsAction3DPrint);
        toolBar->addAction(prefsActionAdvanced);
        toolBar->addAction(prefsActionInput);
        toolBar->addAction(prefsActionInputButton);

        retranslateUi(Preferences);

        stackedWidget->setCurrentIndex(1);
        colorSchemeChooser->setCurrentRow(-1);


        QMetaObject::connectSlotsByName(Preferences);
    } // setupUi

    void retranslateUi(QMainWindow *Preferences)
    {
        Preferences->setWindowTitle(q_("Preferences", nullptr));
        prefsAction3DView->setText(q_("3D View", nullptr));
        prefsActionAdvanced->setText(q_("Advanced", "preferences"));
        prefsActionEditor->setText(q_("Editor", nullptr));
        prefsActionUpdate->setText(q_("Update", nullptr));
        prefsActionFeatures->setText(q_("Features", nullptr));
#if QT_CONFIG(tooltip)
        prefsActionFeatures->setToolTip(q_("Enable/Disable experimental features", nullptr));
#endif // QT_CONFIG(tooltip)
        prefsActionInput->setText(q_("Axes", nullptr));
#if QT_CONFIG(tooltip)
        prefsActionInput->setToolTip(q_("Input driver configuration and Axis mapping", nullptr));
#endif // QT_CONFIG(tooltip)
        prefsActionInputButton->setText(q_("Buttons", nullptr));
        prefsAction3DPrint->setText(q_("3D Print", nullptr));
#if QT_CONFIG(tooltip)
        prefsAction3DPrint->setToolTip(q_("3D Printing Services", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(q_("Color scheme:", nullptr));
        checkBoxShowWarningsIn3dView->setText(q_("Show Warnings and Errors in 3D View", nullptr));
        checkBoxMouseCentricZoom->setText(q_("mouse centric zoom", nullptr));
        groupBoxNumberScroll->setTitle(q_("Number Scroll", nullptr));
        comboBoxModifierNumberScrollWheel->setItemText(0, q_("Alt", nullptr));
        comboBoxModifierNumberScrollWheel->setItemText(1, q_("Left Mouse Button", nullptr));
        comboBoxModifierNumberScrollWheel->setItemText(2, q_("Either", nullptr));

        labelStepSize->setText(q_("Step Size", nullptr));
        checkBoxEnableNumberScrollWheel->setText(q_("Number Scroll Via Mouse Wheel", nullptr));
        labelModifierNumberScrollWheel->setText(q_("Modifier For Wheel Scroll ", nullptr));
        groupBoxAutocomplete->setTitle(q_("Autocomplete", nullptr));
        checkBoxEnableAutocomplete->setText(q_("Enable Autocompletion", nullptr));
        labelCharacterThreshold->setText(q_("Character Threshold", nullptr));
        groupBoxLineWrap->setTitle(q_("Line wrap", nullptr));
        comboBoxLineWrap->setItemText(0, q_("None", nullptr));
        comboBoxLineWrap->setItemText(1, q_("Wrap at character boundaries", nullptr));
        comboBoxLineWrap->setItemText(2, q_("Wrap at word boundaries", nullptr));

        labelLineWrapIndentation->setText(q_("Line wrap indentation", nullptr));
        labelLineWrapVisualization->setText(q_("Line wrap visualization", nullptr));
        labelLineWrapIndentationStyle->setText(q_("Style", nullptr));
        comboBoxLineWrapIndentationStyle->setItemText(0, q_("Fixed", nullptr));
        comboBoxLineWrapIndentationStyle->setItemText(1, q_("Same", nullptr));
        comboBoxLineWrapIndentationStyle->setItemText(2, q_("Indented", nullptr));

        labelLineWrapIndentationIndent->setText(q_("Indent", nullptr));
        labelLineWrapVisualizationStart->setText(q_("Start", nullptr));
        comboBoxLineWrapVisualizationStart->setItemText(0, q_("None", nullptr));
        comboBoxLineWrapVisualizationStart->setItemText(1, q_("Text", nullptr));
        comboBoxLineWrapVisualizationStart->setItemText(2, q_("Border", nullptr));
        comboBoxLineWrapVisualizationStart->setItemText(3, q_("Margin", nullptr));

        labelLineWrapVisualizationEnd->setText(q_("End", nullptr));
        comboBoxLineWrapVisualizationEnd->setItemText(0, q_("None", nullptr));
        comboBoxLineWrapVisualizationEnd->setItemText(1, q_("Text", nullptr));
        comboBoxLineWrapVisualizationEnd->setItemText(2, q_("Border", nullptr));
        comboBoxLineWrapVisualizationEnd->setItemText(3, q_("Margin", nullptr));

        labelLineWrap->setText(q_("Line wrap", nullptr));
        groupBoxDisplay->setTitle(q_("Display", nullptr));
        checkBoxHighlightCurrentLine->setText(q_("Highlight current line", nullptr));
        checkBoxEnableLineNumbers->setText(q_("Display Line Numbers", nullptr));
        checkBoxEnableBraceMatching->setText(q_("Enable brace matching", nullptr));
        label_3->setText(q_("Font", nullptr));
        labelColorSyntaxHighlighting->setText(q_("Color syntax highlighting", nullptr));
        mouseWheelZoomBox->setText(q_("Ctrl/Cmd-Mouse-wheel zooms text", nullptr));
        groupBoxIndentation->setTitle(q_("Indentation", nullptr));
        checkBoxAutoIndent->setText(q_("Auto Indent", nullptr));
        checkBoxBackspaceUnindents->setText(q_("Backspace unindents", nullptr));
        label_10->setText(q_("Indent using", nullptr));
        comboBoxIndentUsing->setItemText(0, q_("Spaces", nullptr));
        comboBoxIndentUsing->setItemText(1, q_("Tabs", nullptr));

        labelIndentationWidth->setText(q_("Indentation width", nullptr));
        labelTabWidth->setText(q_("Tab width", nullptr));
        label_9->setText(q_("Tab key function", nullptr));
        comboBoxTabKeyFunction->setItemText(0, q_("Indent", nullptr));
        comboBoxTabKeyFunction->setItemText(1, q_("Insert Tab", nullptr));

        labelWhitespaces->setText(q_("Show whitespace", nullptr));
        comboBoxShowWhitespace->setItemText(0, q_("Never", nullptr));
        comboBoxShowWhitespace->setItemText(1, q_("Always", nullptr));
        comboBoxShowWhitespace->setItemText(2, q_("Only after indentation", nullptr));

        labelShowWhitespacesSize->setText(q_("Size", nullptr));
        updateCheckBox->setText(q_("Automatically check for updates", nullptr));
        snapshotCheckBox->setText(q_("Include development snapshots", nullptr));
        checkNowButton->setText(q_("Check Now", nullptr));
        lastCheckedLabel->setText(q_("Last checked: ", nullptr));
        labelFeatures->setText(q_("Features", nullptr));
        groupBoxOctoPrint->setTitle(q_("OctoPrint", nullptr));
        labelOctoPrintURL->setText(q_("URL", nullptr));
        labelOctoPrintApiKey->setText(q_("API Key", nullptr));
        pushButtonOctoPrintSlicingEngine->setText(q_("Load", nullptr));
        pushButtonOctoPrintSlicingProfile->setText(q_("Load", nullptr));
        labelOctoPrintCheckConnection->setText(QString());
        pushButtonOctoPrintCheckConnection->setText(q_("Check", nullptr));
        labelOctoPrintSlicingProfile->setText(q_("Slicing Profile", nullptr));
        labelOctoPrintSlicingEngine->setText(q_("Slicing Engine", nullptr));
        labelOctoPrintFileFormat->setText(q_("File Format", nullptr));
        labelOctoPrintAction->setText(q_("Action", nullptr));
        pushButtonOctoPrintApiKey->setText(q_("Show", nullptr));
        labelOctoPrintApiKeyNote->setText(q_("Note: The API key is stored unencrypted in the application settings.", nullptr));
        groupBox_CSG->setTitle(q_("OpenCSG", nullptr));
        openCSGWarningBox->setText(q_("Show capability warning", nullptr));
        enableOpenCSGBox->setText(q_("Enable for OpenGL 1.x", nullptr));
        label_7->setText(q_("Turn off rendering at ", nullptr));
        label_8->setText(q_("elements", nullptr));
        forceGoldfeatherBox->setText(q_("Force Goldfeather", nullptr));
        groupBox_CGAL->setTitle(q_("CGAL", nullptr));
        label_4->setText(q_("CGAL Cache size", nullptr));
        label_2->setText(q_("MB", nullptr));
        label_5->setText(q_("PolySet Cache size", nullptr));
        label_6->setText(q_("MB", nullptr));
        groupBox_UI->setTitle(q_("User Interface", nullptr));
        reorderCheckBox->setText(q_("Enable docking of Editor and Console in different places", nullptr));
        undockCheckBox->setText(q_("Enable undocking of Editor and Console to separate windows", nullptr));
        launcherBox->setText(q_("Show Welcome Screen", nullptr));
        localizationCheckBox->setText(q_("Enable user interface localization (requires restart of OpenSCAD)", nullptr));
        autoReloadRaiseCheckBox->setText(q_("Bring window to front after automatic reload", nullptr));
        enableSoundOnRenderCompleteCheckBox->setText(q_("Play sound notification on render complete", nullptr));
        timeThresholdOnRenderCompleteSoundLabel->setText(q_("Play sound when render completion time is at least", nullptr));
        secLabel->setText(q_("sec", nullptr));
        groupBox_lang->setTitle(q_("OpenSCAD Language Features", nullptr));
        enableParameterCheckBox->setText(q_("Warn when there is a parameter mismatch in user module calls", nullptr));
        enableHardwarningsCheckBox->setText(q_("Stop on the first warning", nullptr));
        enableRangeCheckBox->setText(q_("Check the parameter range for builtin modules", nullptr));
        groupBox_Export->setTitle(q_("Export Features", nullptr));
        useAsciiSTLCheckBox->setText(q_("Default to ASCII STL export", nullptr));
        groupBox_Debug->setTitle(q_("Debugging", nullptr));
        enableHidapiTraceCheckBox->setText(q_("Enable tracing of HIDAPI events (requires restart of OpenSCAD)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Preferences: public Ui_Preferences {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFERENCES_H
