/********************************************************************************
** Form generated from reading UI file 'LibraryInfoDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LIBRARYINFODIALOG_H
#define UI_LIBRARYINFODIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextEdit>

QT_BEGIN_NAMESPACE

class Ui_LibraryInfoDialog
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QTextEdit *infoTextBox;
    QPushButton *okButton;

    void setupUi(QDialog *LibraryInfoDialog)
    {
        if (LibraryInfoDialog->objectName().isEmpty())
            LibraryInfoDialog->setObjectName(QString::fromUtf8("LibraryInfoDialog"));
        LibraryInfoDialog->resize(587, 466);
        gridLayout = new QGridLayout(LibraryInfoDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        gridLayout->setVerticalSpacing(6);
        label = new QLabel(LibraryInfoDialog);
        label->setObjectName(QString::fromUtf8("label"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setMinimumSize(QSize(0, 20));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        infoTextBox = new QTextEdit(LibraryInfoDialog);
        infoTextBox->setObjectName(QString::fromUtf8("infoTextBox"));
        infoTextBox->setMinimumSize(QSize(0, 0));
        infoTextBox->setReadOnly(true);

        gridLayout->addWidget(infoTextBox, 1, 0, 1, 1);

        okButton = new QPushButton(LibraryInfoDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));
        sizePolicy.setHeightForWidth(okButton->sizePolicy().hasHeightForWidth());
        okButton->setSizePolicy(sizePolicy);

        gridLayout->addWidget(okButton, 2, 0, 1, 1, Qt::AlignRight);


        retranslateUi(LibraryInfoDialog);

        QMetaObject::connectSlotsByName(LibraryInfoDialog);
    } // setupUi

    void retranslateUi(QDialog *LibraryInfoDialog)
    {
        LibraryInfoDialog->setWindowTitle(q_("Lib & Build Info", nullptr));
        label->setText(q_("OpenSCAD Detailed Library and Build Information", nullptr));
        okButton->setText(q_("&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LibraryInfoDialog: public Ui_LibraryInfoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LIBRARYINFODIALOG_H
