/****************************************************************************
** Meta object code from reading C++ file 'tabmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/tabmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tabmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TabManager_t {
    QByteArrayData data[40];
    char stringdata0[543];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TabManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TabManager_t qt_meta_stringdata_TabManager = {
    {
QT_MOC_LITERAL(0, 0, 10), // "TabManager"
QT_MOC_LITERAL(1, 11, 15), // "tabCountChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 11), // "tabSwitched"
QT_MOC_LITERAL(4, 40, 17), // "closeTabRequested"
QT_MOC_LITERAL(5, 58, 18), // "middleMouseClicked"
QT_MOC_LITERAL(6, 77, 14), // "highlightError"
QT_MOC_LITERAL(7, 92, 20), // "unhighlightLastError"
QT_MOC_LITERAL(8, 113, 4), // "undo"
QT_MOC_LITERAL(9, 118, 4), // "redo"
QT_MOC_LITERAL(10, 123, 3), // "cut"
QT_MOC_LITERAL(11, 127, 4), // "copy"
QT_MOC_LITERAL(12, 132, 5), // "paste"
QT_MOC_LITERAL(13, 138, 15), // "indentSelection"
QT_MOC_LITERAL(14, 154, 17), // "unindentSelection"
QT_MOC_LITERAL(15, 172, 16), // "commentSelection"
QT_MOC_LITERAL(16, 189, 18), // "uncommentSelection"
QT_MOC_LITERAL(17, 208, 21), // "updateActionUndoState"
QT_MOC_LITERAL(18, 230, 14), // "toggleBookmark"
QT_MOC_LITERAL(19, 245, 12), // "nextBookmark"
QT_MOC_LITERAL(20, 258, 12), // "prevBookmark"
QT_MOC_LITERAL(21, 271, 15), // "jumpToNextError"
QT_MOC_LITERAL(22, 287, 12), // "copyFileName"
QT_MOC_LITERAL(23, 300, 12), // "copyFilePath"
QT_MOC_LITERAL(24, 313, 8), // "closeTab"
QT_MOC_LITERAL(25, 322, 20), // "showContextMenuEvent"
QT_MOC_LITERAL(26, 343, 24), // "showTabHeaderContextMenu"
QT_MOC_LITERAL(27, 368, 13), // "stopAnimation"
QT_MOC_LITERAL(28, 382, 15), // "updateFindState"
QT_MOC_LITERAL(29, 398, 27), // "onHyperlinkIndicatorClicked"
QT_MOC_LITERAL(30, 426, 3), // "pos"
QT_MOC_LITERAL(31, 430, 9), // "actionNew"
QT_MOC_LITERAL(32, 440, 21), // "setContentRenderState"
QT_MOC_LITERAL(33, 462, 14), // "setTabModified"
QT_MOC_LITERAL(34, 477, 16), // "EditorInterface*"
QT_MOC_LITERAL(35, 494, 7), // "saveAll"
QT_MOC_LITERAL(36, 502, 15), // "closeCurrentTab"
QT_MOC_LITERAL(37, 518, 7), // "nextTab"
QT_MOC_LITERAL(38, 526, 7), // "prevTab"
QT_MOC_LITERAL(39, 534, 8) // "setFocus"

    },
    "TabManager\0tabCountChanged\0\0tabSwitched\0"
    "closeTabRequested\0middleMouseClicked\0"
    "highlightError\0unhighlightLastError\0"
    "undo\0redo\0cut\0copy\0paste\0indentSelection\0"
    "unindentSelection\0commentSelection\0"
    "uncommentSelection\0updateActionUndoState\0"
    "toggleBookmark\0nextBookmark\0prevBookmark\0"
    "jumpToNextError\0copyFileName\0copyFilePath\0"
    "closeTab\0showContextMenuEvent\0"
    "showTabHeaderContextMenu\0stopAnimation\0"
    "updateFindState\0onHyperlinkIndicatorClicked\0"
    "pos\0actionNew\0setContentRenderState\0"
    "setTabModified\0EditorInterface*\0saveAll\0"
    "closeCurrentTab\0nextTab\0prevTab\0"
    "setFocus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TabManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  194,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  197,    2, 0x08 /* Private */,
       4,    1,  200,    2, 0x08 /* Private */,
       5,    1,  203,    2, 0x08 /* Private */,
       6,    1,  206,    2, 0x08 /* Private */,
       7,    0,  209,    2, 0x08 /* Private */,
       8,    0,  210,    2, 0x08 /* Private */,
       9,    0,  211,    2, 0x08 /* Private */,
      10,    0,  212,    2, 0x08 /* Private */,
      11,    0,  213,    2, 0x08 /* Private */,
      12,    0,  214,    2, 0x08 /* Private */,
      13,    0,  215,    2, 0x08 /* Private */,
      14,    0,  216,    2, 0x08 /* Private */,
      15,    0,  217,    2, 0x08 /* Private */,
      16,    0,  218,    2, 0x08 /* Private */,
      17,    0,  219,    2, 0x08 /* Private */,
      18,    0,  220,    2, 0x08 /* Private */,
      19,    0,  221,    2, 0x08 /* Private */,
      20,    0,  222,    2, 0x08 /* Private */,
      21,    0,  223,    2, 0x08 /* Private */,
      22,    0,  224,    2, 0x08 /* Private */,
      23,    0,  225,    2, 0x08 /* Private */,
      24,    0,  226,    2, 0x08 /* Private */,
      25,    1,  227,    2, 0x08 /* Private */,
      26,    1,  230,    2, 0x08 /* Private */,
      27,    0,  233,    2, 0x08 /* Private */,
      28,    0,  234,    2, 0x08 /* Private */,
      29,    1,  235,    2, 0x08 /* Private */,
      31,    0,  238,    2, 0x0a /* Public */,
      32,    0,  239,    2, 0x0a /* Public */,
      33,    2,  240,    2, 0x0a /* Public */,
      35,    0,  245,    2, 0x0a /* Public */,
      36,    0,  246,    2, 0x0a /* Public */,
      37,    0,  247,    2, 0x0a /* Public */,
      38,    0,  248,    2, 0x0a /* Public */,
      39,    0,  249,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   30,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 34,    2,    2,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TabManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TabManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->tabCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->tabSwitched((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->closeTabRequested((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->middleMouseClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->highlightError((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->unhighlightLastError(); break;
        case 6: _t->undo(); break;
        case 7: _t->redo(); break;
        case 8: _t->cut(); break;
        case 9: _t->copy(); break;
        case 10: _t->paste(); break;
        case 11: _t->indentSelection(); break;
        case 12: _t->unindentSelection(); break;
        case 13: _t->commentSelection(); break;
        case 14: _t->uncommentSelection(); break;
        case 15: _t->updateActionUndoState(); break;
        case 16: _t->toggleBookmark(); break;
        case 17: _t->nextBookmark(); break;
        case 18: _t->prevBookmark(); break;
        case 19: _t->jumpToNextError(); break;
        case 20: _t->copyFileName(); break;
        case 21: _t->copyFilePath(); break;
        case 22: _t->closeTab(); break;
        case 23: _t->showContextMenuEvent((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 24: _t->showTabHeaderContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 25: _t->stopAnimation(); break;
        case 26: _t->updateFindState(); break;
        case 27: _t->onHyperlinkIndicatorClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->actionNew(); break;
        case 29: _t->setContentRenderState(); break;
        case 30: _t->setTabModified((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< EditorInterface*(*)>(_a[2]))); break;
        case 31: { bool _r = _t->saveAll();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->closeCurrentTab(); break;
        case 33: _t->nextTab(); break;
        case 34: _t->prevTab(); break;
        case 35: _t->setFocus(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< EditorInterface* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TabManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabManager::tabCountChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TabManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TabManager.data,
    qt_meta_data_TabManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TabManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TabManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TabManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TabManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void TabManager::tabCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
