/********************************************************************************
** Form generated from reading UI file 'ProgressWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROGRESSWIDGET_H
#define UI_PROGRESSWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ProgressWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QProgressBar *progressBar;
    QPushButton *stopButton;

    void setupUi(QWidget *ProgressWidget)
    {
        if (ProgressWidget->objectName().isEmpty())
            ProgressWidget->setObjectName(QString::fromUtf8("ProgressWidget"));
        ProgressWidget->resize(121, 20);
        horizontalLayout = new QHBoxLayout(ProgressWidget);
        horizontalLayout->setSpacing(5);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        progressBar = new QProgressBar(ProgressWidget);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setMaximum(1000);
        progressBar->setValue(24);

        horizontalLayout->addWidget(progressBar);

        stopButton = new QPushButton(ProgressWidget);
        stopButton->setObjectName(QString::fromUtf8("stopButton"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(stopButton->sizePolicy().hasHeightForWidth());
        stopButton->setSizePolicy(sizePolicy);
        stopButton->setMaximumSize(QSize(12, 12));
        stopButton->setBaseSize(QSize(12, 12));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/stopbutton.png"), QSize(), QIcon::Normal, QIcon::Off);
        stopButton->setIcon(icon);
        stopButton->setIconSize(QSize(12, 12));
        stopButton->setFlat(true);

        horizontalLayout->addWidget(stopButton);


        retranslateUi(ProgressWidget);

        QMetaObject::connectSlotsByName(ProgressWidget);
    } // setupUi

    void retranslateUi(QWidget *ProgressWidget)
    {
        progressBar->setFormat(q_("%v / %m", nullptr));
        (void)ProgressWidget;
    } // retranslateUi

};

namespace Ui {
    class ProgressWidget: public Ui_ProgressWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROGRESSWIDGET_H
