/********************************************************************************
** Form generated from reading UI file 'PrintInitDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTINITDIALOG_H
#define UI_PRINTINITDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_PrintInitDialog
{
public:
    QGridLayout *gridLayout;
    QPushButton *okButton;
    QPushButton *cancelButton;
    QTextBrowser *textBrowser;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QPushButton *printServiceButton;
    QPushButton *octoPrintButton;
    QSpacerItem *horizontalSpacer;

    void setupUi(QDialog *PrintInitDialog)
    {
        if (PrintInitDialog->objectName().isEmpty())
            PrintInitDialog->setObjectName(QString::fromUtf8("PrintInitDialog"));
        PrintInitDialog->resize(504, 246);
        gridLayout = new QGridLayout(PrintInitDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setHorizontalSpacing(16);
        gridLayout->setVerticalSpacing(6);
        okButton = new QPushButton(PrintInitDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(okButton->sizePolicy().hasHeightForWidth());
        okButton->setSizePolicy(sizePolicy);

        gridLayout->addWidget(okButton, 4, 2, 1, 1);

        cancelButton = new QPushButton(PrintInitDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        gridLayout->addWidget(cancelButton, 4, 1, 1, 1);

        textBrowser = new QTextBrowser(PrintInitDialog);
        textBrowser->setObjectName(QString::fromUtf8("textBrowser"));
        textBrowser->setMinimumSize(QSize(480, 128));
        textBrowser->setReadOnly(true);
        textBrowser->setSource(QUrl(QString::fromUtf8("qrc:/src/PrintInitDialog.html")));
        textBrowser->setOpenExternalLinks(true);

        gridLayout->addWidget(textBrowser, 0, 0, 1, 3);

        verticalSpacer = new QSpacerItem(20, 6, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);

        groupBox = new QGroupBox(PrintInitDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        groupBox->setFlat(false);
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        printServiceButton = new QPushButton(groupBox);
        printServiceButton->setObjectName(QString::fromUtf8("printServiceButton"));
        printServiceButton->setCheckable(true);
        printServiceButton->setAutoExclusive(true);

        horizontalLayout->addWidget(printServiceButton);

        octoPrintButton = new QPushButton(groupBox);
        octoPrintButton->setObjectName(QString::fromUtf8("octoPrintButton"));
        octoPrintButton->setCheckable(true);
        octoPrintButton->setAutoExclusive(true);

        horizontalLayout->addWidget(octoPrintButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        gridLayout->addWidget(groupBox, 2, 0, 1, 3);


        retranslateUi(PrintInitDialog);

        QMetaObject::connectSlotsByName(PrintInitDialog);
    } // setupUi

    void retranslateUi(QDialog *PrintInitDialog)
    {
        PrintInitDialog->setWindowTitle(q_("3D Print", nullptr));
        okButton->setText(q_("&OK", nullptr));
        cancelButton->setText(q_("Cancel", nullptr));
        groupBox->setTitle(QString());
        printServiceButton->setText(q_("%1", nullptr));
        octoPrintButton->setText(q_("OctoPrint", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PrintInitDialog: public Ui_PrintInitDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PRINTINITDIALOG_H
