/****************************************************************************
** Meta object code from reading C++ file 'AxisConfigWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/input/AxisConfigWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AxisConfigWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AxisConfigWidget_t {
    QByteArrayData data[50];
    char stringdata0[1571];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AxisConfigWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AxisConfigWidget_t qt_meta_stringdata_AxisConfigWidget = {
    {
QT_MOC_LITERAL(0, 0, 16), // "AxisConfigWidget"
QT_MOC_LITERAL(1, 17, 19), // "inputMappingChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 23), // "inputCalibrationChanged"
QT_MOC_LITERAL(4, 62, 16), // "inputGainChanged"
QT_MOC_LITERAL(5, 79, 11), // "on_AxisTrim"
QT_MOC_LITERAL(6, 91, 16), // "on_AxisTrimReset"
QT_MOC_LITERAL(7, 108, 33), // "on_comboBoxTranslationX_activ..."
QT_MOC_LITERAL(8, 142, 3), // "val"
QT_MOC_LITERAL(9, 146, 33), // "on_comboBoxTranslationY_activ..."
QT_MOC_LITERAL(10, 180, 33), // "on_comboBoxTranslationZ_activ..."
QT_MOC_LITERAL(11, 214, 38), // "on_comboBoxTranslationXVPRel_..."
QT_MOC_LITERAL(12, 253, 38), // "on_comboBoxTranslationYVPRel_..."
QT_MOC_LITERAL(13, 292, 38), // "on_comboBoxTranslationZVPRel_..."
QT_MOC_LITERAL(14, 331, 30), // "on_comboBoxRotationX_activated"
QT_MOC_LITERAL(15, 362, 30), // "on_comboBoxRotationY_activated"
QT_MOC_LITERAL(16, 393, 30), // "on_comboBoxRotationZ_activated"
QT_MOC_LITERAL(17, 424, 35), // "on_comboBoxRotationXVPRel_act..."
QT_MOC_LITERAL(18, 460, 35), // "on_comboBoxRotationYVPRel_act..."
QT_MOC_LITERAL(19, 496, 35), // "on_comboBoxRotationZVPRel_act..."
QT_MOC_LITERAL(20, 532, 25), // "on_comboBoxZoom_activated"
QT_MOC_LITERAL(21, 558, 26), // "on_comboBoxZoom2_activated"
QT_MOC_LITERAL(22, 585, 39), // "on_doubleSpinBoxRotateGain_va..."
QT_MOC_LITERAL(23, 625, 44), // "on_doubleSpinBoxTranslationGa..."
QT_MOC_LITERAL(24, 670, 49), // "on_doubleSpinBoxTranslationVP..."
QT_MOC_LITERAL(25, 720, 37), // "on_doubleSpinBoxZoomGain_valu..."
QT_MOC_LITERAL(26, 758, 38), // "on_doubleSpinBoxDeadzone0_val..."
QT_MOC_LITERAL(27, 797, 38), // "on_doubleSpinBoxDeadzone1_val..."
QT_MOC_LITERAL(28, 836, 38), // "on_doubleSpinBoxDeadzone2_val..."
QT_MOC_LITERAL(29, 875, 38), // "on_doubleSpinBoxDeadzone3_val..."
QT_MOC_LITERAL(30, 914, 38), // "on_doubleSpinBoxDeadzone4_val..."
QT_MOC_LITERAL(31, 953, 38), // "on_doubleSpinBoxDeadzone5_val..."
QT_MOC_LITERAL(32, 992, 38), // "on_doubleSpinBoxDeadzone6_val..."
QT_MOC_LITERAL(33, 1031, 38), // "on_doubleSpinBoxDeadzone7_val..."
QT_MOC_LITERAL(34, 1070, 38), // "on_doubleSpinBoxDeadzone8_val..."
QT_MOC_LITERAL(35, 1109, 34), // "on_doubleSpinBoxTrim0_valueCh..."
QT_MOC_LITERAL(36, 1144, 34), // "on_doubleSpinBoxTrim1_valueCh..."
QT_MOC_LITERAL(37, 1179, 34), // "on_doubleSpinBoxTrim2_valueCh..."
QT_MOC_LITERAL(38, 1214, 34), // "on_doubleSpinBoxTrim3_valueCh..."
QT_MOC_LITERAL(39, 1249, 34), // "on_doubleSpinBoxTrim4_valueCh..."
QT_MOC_LITERAL(40, 1284, 34), // "on_doubleSpinBoxTrim5_valueCh..."
QT_MOC_LITERAL(41, 1319, 34), // "on_doubleSpinBoxTrim6_valueCh..."
QT_MOC_LITERAL(42, 1354, 34), // "on_doubleSpinBoxTrim7_valueCh..."
QT_MOC_LITERAL(43, 1389, 34), // "on_doubleSpinBoxTrim8_valueCh..."
QT_MOC_LITERAL(44, 1424, 25), // "on_checkBoxHIDAPI_toggled"
QT_MOC_LITERAL(45, 1450, 27), // "on_checkBoxSpaceNav_toggled"
QT_MOC_LITERAL(46, 1478, 27), // "on_checkBoxJoystick_toggled"
QT_MOC_LITERAL(47, 1506, 27), // "on_checkBoxQGamepad_toggled"
QT_MOC_LITERAL(48, 1534, 23), // "on_checkBoxDBus_toggled"
QT_MOC_LITERAL(49, 1558, 12) // "updateStates"

    },
    "AxisConfigWidget\0inputMappingChanged\0"
    "\0inputCalibrationChanged\0inputGainChanged\0"
    "on_AxisTrim\0on_AxisTrimReset\0"
    "on_comboBoxTranslationX_activated\0val\0"
    "on_comboBoxTranslationY_activated\0"
    "on_comboBoxTranslationZ_activated\0"
    "on_comboBoxTranslationXVPRel_activated\0"
    "on_comboBoxTranslationYVPRel_activated\0"
    "on_comboBoxTranslationZVPRel_activated\0"
    "on_comboBoxRotationX_activated\0"
    "on_comboBoxRotationY_activated\0"
    "on_comboBoxRotationZ_activated\0"
    "on_comboBoxRotationXVPRel_activated\0"
    "on_comboBoxRotationYVPRel_activated\0"
    "on_comboBoxRotationZVPRel_activated\0"
    "on_comboBoxZoom_activated\0"
    "on_comboBoxZoom2_activated\0"
    "on_doubleSpinBoxRotateGain_valueChanged\0"
    "on_doubleSpinBoxTranslationGain_valueChanged\0"
    "on_doubleSpinBoxTranslationVPRelGain_valueChanged\0"
    "on_doubleSpinBoxZoomGain_valueChanged\0"
    "on_doubleSpinBoxDeadzone0_valueChanged\0"
    "on_doubleSpinBoxDeadzone1_valueChanged\0"
    "on_doubleSpinBoxDeadzone2_valueChanged\0"
    "on_doubleSpinBoxDeadzone3_valueChanged\0"
    "on_doubleSpinBoxDeadzone4_valueChanged\0"
    "on_doubleSpinBoxDeadzone5_valueChanged\0"
    "on_doubleSpinBoxDeadzone6_valueChanged\0"
    "on_doubleSpinBoxDeadzone7_valueChanged\0"
    "on_doubleSpinBoxDeadzone8_valueChanged\0"
    "on_doubleSpinBoxTrim0_valueChanged\0"
    "on_doubleSpinBoxTrim1_valueChanged\0"
    "on_doubleSpinBoxTrim2_valueChanged\0"
    "on_doubleSpinBoxTrim3_valueChanged\0"
    "on_doubleSpinBoxTrim4_valueChanged\0"
    "on_doubleSpinBoxTrim5_valueChanged\0"
    "on_doubleSpinBoxTrim6_valueChanged\0"
    "on_doubleSpinBoxTrim7_valueChanged\0"
    "on_doubleSpinBoxTrim8_valueChanged\0"
    "on_checkBoxHIDAPI_toggled\0"
    "on_checkBoxSpaceNav_toggled\0"
    "on_checkBoxJoystick_toggled\0"
    "on_checkBoxQGamepad_toggled\0"
    "on_checkBoxDBus_toggled\0updateStates"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AxisConfigWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      47,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  249,    2, 0x06 /* Public */,
       3,    0,  250,    2, 0x06 /* Public */,
       4,    0,  251,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  252,    2, 0x0a /* Public */,
       6,    0,  253,    2, 0x0a /* Public */,
       7,    1,  254,    2, 0x0a /* Public */,
       9,    1,  257,    2, 0x0a /* Public */,
      10,    1,  260,    2, 0x0a /* Public */,
      11,    1,  263,    2, 0x0a /* Public */,
      12,    1,  266,    2, 0x0a /* Public */,
      13,    1,  269,    2, 0x0a /* Public */,
      14,    1,  272,    2, 0x0a /* Public */,
      15,    1,  275,    2, 0x0a /* Public */,
      16,    1,  278,    2, 0x0a /* Public */,
      17,    1,  281,    2, 0x0a /* Public */,
      18,    1,  284,    2, 0x0a /* Public */,
      19,    1,  287,    2, 0x0a /* Public */,
      20,    1,  290,    2, 0x0a /* Public */,
      21,    1,  293,    2, 0x0a /* Public */,
      22,    1,  296,    2, 0x0a /* Public */,
      23,    1,  299,    2, 0x0a /* Public */,
      24,    1,  302,    2, 0x0a /* Public */,
      25,    1,  305,    2, 0x0a /* Public */,
      26,    1,  308,    2, 0x0a /* Public */,
      27,    1,  311,    2, 0x0a /* Public */,
      28,    1,  314,    2, 0x0a /* Public */,
      29,    1,  317,    2, 0x0a /* Public */,
      30,    1,  320,    2, 0x0a /* Public */,
      31,    1,  323,    2, 0x0a /* Public */,
      32,    1,  326,    2, 0x0a /* Public */,
      33,    1,  329,    2, 0x0a /* Public */,
      34,    1,  332,    2, 0x0a /* Public */,
      35,    1,  335,    2, 0x0a /* Public */,
      36,    1,  338,    2, 0x0a /* Public */,
      37,    1,  341,    2, 0x0a /* Public */,
      38,    1,  344,    2, 0x0a /* Public */,
      39,    1,  347,    2, 0x0a /* Public */,
      40,    1,  350,    2, 0x0a /* Public */,
      41,    1,  353,    2, 0x0a /* Public */,
      42,    1,  356,    2, 0x0a /* Public */,
      43,    1,  359,    2, 0x0a /* Public */,
      44,    1,  362,    2, 0x0a /* Public */,
      45,    1,  365,    2, 0x0a /* Public */,
      46,    1,  368,    2, 0x0a /* Public */,
      47,    1,  371,    2, 0x0a /* Public */,
      48,    1,  374,    2, 0x0a /* Public */,
      49,    0,  377,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Double,    8,
    QMetaType::Void, QMetaType::Double,    8,
    QMetaType::Void, QMetaType::Double,    8,
    QMetaType::Void, QMetaType::Double,    8,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void AxisConfigWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AxisConfigWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->inputMappingChanged(); break;
        case 1: _t->inputCalibrationChanged(); break;
        case 2: _t->inputGainChanged(); break;
        case 3: _t->on_AxisTrim(); break;
        case 4: _t->on_AxisTrimReset(); break;
        case 5: _t->on_comboBoxTranslationX_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->on_comboBoxTranslationY_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->on_comboBoxTranslationZ_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->on_comboBoxTranslationXVPRel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->on_comboBoxTranslationYVPRel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->on_comboBoxTranslationZVPRel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->on_comboBoxRotationX_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->on_comboBoxRotationY_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->on_comboBoxRotationZ_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->on_comboBoxRotationXVPRel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->on_comboBoxRotationYVPRel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->on_comboBoxRotationZVPRel_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->on_comboBoxZoom_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->on_comboBoxZoom2_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->on_doubleSpinBoxRotateGain_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 20: _t->on_doubleSpinBoxTranslationGain_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 21: _t->on_doubleSpinBoxTranslationVPRelGain_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 22: _t->on_doubleSpinBoxZoomGain_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 23: _t->on_doubleSpinBoxDeadzone0_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 24: _t->on_doubleSpinBoxDeadzone1_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 25: _t->on_doubleSpinBoxDeadzone2_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 26: _t->on_doubleSpinBoxDeadzone3_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 27: _t->on_doubleSpinBoxDeadzone4_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 28: _t->on_doubleSpinBoxDeadzone5_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 29: _t->on_doubleSpinBoxDeadzone6_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 30: _t->on_doubleSpinBoxDeadzone7_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 31: _t->on_doubleSpinBoxDeadzone8_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 32: _t->on_doubleSpinBoxTrim0_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 33: _t->on_doubleSpinBoxTrim1_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 34: _t->on_doubleSpinBoxTrim2_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 35: _t->on_doubleSpinBoxTrim3_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 36: _t->on_doubleSpinBoxTrim4_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 37: _t->on_doubleSpinBoxTrim5_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 38: _t->on_doubleSpinBoxTrim6_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 39: _t->on_doubleSpinBoxTrim7_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 40: _t->on_doubleSpinBoxTrim8_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 41: _t->on_checkBoxHIDAPI_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 42: _t->on_checkBoxSpaceNav_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 43: _t->on_checkBoxJoystick_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 44: _t->on_checkBoxQGamepad_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 45: _t->on_checkBoxDBus_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 46: _t->updateStates(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AxisConfigWidget::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AxisConfigWidget::inputMappingChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AxisConfigWidget::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AxisConfigWidget::inputCalibrationChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AxisConfigWidget::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AxisConfigWidget::inputGainChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AxisConfigWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_AxisConfigWidget.data,
    qt_meta_data_AxisConfigWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AxisConfigWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AxisConfigWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AxisConfigWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Axis"))
        return static_cast< Ui::Axis*>(this);
    if (!strcmp(_clname, "InitConfigurator"))
        return static_cast< InitConfigurator*>(this);
    return QWidget::qt_metacast(_clname);
}

int AxisConfigWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 47;
    }
    return _id;
}

// SIGNAL 0
void AxisConfigWidget::inputMappingChanged()const
{
    QMetaObject::activate(const_cast< AxisConfigWidget *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AxisConfigWidget::inputCalibrationChanged()const
{
    QMetaObject::activate(const_cast< AxisConfigWidget *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void AxisConfigWidget::inputGainChanged()const
{
    QMetaObject::activate(const_cast< AxisConfigWidget *>(this), &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
