/********************************************************************************
** Form generated from reading UI file 'AxisConfigWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_AXISCONFIGWIDGET_H
#define UI_AXISCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Axis
{
public:
    QGridLayout *gridLayout;
    QScrollArea *scrollArea_InputAxis;
    QWidget *scrollAreaWidgetInputContents;
    QGridLayout *gridLayout_7;
    QGridLayout *gridLayout_10;
    QLabel *labelAxisTrim3;
    QDoubleSpinBox *doubleSpinBoxDeadzone6;
    QLabel *labelAxisTrim0;
    QProgressBar *progressBarAxis3;
    QDoubleSpinBox *doubleSpinBoxTrim0;
    QPushButton *pushButtonAxisTrimReset;
    QDoubleSpinBox *doubleSpinBoxDeadzone1;
    QProgressBar *progressBarAxis7;
    QProgressBar *progressBarAxis4;
    QDoubleSpinBox *doubleSpinBoxDeadzone2;
    QDoubleSpinBox *doubleSpinBoxDeadzone0;
    QProgressBar *progressBarAxis5;
    QLabel *labelAxisDeadzone0;
    QDoubleSpinBox *doubleSpinBoxTrim1;
    QDoubleSpinBox *doubleSpinBoxTrim6;
    QProgressBar *progressBarAxis1;
    QDoubleSpinBox *doubleSpinBoxTrim2;
    QDoubleSpinBox *doubleSpinBoxTrim5;
    QPushButton *pushButtonAxisTrim;
    QProgressBar *progressBarAxis8;
    QDoubleSpinBox *doubleSpinBoxDeadzone7;
    QDoubleSpinBox *doubleSpinBoxDeadzone3;
    QDoubleSpinBox *doubleSpinBoxDeadzone4;
    QDoubleSpinBox *doubleSpinBoxTrim3;
    QDoubleSpinBox *doubleSpinBoxTrim4;
    QDoubleSpinBox *doubleSpinBoxTrim8;
    QDoubleSpinBox *doubleSpinBoxTrim7;
    QDoubleSpinBox *doubleSpinBoxDeadzone8;
    QDoubleSpinBox *doubleSpinBoxDeadzone5;
    QProgressBar *progressBarAxis6;
    QProgressBar *progressBarAxis2;
    QProgressBar *progressBarAxis0;
    QLabel *labelAxisTrim6;
    QLabel *labelAxisDeadzone3;
    QLabel *labelAxisDeadzone6;
    QLabel *labelAxisDeadzone7;
    QLabel *labelAxisTrim7;
    QLabel *labelAxisDeadzone8;
    QLabel *labelAxisTrim8;
    QLabel *labelAxisDeadzone5;
    QLabel *labelAxisTrim5;
    QLabel *labelAxisDeadzone4;
    QLabel *labelAxisTrim4;
    QLabel *labelAxisDeadzone2;
    QLabel *labelAxisTrim2;
    QLabel *labelAxis2;
    QLabel *labelAxis0;
    QLabel *labelAxis3;
    QLabel *labelAxis6;
    QLabel *labelAxis8;
    QLabel *labelAxis7;
    QLabel *labelAxis5;
    QLabel *labelAxis1;
    QLabel *labelAxis4;
    QLabel *labelAxisDeadzone1;
    QLabel *labelAxisTrim1;
    QGridLayout *gridLayout_11;
    QLabel *labelInputRotation;
    QComboBox *comboBoxRotationX;
    QLabel *labelInputZoom;
    QComboBox *comboBoxZoom;
    QComboBox *comboBoxTranslationX;
    QComboBox *comboBoxTranslationZVPRel;
    QComboBox *comboBoxTranslationZ;
    QDoubleSpinBox *doubleSpinBoxTranslationGain;
    QLabel *label_11;
    QLabel *label_16;
    QDoubleSpinBox *doubleSpinBoxRotateGain;
    QComboBox *comboBoxRotationY;
    QLabel *label_14;
    QDoubleSpinBox *doubleSpinBoxZoomGain;
    QDoubleSpinBox *doubleSpinBoxTranslationVPRelGain;
    QLabel *label_15;
    QComboBox *comboBoxTranslationY;
    QComboBox *comboBoxTranslationYVPRel;
    QComboBox *comboBoxTranslationXVPRel;
    QLabel *labelInputTranslation;
    QComboBox *comboBoxRotationZ;
    QLabel *labelInputTranslationViewPortRelative;
    QComboBox *comboBoxZoom2;
    QFrame *line_2;
    QComboBox *comboBoxRotationXVPRel;
    QLabel *labelInputRotationViewPortRelative;
    QComboBox *comboBoxRotationYVPRel;
    QComboBox *comboBoxRotationZVPRel;
    QDoubleSpinBox *doubleSpinBoxRotateVPRelGain;
    QLabel *label_3;
    QLabel *label;
    QGridLayout *gridLayout_2;
    QCheckBox *checkBoxSpaceNav;
    QCheckBox *checkBoxHIDAPI;
    QCheckBox *checkBoxQGamepad;
    QCheckBox *checkBoxJoystick;
    QCheckBox *checkBoxDBus;
    QLabel *label_2;
    QLabel *label_4;
    QLabel *label_driverInfo;
    QPushButton *pushButtonUpdate;
    QSpacerItem *verticalSpacer_4;

    void setupUi(QWidget *Axis)
    {
        if (Axis->objectName().isEmpty())
            Axis->setObjectName(QString::fromUtf8("Axis"));
        Axis->resize(633, 457);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Axis->sizePolicy().hasHeightForWidth());
        Axis->setSizePolicy(sizePolicy);
        Axis->setProperty("unifiedTitleAndToolBarOnMac", QVariant(true));
        gridLayout = new QGridLayout(Axis);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        scrollArea_InputAxis = new QScrollArea(Axis);
        scrollArea_InputAxis->setObjectName(QString::fromUtf8("scrollArea_InputAxis"));
        scrollArea_InputAxis->setWidgetResizable(true);
        scrollAreaWidgetInputContents = new QWidget();
        scrollAreaWidgetInputContents->setObjectName(QString::fromUtf8("scrollAreaWidgetInputContents"));
        scrollAreaWidgetInputContents->setGeometry(QRect(0, 0, 599, 610));
        gridLayout_7 = new QGridLayout(scrollAreaWidgetInputContents);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        gridLayout_10 = new QGridLayout();
        gridLayout_10->setObjectName(QString::fromUtf8("gridLayout_10"));
        labelAxisTrim3 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim3->setObjectName(QString::fromUtf8("labelAxisTrim3"));
        QFont font;
        font.setPointSize(8);
        labelAxisTrim3->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim3, 7, 2, 1, 1);

        doubleSpinBoxDeadzone6 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone6->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone6"));
        doubleSpinBoxDeadzone6->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone6->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone6, 11, 1, 1, 1);

        labelAxisTrim0 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim0->setObjectName(QString::fromUtf8("labelAxisTrim0"));
        labelAxisTrim0->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim0, 0, 2, 1, 1);

        progressBarAxis3 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis3->setObjectName(QString::fromUtf8("progressBarAxis3"));
        progressBarAxis3->setMinimum(-100);
        progressBarAxis3->setValue(0);
        progressBarAxis3->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis3, 9, 0, 1, 3);

        doubleSpinBoxTrim0 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim0->setObjectName(QString::fromUtf8("doubleSpinBoxTrim0"));
        doubleSpinBoxTrim0->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim0->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim0, 1, 2, 1, 1);

        pushButtonAxisTrimReset = new QPushButton(scrollAreaWidgetInputContents);
        pushButtonAxisTrimReset->setObjectName(QString::fromUtf8("pushButtonAxisTrimReset"));

        gridLayout_10->addWidget(pushButtonAxisTrimReset, 13, 0, 1, 3);

        doubleSpinBoxDeadzone1 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone1->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone1"));
        doubleSpinBoxDeadzone1->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone1->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone1, 1, 4, 1, 1);

        progressBarAxis7 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis7->setObjectName(QString::fromUtf8("progressBarAxis7"));
        progressBarAxis7->setMinimum(-100);
        progressBarAxis7->setValue(0);
        progressBarAxis7->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis7, 12, 3, 1, 3);

        progressBarAxis4 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis4->setObjectName(QString::fromUtf8("progressBarAxis4"));
        progressBarAxis4->setMinimum(-100);
        progressBarAxis4->setValue(0);
        progressBarAxis4->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis4, 9, 3, 1, 3);

        doubleSpinBoxDeadzone2 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone2->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone2"));
        doubleSpinBoxDeadzone2->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone2->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone2, 1, 7, 1, 1);

        doubleSpinBoxDeadzone0 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone0->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone0"));
        doubleSpinBoxDeadzone0->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone0->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone0, 1, 1, 1, 1);

        progressBarAxis5 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis5->setObjectName(QString::fromUtf8("progressBarAxis5"));
        progressBarAxis5->setMinimum(-100);
        progressBarAxis5->setValue(0);
        progressBarAxis5->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis5, 9, 6, 1, 3);

        labelAxisDeadzone0 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone0->setObjectName(QString::fromUtf8("labelAxisDeadzone0"));
        labelAxisDeadzone0->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone0, 0, 1, 1, 1);

        doubleSpinBoxTrim1 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim1->setObjectName(QString::fromUtf8("doubleSpinBoxTrim1"));
        doubleSpinBoxTrim1->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim1->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim1, 1, 5, 1, 1);

        doubleSpinBoxTrim6 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim6->setObjectName(QString::fromUtf8("doubleSpinBoxTrim6"));
        doubleSpinBoxTrim6->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim6->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim6, 11, 2, 1, 1);

        progressBarAxis1 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis1->setObjectName(QString::fromUtf8("progressBarAxis1"));
        progressBarAxis1->setMinimum(-100);
        progressBarAxis1->setValue(0);
        progressBarAxis1->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis1, 6, 3, 1, 3);

        doubleSpinBoxTrim2 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim2->setObjectName(QString::fromUtf8("doubleSpinBoxTrim2"));
        doubleSpinBoxTrim2->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim2->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim2, 1, 8, 1, 1);

        doubleSpinBoxTrim5 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim5->setObjectName(QString::fromUtf8("doubleSpinBoxTrim5"));
        doubleSpinBoxTrim5->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim5->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim5, 8, 8, 1, 1);

        pushButtonAxisTrim = new QPushButton(scrollAreaWidgetInputContents);
        pushButtonAxisTrim->setObjectName(QString::fromUtf8("pushButtonAxisTrim"));

        gridLayout_10->addWidget(pushButtonAxisTrim, 13, 6, 1, 3);

        progressBarAxis8 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis8->setObjectName(QString::fromUtf8("progressBarAxis8"));
        progressBarAxis8->setMinimum(-100);
        progressBarAxis8->setValue(0);
        progressBarAxis8->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis8, 12, 6, 1, 3);

        doubleSpinBoxDeadzone7 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone7->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone7"));
        doubleSpinBoxDeadzone7->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone7->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone7, 11, 4, 1, 1);

        doubleSpinBoxDeadzone3 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone3->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone3"));
        doubleSpinBoxDeadzone3->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone3->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone3, 8, 1, 1, 1);

        doubleSpinBoxDeadzone4 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone4->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone4"));
        doubleSpinBoxDeadzone4->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone4->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone4, 8, 4, 1, 1);

        doubleSpinBoxTrim3 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim3->setObjectName(QString::fromUtf8("doubleSpinBoxTrim3"));
        doubleSpinBoxTrim3->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim3->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim3, 8, 2, 1, 1);

        doubleSpinBoxTrim4 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim4->setObjectName(QString::fromUtf8("doubleSpinBoxTrim4"));
        doubleSpinBoxTrim4->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim4->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim4, 8, 5, 1, 1);

        doubleSpinBoxTrim8 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim8->setObjectName(QString::fromUtf8("doubleSpinBoxTrim8"));
        doubleSpinBoxTrim8->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim8->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim8, 11, 8, 1, 1);

        doubleSpinBoxTrim7 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTrim7->setObjectName(QString::fromUtf8("doubleSpinBoxTrim7"));
        doubleSpinBoxTrim7->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTrim7->setSingleStep(0.010000000000000);

        gridLayout_10->addWidget(doubleSpinBoxTrim7, 11, 5, 1, 1);

        doubleSpinBoxDeadzone8 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone8->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone8"));
        doubleSpinBoxDeadzone8->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone8->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone8, 11, 7, 1, 1);

        doubleSpinBoxDeadzone5 = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxDeadzone5->setObjectName(QString::fromUtf8("doubleSpinBoxDeadzone5"));
        doubleSpinBoxDeadzone5->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxDeadzone5->setSingleStep(0.050000000000000);

        gridLayout_10->addWidget(doubleSpinBoxDeadzone5, 8, 7, 1, 1);

        progressBarAxis6 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis6->setObjectName(QString::fromUtf8("progressBarAxis6"));
        progressBarAxis6->setMinimum(-100);
        progressBarAxis6->setValue(0);
        progressBarAxis6->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis6, 12, 0, 1, 3);

        progressBarAxis2 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis2->setObjectName(QString::fromUtf8("progressBarAxis2"));
        progressBarAxis2->setMinimum(-100);
        progressBarAxis2->setValue(0);
        progressBarAxis2->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis2, 6, 6, 1, 3);

        progressBarAxis0 = new QProgressBar(scrollAreaWidgetInputContents);
        progressBarAxis0->setObjectName(QString::fromUtf8("progressBarAxis0"));
        progressBarAxis0->setMinimum(-100);
        progressBarAxis0->setValue(0);
        progressBarAxis0->setFormat(QString::fromUtf8("0.00"));

        gridLayout_10->addWidget(progressBarAxis0, 6, 0, 1, 3);

        labelAxisTrim6 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim6->setObjectName(QString::fromUtf8("labelAxisTrim6"));
        labelAxisTrim6->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim6, 10, 2, 1, 1);

        labelAxisDeadzone3 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone3->setObjectName(QString::fromUtf8("labelAxisDeadzone3"));
        labelAxisDeadzone3->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone3, 7, 1, 1, 1);

        labelAxisDeadzone6 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone6->setObjectName(QString::fromUtf8("labelAxisDeadzone6"));
        labelAxisDeadzone6->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone6, 10, 1, 1, 1);

        labelAxisDeadzone7 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone7->setObjectName(QString::fromUtf8("labelAxisDeadzone7"));
        labelAxisDeadzone7->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone7, 10, 4, 1, 1);

        labelAxisTrim7 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim7->setObjectName(QString::fromUtf8("labelAxisTrim7"));
        labelAxisTrim7->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim7, 10, 5, 1, 1);

        labelAxisDeadzone8 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone8->setObjectName(QString::fromUtf8("labelAxisDeadzone8"));
        labelAxisDeadzone8->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone8, 10, 7, 1, 1);

        labelAxisTrim8 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim8->setObjectName(QString::fromUtf8("labelAxisTrim8"));
        labelAxisTrim8->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim8, 10, 8, 1, 1);

        labelAxisDeadzone5 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone5->setObjectName(QString::fromUtf8("labelAxisDeadzone5"));
        labelAxisDeadzone5->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone5, 7, 7, 1, 1);

        labelAxisTrim5 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim5->setObjectName(QString::fromUtf8("labelAxisTrim5"));
        labelAxisTrim5->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim5, 7, 8, 1, 1);

        labelAxisDeadzone4 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone4->setObjectName(QString::fromUtf8("labelAxisDeadzone4"));
        labelAxisDeadzone4->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone4, 7, 4, 1, 1);

        labelAxisTrim4 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim4->setObjectName(QString::fromUtf8("labelAxisTrim4"));
        labelAxisTrim4->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim4, 7, 5, 1, 1);

        labelAxisDeadzone2 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone2->setObjectName(QString::fromUtf8("labelAxisDeadzone2"));
        labelAxisDeadzone2->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone2, 0, 7, 1, 1);

        labelAxisTrim2 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim2->setObjectName(QString::fromUtf8("labelAxisTrim2"));
        labelAxisTrim2->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim2, 0, 8, 1, 1);

        labelAxis2 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis2->setObjectName(QString::fromUtf8("labelAxis2"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelAxis2->sizePolicy().hasHeightForWidth());
        labelAxis2->setSizePolicy(sizePolicy1);
        labelAxis2->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis2, 0, 6, 2, 1);

        labelAxis0 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis0->setObjectName(QString::fromUtf8("labelAxis0"));
        sizePolicy1.setHeightForWidth(labelAxis0->sizePolicy().hasHeightForWidth());
        labelAxis0->setSizePolicy(sizePolicy1);
        labelAxis0->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis0, 0, 0, 2, 1);

        labelAxis3 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis3->setObjectName(QString::fromUtf8("labelAxis3"));
        sizePolicy1.setHeightForWidth(labelAxis3->sizePolicy().hasHeightForWidth());
        labelAxis3->setSizePolicy(sizePolicy1);
        labelAxis3->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis3, 7, 0, 2, 1);

        labelAxis6 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis6->setObjectName(QString::fromUtf8("labelAxis6"));
        sizePolicy1.setHeightForWidth(labelAxis6->sizePolicy().hasHeightForWidth());
        labelAxis6->setSizePolicy(sizePolicy1);
        labelAxis6->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis6, 10, 0, 2, 1);

        labelAxis8 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis8->setObjectName(QString::fromUtf8("labelAxis8"));
        sizePolicy1.setHeightForWidth(labelAxis8->sizePolicy().hasHeightForWidth());
        labelAxis8->setSizePolicy(sizePolicy1);
        labelAxis8->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis8, 10, 6, 2, 1);

        labelAxis7 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis7->setObjectName(QString::fromUtf8("labelAxis7"));
        sizePolicy1.setHeightForWidth(labelAxis7->sizePolicy().hasHeightForWidth());
        labelAxis7->setSizePolicy(sizePolicy1);
        labelAxis7->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis7, 10, 3, 2, 1);

        labelAxis5 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis5->setObjectName(QString::fromUtf8("labelAxis5"));
        sizePolicy1.setHeightForWidth(labelAxis5->sizePolicy().hasHeightForWidth());
        labelAxis5->setSizePolicy(sizePolicy1);
        labelAxis5->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis5, 7, 6, 2, 1);

        labelAxis1 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis1->setObjectName(QString::fromUtf8("labelAxis1"));
        sizePolicy1.setHeightForWidth(labelAxis1->sizePolicy().hasHeightForWidth());
        labelAxis1->setSizePolicy(sizePolicy1);
        labelAxis1->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis1, 0, 3, 2, 1);

        labelAxis4 = new QLabel(scrollAreaWidgetInputContents);
        labelAxis4->setObjectName(QString::fromUtf8("labelAxis4"));
        sizePolicy1.setHeightForWidth(labelAxis4->sizePolicy().hasHeightForWidth());
        labelAxis4->setSizePolicy(sizePolicy1);
        labelAxis4->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(labelAxis4, 7, 3, 2, 1);

        labelAxisDeadzone1 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisDeadzone1->setObjectName(QString::fromUtf8("labelAxisDeadzone1"));
        labelAxisDeadzone1->setFont(font);

        gridLayout_10->addWidget(labelAxisDeadzone1, 0, 4, 1, 1);

        labelAxisTrim1 = new QLabel(scrollAreaWidgetInputContents);
        labelAxisTrim1->setObjectName(QString::fromUtf8("labelAxisTrim1"));
        labelAxisTrim1->setFont(font);

        gridLayout_10->addWidget(labelAxisTrim1, 0, 5, 1, 1);


        gridLayout_7->addLayout(gridLayout_10, 24, 0, 1, 2);

        gridLayout_11 = new QGridLayout();
        gridLayout_11->setObjectName(QString::fromUtf8("gridLayout_11"));
        labelInputRotation = new QLabel(scrollAreaWidgetInputContents);
        labelInputRotation->setObjectName(QString::fromUtf8("labelInputRotation"));

        gridLayout_11->addWidget(labelInputRotation, 3, 0, 1, 1);

        comboBoxRotationX = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxRotationX->setObjectName(QString::fromUtf8("comboBoxRotationX"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(comboBoxRotationX->sizePolicy().hasHeightForWidth());
        comboBoxRotationX->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxRotationX, 3, 1, 1, 1);

        labelInputZoom = new QLabel(scrollAreaWidgetInputContents);
        labelInputZoom->setObjectName(QString::fromUtf8("labelInputZoom"));

        gridLayout_11->addWidget(labelInputZoom, 6, 0, 1, 1);

        comboBoxZoom = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxZoom->setObjectName(QString::fromUtf8("comboBoxZoom"));
        sizePolicy2.setHeightForWidth(comboBoxZoom->sizePolicy().hasHeightForWidth());
        comboBoxZoom->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxZoom, 6, 1, 1, 1);

        comboBoxTranslationX = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxTranslationX->setObjectName(QString::fromUtf8("comboBoxTranslationX"));
        sizePolicy2.setHeightForWidth(comboBoxTranslationX->sizePolicy().hasHeightForWidth());
        comboBoxTranslationX->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxTranslationX, 1, 1, 1, 1);

        comboBoxTranslationZVPRel = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxTranslationZVPRel->setObjectName(QString::fromUtf8("comboBoxTranslationZVPRel"));
        sizePolicy2.setHeightForWidth(comboBoxTranslationZVPRel->sizePolicy().hasHeightForWidth());
        comboBoxTranslationZVPRel->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxTranslationZVPRel, 2, 3, 1, 1);

        comboBoxTranslationZ = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxTranslationZ->setObjectName(QString::fromUtf8("comboBoxTranslationZ"));
        sizePolicy2.setHeightForWidth(comboBoxTranslationZ->sizePolicy().hasHeightForWidth());
        comboBoxTranslationZ->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxTranslationZ, 1, 3, 1, 1);

        doubleSpinBoxTranslationGain = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTranslationGain->setObjectName(QString::fromUtf8("doubleSpinBoxTranslationGain"));
        doubleSpinBoxTranslationGain->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTranslationGain->setMinimum(0.100000000000000);
        doubleSpinBoxTranslationGain->setMaximum(9.990000000000000);
        doubleSpinBoxTranslationGain->setSingleStep(0.100000000000000);
        doubleSpinBoxTranslationGain->setValue(1.000000000000000);

        gridLayout_11->addWidget(doubleSpinBoxTranslationGain, 1, 4, 1, 1);

        label_11 = new QLabel(scrollAreaWidgetInputContents);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setAlignment(Qt::AlignCenter);

        gridLayout_11->addWidget(label_11, 0, 1, 1, 1);

        label_16 = new QLabel(scrollAreaWidgetInputContents);
        label_16->setObjectName(QString::fromUtf8("label_16"));
        label_16->setAlignment(Qt::AlignCenter);

        gridLayout_11->addWidget(label_16, 0, 4, 1, 1);

        doubleSpinBoxRotateGain = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxRotateGain->setObjectName(QString::fromUtf8("doubleSpinBoxRotateGain"));
        doubleSpinBoxRotateGain->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxRotateGain->setMinimum(0.100000000000000);
        doubleSpinBoxRotateGain->setMaximum(9.990000000000000);
        doubleSpinBoxRotateGain->setSingleStep(0.100000000000000);
        doubleSpinBoxRotateGain->setValue(1.000000000000000);

        gridLayout_11->addWidget(doubleSpinBoxRotateGain, 3, 4, 1, 1);

        comboBoxRotationY = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxRotationY->setObjectName(QString::fromUtf8("comboBoxRotationY"));
        sizePolicy2.setHeightForWidth(comboBoxRotationY->sizePolicy().hasHeightForWidth());
        comboBoxRotationY->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxRotationY, 3, 2, 1, 1);

        label_14 = new QLabel(scrollAreaWidgetInputContents);
        label_14->setObjectName(QString::fromUtf8("label_14"));
        label_14->setAlignment(Qt::AlignCenter);

        gridLayout_11->addWidget(label_14, 0, 2, 1, 1);

        doubleSpinBoxZoomGain = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxZoomGain->setObjectName(QString::fromUtf8("doubleSpinBoxZoomGain"));
        doubleSpinBoxZoomGain->setMinimum(0.100000000000000);
        doubleSpinBoxZoomGain->setMaximum(9.990000000000000);
        doubleSpinBoxZoomGain->setSingleStep(0.100000000000000);
        doubleSpinBoxZoomGain->setValue(1.000000000000000);

        gridLayout_11->addWidget(doubleSpinBoxZoomGain, 6, 4, 1, 1);

        doubleSpinBoxTranslationVPRelGain = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxTranslationVPRelGain->setObjectName(QString::fromUtf8("doubleSpinBoxTranslationVPRelGain"));
        doubleSpinBoxTranslationVPRelGain->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxTranslationVPRelGain->setMinimum(0.100000000000000);
        doubleSpinBoxTranslationVPRelGain->setMaximum(9.990000000000000);
        doubleSpinBoxTranslationVPRelGain->setSingleStep(0.100000000000000);
        doubleSpinBoxTranslationVPRelGain->setValue(1.000000000000000);

        gridLayout_11->addWidget(doubleSpinBoxTranslationVPRelGain, 2, 4, 1, 1);

        label_15 = new QLabel(scrollAreaWidgetInputContents);
        label_15->setObjectName(QString::fromUtf8("label_15"));
        label_15->setAlignment(Qt::AlignCenter);

        gridLayout_11->addWidget(label_15, 0, 3, 1, 1);

        comboBoxTranslationY = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxTranslationY->setObjectName(QString::fromUtf8("comboBoxTranslationY"));
        sizePolicy2.setHeightForWidth(comboBoxTranslationY->sizePolicy().hasHeightForWidth());
        comboBoxTranslationY->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxTranslationY, 1, 2, 1, 1);

        comboBoxTranslationYVPRel = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxTranslationYVPRel->setObjectName(QString::fromUtf8("comboBoxTranslationYVPRel"));
        sizePolicy2.setHeightForWidth(comboBoxTranslationYVPRel->sizePolicy().hasHeightForWidth());
        comboBoxTranslationYVPRel->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxTranslationYVPRel, 2, 2, 1, 1);

        comboBoxTranslationXVPRel = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxTranslationXVPRel->setObjectName(QString::fromUtf8("comboBoxTranslationXVPRel"));
        sizePolicy2.setHeightForWidth(comboBoxTranslationXVPRel->sizePolicy().hasHeightForWidth());
        comboBoxTranslationXVPRel->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxTranslationXVPRel, 2, 1, 1, 1);

        labelInputTranslation = new QLabel(scrollAreaWidgetInputContents);
        labelInputTranslation->setObjectName(QString::fromUtf8("labelInputTranslation"));

        gridLayout_11->addWidget(labelInputTranslation, 1, 0, 1, 1);

        comboBoxRotationZ = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxRotationZ->setObjectName(QString::fromUtf8("comboBoxRotationZ"));
        sizePolicy2.setHeightForWidth(comboBoxRotationZ->sizePolicy().hasHeightForWidth());
        comboBoxRotationZ->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxRotationZ, 3, 3, 1, 1);

        labelInputTranslationViewPortRelative = new QLabel(scrollAreaWidgetInputContents);
        labelInputTranslationViewPortRelative->setObjectName(QString::fromUtf8("labelInputTranslationViewPortRelative"));

        gridLayout_11->addWidget(labelInputTranslationViewPortRelative, 2, 0, 1, 1);

        comboBoxZoom2 = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxZoom2->setObjectName(QString::fromUtf8("comboBoxZoom2"));
        sizePolicy2.setHeightForWidth(comboBoxZoom2->sizePolicy().hasHeightForWidth());
        comboBoxZoom2->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxZoom2, 6, 2, 1, 1);

        line_2 = new QFrame(scrollAreaWidgetInputContents);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        gridLayout_11->addWidget(line_2, 5, 1, 1, 3);

        comboBoxRotationXVPRel = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxRotationXVPRel->setObjectName(QString::fromUtf8("comboBoxRotationXVPRel"));
        sizePolicy2.setHeightForWidth(comboBoxRotationXVPRel->sizePolicy().hasHeightForWidth());
        comboBoxRotationXVPRel->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxRotationXVPRel, 4, 1, 1, 1);

        labelInputRotationViewPortRelative = new QLabel(scrollAreaWidgetInputContents);
        labelInputRotationViewPortRelative->setObjectName(QString::fromUtf8("labelInputRotationViewPortRelative"));

        gridLayout_11->addWidget(labelInputRotationViewPortRelative, 4, 0, 1, 1);

        comboBoxRotationYVPRel = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxRotationYVPRel->setObjectName(QString::fromUtf8("comboBoxRotationYVPRel"));
        sizePolicy2.setHeightForWidth(comboBoxRotationYVPRel->sizePolicy().hasHeightForWidth());
        comboBoxRotationYVPRel->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxRotationYVPRel, 4, 2, 1, 1);

        comboBoxRotationZVPRel = new QComboBox(scrollAreaWidgetInputContents);
        comboBoxRotationZVPRel->setObjectName(QString::fromUtf8("comboBoxRotationZVPRel"));
        sizePolicy2.setHeightForWidth(comboBoxRotationZVPRel->sizePolicy().hasHeightForWidth());
        comboBoxRotationZVPRel->setSizePolicy(sizePolicy2);

        gridLayout_11->addWidget(comboBoxRotationZVPRel, 4, 3, 1, 1);

        doubleSpinBoxRotateVPRelGain = new QDoubleSpinBox(scrollAreaWidgetInputContents);
        doubleSpinBoxRotateVPRelGain->setObjectName(QString::fromUtf8("doubleSpinBoxRotateVPRelGain"));
        doubleSpinBoxRotateVPRelGain->setCorrectionMode(QAbstractSpinBox::CorrectToNearestValue);
        doubleSpinBoxRotateVPRelGain->setMinimum(0.100000000000000);
        doubleSpinBoxRotateVPRelGain->setMaximum(9.990000000000000);
        doubleSpinBoxRotateVPRelGain->setSingleStep(0.100000000000000);
        doubleSpinBoxRotateVPRelGain->setValue(1.000000000000000);

        gridLayout_11->addWidget(doubleSpinBoxRotateVPRelGain, 4, 4, 1, 1);


        gridLayout_7->addLayout(gridLayout_11, 13, 0, 1, 2);

        label_3 = new QLabel(scrollAreaWidgetInputContents);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_3->setFont(font1);

        gridLayout_7->addWidget(label_3, 14, 0, 1, 2);

        label = new QLabel(scrollAreaWidgetInputContents);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout_7->addWidget(label, 0, 0, 1, 2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        checkBoxSpaceNav = new QCheckBox(scrollAreaWidgetInputContents);
        checkBoxSpaceNav->setObjectName(QString::fromUtf8("checkBoxSpaceNav"));
        checkBoxSpaceNav->setEnabled(false);

        gridLayout_2->addWidget(checkBoxSpaceNav, 2, 1, 1, 1);

        checkBoxHIDAPI = new QCheckBox(scrollAreaWidgetInputContents);
        checkBoxHIDAPI->setObjectName(QString::fromUtf8("checkBoxHIDAPI"));
        checkBoxHIDAPI->setEnabled(false);

        gridLayout_2->addWidget(checkBoxHIDAPI, 2, 0, 1, 1);

        checkBoxQGamepad = new QCheckBox(scrollAreaWidgetInputContents);
        checkBoxQGamepad->setObjectName(QString::fromUtf8("checkBoxQGamepad"));
        checkBoxQGamepad->setEnabled(false);

        gridLayout_2->addWidget(checkBoxQGamepad, 2, 3, 1, 1);

        checkBoxJoystick = new QCheckBox(scrollAreaWidgetInputContents);
        checkBoxJoystick->setObjectName(QString::fromUtf8("checkBoxJoystick"));
        checkBoxJoystick->setEnabled(false);

        gridLayout_2->addWidget(checkBoxJoystick, 2, 2, 1, 1);

        checkBoxDBus = new QCheckBox(scrollAreaWidgetInputContents);
        checkBoxDBus->setObjectName(QString::fromUtf8("checkBoxDBus"));
        checkBoxDBus->setEnabled(false);

        gridLayout_2->addWidget(checkBoxDBus, 2, 4, 1, 1);


        gridLayout_7->addLayout(gridLayout_2, 1, 0, 1, 2);

        label_2 = new QLabel(scrollAreaWidgetInputContents);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setFont(font1);

        gridLayout_7->addWidget(label_2, 2, 0, 1, 2);

        label_4 = new QLabel(scrollAreaWidgetInputContents);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setFont(font1);

        gridLayout_7->addWidget(label_4, 34, 0, 1, 2);

        label_driverInfo = new QLabel(scrollAreaWidgetInputContents);
        label_driverInfo->setObjectName(QString::fromUtf8("label_driverInfo"));

        gridLayout_7->addWidget(label_driverInfo, 35, 0, 1, 2);

        pushButtonUpdate = new QPushButton(scrollAreaWidgetInputContents);
        pushButtonUpdate->setObjectName(QString::fromUtf8("pushButtonUpdate"));

        gridLayout_7->addWidget(pushButtonUpdate, 36, 0, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_7->addItem(verticalSpacer_4, 37, 0, 1, 1);

        scrollArea_InputAxis->setWidget(scrollAreaWidgetInputContents);

        gridLayout->addWidget(scrollArea_InputAxis, 0, 0, 1, 1);


        retranslateUi(Axis);

        QMetaObject::connectSlotsByName(Axis);
    } // setupUi

    void retranslateUi(QWidget *Axis)
    {
        Axis->setWindowTitle(q_("Preferences", nullptr));
        labelAxisTrim3->setText(q_("Trim", nullptr));
        labelAxisTrim0->setText(q_("Trim", nullptr));
        pushButtonAxisTrimReset->setText(q_("Reset Trim", nullptr));
        labelAxisDeadzone0->setText(q_("DeadZone", nullptr));
        pushButtonAxisTrim->setText(q_("Auto Trim Axes", nullptr));
        labelAxisTrim6->setText(q_("Trim", nullptr));
        labelAxisDeadzone3->setText(q_("DeadZone", nullptr));
        labelAxisDeadzone6->setText(q_("DeadZone", nullptr));
        labelAxisDeadzone7->setText(q_("DeadZone", nullptr));
        labelAxisTrim7->setText(q_("Trim", nullptr));
        labelAxisDeadzone8->setText(q_("DeadZone", nullptr));
        labelAxisTrim8->setText(q_("Trim", nullptr));
        labelAxisDeadzone5->setText(q_("DeadZone", nullptr));
        labelAxisTrim5->setText(q_("Trim", nullptr));
        labelAxisDeadzone4->setText(q_("DeadZone", nullptr));
        labelAxisTrim4->setText(q_("Trim", nullptr));
        labelAxisDeadzone2->setText(q_("DeadZone", nullptr));
        labelAxisTrim2->setText(q_("Trim", nullptr));
        labelAxis2->setText(q_("Axis 2", nullptr));
        labelAxis0->setText(q_("Axis 0", nullptr));
        labelAxis3->setText(q_("Axis 3", nullptr));
        labelAxis6->setText(q_("Axis 6", nullptr));
        labelAxis8->setText(q_("Axis 8", nullptr));
        labelAxis7->setText(q_("Axis 7", nullptr));
        labelAxis5->setText(q_("Axis 5", nullptr));
        labelAxis1->setText(q_("Axis 1", nullptr));
        labelAxis4->setText(q_("Axis 4", nullptr));
        labelAxisDeadzone1->setText(q_("DeadZone", nullptr));
        labelAxisTrim1->setText(q_("Trim", nullptr));
        labelInputRotation->setText(q_("Rotation", nullptr));
        labelInputZoom->setText(q_("Zoom", nullptr));
        label_11->setText(q_("X", nullptr));
        label_16->setText(q_("Gain", nullptr));
        label_14->setText(q_("Y", nullptr));
        label_15->setText(q_("Z", nullptr));
        labelInputTranslation->setText(q_("Translation", nullptr));
        labelInputTranslationViewPortRelative->setText(q_("ViewPort rel.<br/>Translation", nullptr));
        labelInputRotationViewPortRelative->setText(q_("ViewPort rel.<br />Rotation", nullptr));
        label_3->setText(q_("Axis Setup:", nullptr));
        label->setText(q_("<b>Driver selection:</b> <i>(changes requires restarting OpenSCAD)</i>", nullptr));
        checkBoxSpaceNav->setText(q_("SpaceNav", nullptr));
        checkBoxHIDAPI->setText(q_("HIDAPI", nullptr));
        checkBoxQGamepad->setText(q_("QGamepad", nullptr));
        checkBoxJoystick->setText(q_("Joystick", nullptr));
        checkBoxDBus->setText(q_("DBus", nullptr));
        label_2->setText(q_("Axes Mapping:", nullptr));
        label_4->setText(q_("Status:", nullptr));
        label_driverInfo->setText(q_("TextLabel", nullptr));
        pushButtonUpdate->setText(q_("Update", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Axis: public Ui_Axis {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_AXISCONFIGWIDGET_H
