/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../src/MainWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[174];
    char stringdata0[2832];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 14), // "highlightError"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 20), // "unhighlightLastError"
QT_MOC_LITERAL(4, 48, 20), // "setTabToolBarVisible"
QT_MOC_LITERAL(5, 69, 15), // "updatedAnimTval"
QT_MOC_LITERAL(6, 85, 14), // "updatedAnimFps"
QT_MOC_LITERAL(7, 100, 16), // "updatedAnimSteps"
QT_MOC_LITERAL(8, 117, 15), // "updatedAnimDump"
QT_MOC_LITERAL(9, 133, 7), // "checked"
QT_MOC_LITERAL(10, 141, 10), // "updateTVal"
QT_MOC_LITERAL(11, 152, 16), // "updateUndockMode"
QT_MOC_LITERAL(12, 169, 10), // "undockMode"
QT_MOC_LITERAL(13, 180, 17), // "updateReorderMode"
QT_MOC_LITERAL(14, 198, 11), // "reorderMode"
QT_MOC_LITERAL(15, 210, 7), // "setFont"
QT_MOC_LITERAL(16, 218, 6), // "family"
QT_MOC_LITERAL(17, 225, 4), // "size"
QT_MOC_LITERAL(18, 230, 14), // "setColorScheme"
QT_MOC_LITERAL(19, 245, 2), // "cs"
QT_MOC_LITERAL(20, 248, 12), // "showProgress"
QT_MOC_LITERAL(21, 261, 22), // "openCSGSettingsChanged"
QT_MOC_LITERAL(22, 284, 13), // "consoleOutput"
QT_MOC_LITERAL(23, 298, 7), // "Message"
QT_MOC_LITERAL(24, 306, 6), // "msgObj"
QT_MOC_LITERAL(25, 313, 9), // "setCursor"
QT_MOC_LITERAL(26, 323, 14), // "errorLogOutput"
QT_MOC_LITERAL(27, 338, 7), // "log_msg"
QT_MOC_LITERAL(28, 346, 17), // "updateRecentFiles"
QT_MOC_LITERAL(29, 364, 16), // "EditorInterface*"
QT_MOC_LITERAL(30, 381, 3), // "edt"
QT_MOC_LITERAL(31, 385, 23), // "updateRecentFileActions"
QT_MOC_LITERAL(32, 409, 14), // "handleFileDrop"
QT_MOC_LITERAL(33, 424, 3), // "url"
QT_MOC_LITERAL(34, 428, 10), // "actionOpen"
QT_MOC_LITERAL(35, 439, 15), // "actionNewWindow"
QT_MOC_LITERAL(36, 455, 16), // "actionOpenWindow"
QT_MOC_LITERAL(37, 472, 16), // "actionOpenRecent"
QT_MOC_LITERAL(38, 489, 17), // "actionOpenExample"
QT_MOC_LITERAL(39, 507, 16), // "clearRecentFiles"
QT_MOC_LITERAL(40, 524, 10), // "actionSave"
QT_MOC_LITERAL(41, 535, 12), // "actionSaveAs"
QT_MOC_LITERAL(42, 548, 12), // "actionReload"
QT_MOC_LITERAL(43, 561, 23), // "actionShowLibraryFolder"
QT_MOC_LITERAL(44, 585, 19), // "convertTabsToSpaces"
QT_MOC_LITERAL(45, 605, 15), // "instantiateRoot"
QT_MOC_LITERAL(46, 621, 11), // "compileDone"
QT_MOC_LITERAL(47, 633, 9), // "didchange"
QT_MOC_LITERAL(48, 643, 12), // "compileEnded"
QT_MOC_LITERAL(49, 656, 21), // "changeParameterWidget"
QT_MOC_LITERAL(50, 678, 23), // "copyViewportTranslation"
QT_MOC_LITERAL(51, 702, 20), // "copyViewportRotation"
QT_MOC_LITERAL(52, 723, 20), // "copyViewportDistance"
QT_MOC_LITERAL(53, 744, 15), // "copyViewportFov"
QT_MOC_LITERAL(54, 760, 11), // "preferences"
QT_MOC_LITERAL(55, 772, 17), // "hideEditorToolbar"
QT_MOC_LITERAL(56, 790, 17), // "hide3DViewToolbar"
QT_MOC_LITERAL(57, 808, 8), // "showLink"
QT_MOC_LITERAL(58, 817, 10), // "showEditor"
QT_MOC_LITERAL(59, 828, 10), // "hideEditor"
QT_MOC_LITERAL(60, 839, 11), // "showConsole"
QT_MOC_LITERAL(61, 851, 11), // "hideConsole"
QT_MOC_LITERAL(62, 863, 12), // "showErrorLog"
QT_MOC_LITERAL(63, 876, 12), // "hideErrorLog"
QT_MOC_LITERAL(64, 889, 14), // "showParameters"
QT_MOC_LITERAL(65, 904, 14), // "hideParameters"
QT_MOC_LITERAL(66, 919, 37), // "on_windowActionSelectEditor_t..."
QT_MOC_LITERAL(67, 957, 38), // "on_windowActionSelectConsole_..."
QT_MOC_LITERAL(68, 996, 41), // "on_windowActionSelectCustomiz..."
QT_MOC_LITERAL(69, 1038, 39), // "on_windowActionSelectErrorLog..."
QT_MOC_LITERAL(70, 1078, 35), // "on_windowActionNextWindow_tri..."
QT_MOC_LITERAL(71, 1114, 39), // "on_windowActionPreviousWindow..."
QT_MOC_LITERAL(72, 1154, 37), // "on_editActionInsertTemplate_t..."
QT_MOC_LITERAL(73, 1192, 8), // "hideFind"
QT_MOC_LITERAL(74, 1201, 8), // "showFind"
QT_MOC_LITERAL(75, 1210, 18), // "showFindAndReplace"
QT_MOC_LITERAL(76, 1229, 14), // "selectFindType"
QT_MOC_LITERAL(77, 1244, 10), // "findString"
QT_MOC_LITERAL(78, 1255, 8), // "findNext"
QT_MOC_LITERAL(79, 1264, 8), // "findPrev"
QT_MOC_LITERAL(80, 1273, 19), // "useSelectionForFind"
QT_MOC_LITERAL(81, 1293, 7), // "replace"
QT_MOC_LITERAL(82, 1301, 10), // "replaceAll"
QT_MOC_LITERAL(83, 1312, 17), // "findBufferChanged"
QT_MOC_LITERAL(84, 1330, 16), // "updateFindBuffer"
QT_MOC_LITERAL(85, 1347, 5), // "event"
QT_MOC_LITERAL(86, 1353, 7), // "QEvent*"
QT_MOC_LITERAL(87, 1361, 19), // "actionRenderPreview"
QT_MOC_LITERAL(88, 1381, 22), // "rebuildParameterWidget"
QT_MOC_LITERAL(89, 1404, 9), // "csgRender"
QT_MOC_LITERAL(90, 1414, 15), // "csgReloadRender"
QT_MOC_LITERAL(91, 1430, 13), // "action3DPrint"
QT_MOC_LITERAL(92, 1444, 15), // "sendToOctoPrint"
QT_MOC_LITERAL(93, 1460, 18), // "sendToPrintService"
QT_MOC_LITERAL(94, 1479, 12), // "actionRender"
QT_MOC_LITERAL(95, 1492, 16), // "actionRenderDone"
QT_MOC_LITERAL(96, 1509, 26), // "shared_ptr<const Geometry>"
QT_MOC_LITERAL(97, 1536, 10), // "cgalRender"
QT_MOC_LITERAL(98, 1547, 19), // "actionCheckValidity"
QT_MOC_LITERAL(99, 1567, 16), // "actionDisplayAST"
QT_MOC_LITERAL(100, 1584, 20), // "actionDisplayCSGTree"
QT_MOC_LITERAL(101, 1605, 24), // "actionDisplayCSGProducts"
QT_MOC_LITERAL(102, 1630, 9), // "canExport"
QT_MOC_LITERAL(103, 1640, 3), // "dim"
QT_MOC_LITERAL(104, 1644, 12), // "actionExport"
QT_MOC_LITERAL(105, 1657, 10), // "FileFormat"
QT_MOC_LITERAL(106, 1668, 6), // "format"
QT_MOC_LITERAL(107, 1675, 11), // "const char*"
QT_MOC_LITERAL(108, 1687, 9), // "type_name"
QT_MOC_LITERAL(109, 1697, 6), // "suffix"
QT_MOC_LITERAL(110, 1704, 15), // "actionExportSTL"
QT_MOC_LITERAL(111, 1720, 15), // "actionExport3MF"
QT_MOC_LITERAL(112, 1736, 15), // "actionExportOFF"
QT_MOC_LITERAL(113, 1752, 15), // "actionExportAMF"
QT_MOC_LITERAL(114, 1768, 15), // "actionExportDXF"
QT_MOC_LITERAL(115, 1784, 15), // "actionExportSVG"
QT_MOC_LITERAL(116, 1800, 15), // "actionExportPDF"
QT_MOC_LITERAL(117, 1816, 15), // "actionExportCSG"
QT_MOC_LITERAL(118, 1832, 17), // "actionExportImage"
QT_MOC_LITERAL(119, 1850, 18), // "actionCopyViewport"
QT_MOC_LITERAL(120, 1869, 17), // "actionFlushCaches"
QT_MOC_LITERAL(121, 1887, 25), // "actionReloadRenderPreview"
QT_MOC_LITERAL(122, 1913, 31), // "on_editorDock_visibilityChanged"
QT_MOC_LITERAL(123, 1945, 32), // "on_consoleDock_visibilityChanged"
QT_MOC_LITERAL(124, 1978, 34), // "on_parameterDock_visibilityCh..."
QT_MOC_LITERAL(125, 2013, 33), // "on_errorLogDock_visibilityCha..."
QT_MOC_LITERAL(126, 2047, 39), // "on_toolButtonCompileResultClo..."
QT_MOC_LITERAL(127, 2087, 21), // "editorTopLevelChanged"
QT_MOC_LITERAL(128, 2109, 22), // "consoleTopLevelChanged"
QT_MOC_LITERAL(129, 2132, 24), // "parameterTopLevelChanged"
QT_MOC_LITERAL(130, 2157, 23), // "errorLogTopLevelChanged"
QT_MOC_LITERAL(131, 2181, 13), // "processEvents"
QT_MOC_LITERAL(132, 2195, 10), // "jumpToLine"
QT_MOC_LITERAL(133, 2206, 16), // "openFileFromPath"
QT_MOC_LITERAL(134, 2223, 15), // "viewModePreview"
QT_MOC_LITERAL(135, 2239, 15), // "viewModeSurface"
QT_MOC_LITERAL(136, 2255, 17), // "viewModeWireframe"
QT_MOC_LITERAL(137, 2273, 22), // "viewModeThrownTogether"
QT_MOC_LITERAL(138, 2296, 17), // "viewModeShowEdges"
QT_MOC_LITERAL(139, 2314, 16), // "viewModeShowAxes"
QT_MOC_LITERAL(140, 2331, 22), // "viewModeShowCrosshairs"
QT_MOC_LITERAL(141, 2354, 29), // "viewModeShowScaleProportional"
QT_MOC_LITERAL(142, 2384, 15), // "viewModeAnimate"
QT_MOC_LITERAL(143, 2400, 12), // "viewAngleTop"
QT_MOC_LITERAL(144, 2413, 15), // "viewAngleBottom"
QT_MOC_LITERAL(145, 2429, 13), // "viewAngleLeft"
QT_MOC_LITERAL(146, 2443, 14), // "viewAngleRight"
QT_MOC_LITERAL(147, 2458, 14), // "viewAngleFront"
QT_MOC_LITERAL(148, 2473, 13), // "viewAngleBack"
QT_MOC_LITERAL(149, 2487, 17), // "viewAngleDiagonal"
QT_MOC_LITERAL(150, 2505, 10), // "viewCenter"
QT_MOC_LITERAL(151, 2516, 15), // "viewPerspective"
QT_MOC_LITERAL(152, 2532, 14), // "viewOrthogonal"
QT_MOC_LITERAL(153, 2547, 21), // "viewTogglePerspective"
QT_MOC_LITERAL(154, 2569, 13), // "viewResetView"
QT_MOC_LITERAL(155, 2583, 7), // "viewAll"
QT_MOC_LITERAL(156, 2591, 23), // "animateUpdateDocChanged"
QT_MOC_LITERAL(157, 2615, 13), // "animateUpdate"
QT_MOC_LITERAL(158, 2629, 12), // "selectObject"
QT_MOC_LITERAL(159, 2642, 10), // "coordinate"
QT_MOC_LITERAL(160, 2653, 14), // "dragEnterEvent"
QT_MOC_LITERAL(161, 2668, 16), // "QDragEnterEvent*"
QT_MOC_LITERAL(162, 2685, 9), // "dropEvent"
QT_MOC_LITERAL(163, 2695, 11), // "QDropEvent*"
QT_MOC_LITERAL(164, 2707, 9), // "helpAbout"
QT_MOC_LITERAL(165, 2717, 12), // "helpHomepage"
QT_MOC_LITERAL(166, 2730, 10), // "helpManual"
QT_MOC_LITERAL(167, 2741, 14), // "helpCheatSheet"
QT_MOC_LITERAL(168, 2756, 11), // "helpLibrary"
QT_MOC_LITERAL(169, 2768, 12), // "helpFontInfo"
QT_MOC_LITERAL(170, 2781, 4), // "quit"
QT_MOC_LITERAL(171, 2786, 15), // "checkAutoReload"
QT_MOC_LITERAL(172, 2802, 15), // "waitAfterReload"
QT_MOC_LITERAL(173, 2818, 13) // "autoReloadSet"

    },
    "MainWindow\0highlightError\0\0"
    "unhighlightLastError\0setTabToolBarVisible\0"
    "updatedAnimTval\0updatedAnimFps\0"
    "updatedAnimSteps\0updatedAnimDump\0"
    "checked\0updateTVal\0updateUndockMode\0"
    "undockMode\0updateReorderMode\0reorderMode\0"
    "setFont\0family\0size\0setColorScheme\0"
    "cs\0showProgress\0openCSGSettingsChanged\0"
    "consoleOutput\0Message\0msgObj\0setCursor\0"
    "errorLogOutput\0log_msg\0updateRecentFiles\0"
    "EditorInterface*\0edt\0updateRecentFileActions\0"
    "handleFileDrop\0url\0actionOpen\0"
    "actionNewWindow\0actionOpenWindow\0"
    "actionOpenRecent\0actionOpenExample\0"
    "clearRecentFiles\0actionSave\0actionSaveAs\0"
    "actionReload\0actionShowLibraryFolder\0"
    "convertTabsToSpaces\0instantiateRoot\0"
    "compileDone\0didchange\0compileEnded\0"
    "changeParameterWidget\0copyViewportTranslation\0"
    "copyViewportRotation\0copyViewportDistance\0"
    "copyViewportFov\0preferences\0"
    "hideEditorToolbar\0hide3DViewToolbar\0"
    "showLink\0showEditor\0hideEditor\0"
    "showConsole\0hideConsole\0showErrorLog\0"
    "hideErrorLog\0showParameters\0hideParameters\0"
    "on_windowActionSelectEditor_triggered\0"
    "on_windowActionSelectConsole_triggered\0"
    "on_windowActionSelectCustomizer_triggered\0"
    "on_windowActionSelectErrorLog_triggered\0"
    "on_windowActionNextWindow_triggered\0"
    "on_windowActionPreviousWindow_triggered\0"
    "on_editActionInsertTemplate_triggered\0"
    "hideFind\0showFind\0showFindAndReplace\0"
    "selectFindType\0findString\0findNext\0"
    "findPrev\0useSelectionForFind\0replace\0"
    "replaceAll\0findBufferChanged\0"
    "updateFindBuffer\0event\0QEvent*\0"
    "actionRenderPreview\0rebuildParameterWidget\0"
    "csgRender\0csgReloadRender\0action3DPrint\0"
    "sendToOctoPrint\0sendToPrintService\0"
    "actionRender\0actionRenderDone\0"
    "shared_ptr<const Geometry>\0cgalRender\0"
    "actionCheckValidity\0actionDisplayAST\0"
    "actionDisplayCSGTree\0actionDisplayCSGProducts\0"
    "canExport\0dim\0actionExport\0FileFormat\0"
    "format\0const char*\0type_name\0suffix\0"
    "actionExportSTL\0actionExport3MF\0"
    "actionExportOFF\0actionExportAMF\0"
    "actionExportDXF\0actionExportSVG\0"
    "actionExportPDF\0actionExportCSG\0"
    "actionExportImage\0actionCopyViewport\0"
    "actionFlushCaches\0actionReloadRenderPreview\0"
    "on_editorDock_visibilityChanged\0"
    "on_consoleDock_visibilityChanged\0"
    "on_parameterDock_visibilityChanged\0"
    "on_errorLogDock_visibilityChanged\0"
    "on_toolButtonCompileResultClose_clicked\0"
    "editorTopLevelChanged\0consoleTopLevelChanged\0"
    "parameterTopLevelChanged\0"
    "errorLogTopLevelChanged\0processEvents\0"
    "jumpToLine\0openFileFromPath\0viewModePreview\0"
    "viewModeSurface\0viewModeWireframe\0"
    "viewModeThrownTogether\0viewModeShowEdges\0"
    "viewModeShowAxes\0viewModeShowCrosshairs\0"
    "viewModeShowScaleProportional\0"
    "viewModeAnimate\0viewAngleTop\0"
    "viewAngleBottom\0viewAngleLeft\0"
    "viewAngleRight\0viewAngleFront\0"
    "viewAngleBack\0viewAngleDiagonal\0"
    "viewCenter\0viewPerspective\0viewOrthogonal\0"
    "viewTogglePerspective\0viewResetView\0"
    "viewAll\0animateUpdateDocChanged\0"
    "animateUpdate\0selectObject\0coordinate\0"
    "dragEnterEvent\0QDragEnterEvent*\0"
    "dropEvent\0QDropEvent*\0helpAbout\0"
    "helpHomepage\0helpManual\0helpCheatSheet\0"
    "helpLibrary\0helpFontInfo\0quit\0"
    "checkAutoReload\0waitAfterReload\0"
    "autoReloadSet"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     148,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  754,    2, 0x06 /* Public */,
       3,    0,  757,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,  758,    2, 0x08 /* Private */,
       5,    0,  761,    2, 0x08 /* Private */,
       6,    0,  762,    2, 0x08 /* Private */,
       7,    0,  763,    2, 0x08 /* Private */,
       8,    1,  764,    2, 0x08 /* Private */,
      10,    0,  767,    2, 0x08 /* Private */,
      11,    1,  768,    2, 0x08 /* Private */,
      13,    1,  771,    2, 0x08 /* Private */,
      15,    2,  774,    2, 0x08 /* Private */,
      18,    1,  779,    2, 0x08 /* Private */,
      20,    0,  782,    2, 0x08 /* Private */,
      21,    0,  783,    2, 0x08 /* Private */,
      22,    1,  784,    2, 0x08 /* Private */,
      25,    0,  787,    2, 0x08 /* Private */,
      26,    1,  788,    2, 0x08 /* Private */,
      28,    1,  791,    2, 0x0a /* Public */,
      31,    0,  794,    2, 0x0a /* Public */,
      32,    1,  795,    2, 0x0a /* Public */,
      34,    0,  798,    2, 0x08 /* Private */,
      35,    0,  799,    2, 0x08 /* Private */,
      36,    0,  800,    2, 0x08 /* Private */,
      37,    0,  801,    2, 0x08 /* Private */,
      38,    0,  802,    2, 0x08 /* Private */,
      39,    0,  803,    2, 0x08 /* Private */,
      40,    0,  804,    2, 0x08 /* Private */,
      41,    0,  805,    2, 0x08 /* Private */,
      42,    0,  806,    2, 0x08 /* Private */,
      43,    0,  807,    2, 0x08 /* Private */,
      44,    0,  808,    2, 0x08 /* Private */,
      45,    0,  809,    2, 0x08 /* Private */,
      46,    1,  810,    2, 0x08 /* Private */,
      48,    0,  813,    2, 0x08 /* Private */,
      49,    0,  814,    2, 0x08 /* Private */,
      50,    0,  815,    2, 0x08 /* Private */,
      51,    0,  816,    2, 0x08 /* Private */,
      52,    0,  817,    2, 0x08 /* Private */,
      53,    0,  818,    2, 0x08 /* Private */,
      54,    0,  819,    2, 0x08 /* Private */,
      55,    0,  820,    2, 0x08 /* Private */,
      56,    0,  821,    2, 0x08 /* Private */,
      57,    1,  822,    2, 0x08 /* Private */,
      58,    0,  825,    2, 0x08 /* Private */,
      59,    0,  826,    2, 0x08 /* Private */,
      60,    0,  827,    2, 0x08 /* Private */,
      61,    0,  828,    2, 0x08 /* Private */,
      62,    0,  829,    2, 0x08 /* Private */,
      63,    0,  830,    2, 0x08 /* Private */,
      64,    0,  831,    2, 0x08 /* Private */,
      65,    0,  832,    2, 0x08 /* Private */,
      66,    0,  833,    2, 0x08 /* Private */,
      67,    0,  834,    2, 0x08 /* Private */,
      68,    0,  835,    2, 0x08 /* Private */,
      69,    0,  836,    2, 0x08 /* Private */,
      70,    0,  837,    2, 0x08 /* Private */,
      71,    0,  838,    2, 0x08 /* Private */,
      72,    0,  839,    2, 0x08 /* Private */,
      73,    0,  840,    2, 0x0a /* Public */,
      74,    0,  841,    2, 0x0a /* Public */,
      75,    0,  842,    2, 0x0a /* Public */,
      76,    1,  843,    2, 0x08 /* Private */,
      77,    1,  846,    2, 0x08 /* Private */,
      78,    0,  849,    2, 0x08 /* Private */,
      79,    0,  850,    2, 0x08 /* Private */,
      80,    0,  851,    2, 0x08 /* Private */,
      81,    0,  852,    2, 0x08 /* Private */,
      82,    0,  853,    2, 0x08 /* Private */,
      83,    0,  854,    2, 0x08 /* Private */,
      84,    1,  855,    2, 0x08 /* Private */,
      85,    1,  858,    2, 0x08 /* Private */,
      87,    1,  861,    2, 0x08 /* Private */,
      87,    0,  864,    2, 0x28 /* Private | MethodCloned */,
      89,    0,  865,    2, 0x08 /* Private */,
      90,    0,  866,    2, 0x08 /* Private */,
      91,    0,  867,    2, 0x08 /* Private */,
      92,    0,  868,    2, 0x08 /* Private */,
      93,    0,  869,    2, 0x08 /* Private */,
      94,    0,  870,    2, 0x08 /* Private */,
      95,    1,  871,    2, 0x08 /* Private */,
      97,    0,  874,    2, 0x08 /* Private */,
      98,    0,  875,    2, 0x08 /* Private */,
      99,    0,  876,    2, 0x08 /* Private */,
     100,    0,  877,    2, 0x08 /* Private */,
     101,    0,  878,    2, 0x08 /* Private */,
     102,    1,  879,    2, 0x08 /* Private */,
     104,    4,  882,    2, 0x08 /* Private */,
     110,    0,  891,    2, 0x08 /* Private */,
     111,    0,  892,    2, 0x08 /* Private */,
     112,    0,  893,    2, 0x08 /* Private */,
     113,    0,  894,    2, 0x08 /* Private */,
     114,    0,  895,    2, 0x08 /* Private */,
     115,    0,  896,    2, 0x08 /* Private */,
     116,    0,  897,    2, 0x08 /* Private */,
     117,    0,  898,    2, 0x08 /* Private */,
     118,    0,  899,    2, 0x08 /* Private */,
     119,    0,  900,    2, 0x08 /* Private */,
     120,    0,  901,    2, 0x08 /* Private */,
     121,    0,  902,    2, 0x0a /* Public */,
     122,    1,  903,    2, 0x0a /* Public */,
     123,    1,  906,    2, 0x0a /* Public */,
     124,    1,  909,    2, 0x0a /* Public */,
     125,    1,  912,    2, 0x0a /* Public */,
     126,    0,  915,    2, 0x0a /* Public */,
     127,    1,  916,    2, 0x0a /* Public */,
     128,    1,  919,    2, 0x0a /* Public */,
     129,    1,  922,    2, 0x0a /* Public */,
     130,    1,  925,    2, 0x0a /* Public */,
     131,    0,  928,    2, 0x0a /* Public */,
     132,    2,  929,    2, 0x0a /* Public */,
     133,    2,  934,    2, 0x0a /* Public */,
     134,    0,  939,    2, 0x0a /* Public */,
     135,    0,  940,    2, 0x0a /* Public */,
     136,    0,  941,    2, 0x0a /* Public */,
     137,    0,  942,    2, 0x0a /* Public */,
     138,    0,  943,    2, 0x0a /* Public */,
     139,    0,  944,    2, 0x0a /* Public */,
     140,    0,  945,    2, 0x0a /* Public */,
     141,    0,  946,    2, 0x0a /* Public */,
     142,    0,  947,    2, 0x0a /* Public */,
     143,    0,  948,    2, 0x0a /* Public */,
     144,    0,  949,    2, 0x0a /* Public */,
     145,    0,  950,    2, 0x0a /* Public */,
     146,    0,  951,    2, 0x0a /* Public */,
     147,    0,  952,    2, 0x0a /* Public */,
     148,    0,  953,    2, 0x0a /* Public */,
     149,    0,  954,    2, 0x0a /* Public */,
     150,    0,  955,    2, 0x0a /* Public */,
     151,    0,  956,    2, 0x0a /* Public */,
     152,    0,  957,    2, 0x0a /* Public */,
     153,    0,  958,    2, 0x0a /* Public */,
     154,    0,  959,    2, 0x0a /* Public */,
     155,    0,  960,    2, 0x0a /* Public */,
     156,    0,  961,    2, 0x0a /* Public */,
     157,    0,  962,    2, 0x0a /* Public */,
     158,    1,  963,    2, 0x0a /* Public */,
     160,    1,  966,    2, 0x0a /* Public */,
     162,    1,  969,    2, 0x0a /* Public */,
     164,    0,  972,    2, 0x0a /* Public */,
     165,    0,  973,    2, 0x0a /* Public */,
     166,    0,  974,    2, 0x0a /* Public */,
     167,    0,  975,    2, 0x0a /* Public */,
     168,    0,  976,    2, 0x0a /* Public */,
     169,    0,  977,    2, 0x0a /* Public */,
     170,    0,  978,    2, 0x0a /* Public */,
     171,    0,  979,    2, 0x0a /* Public */,
     172,    0,  980,    2, 0x0a /* Public */,
     173,    1,  981,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::UInt,   16,   17,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   27,
    QMetaType::Void, 0x80000000 | 29,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   33,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   47,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Bool, 0x80000000 | 86,   85,
    QMetaType::Void, QMetaType::Bool,   88,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 96,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::UInt,  103,
    QMetaType::Void, 0x80000000 | 105, 0x80000000 | 107, 0x80000000 | 107, QMetaType::UInt,  106,  108,  109,  103,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,  159,
    QMetaType::Void, 0x80000000 | 161,   85,
    QMetaType::Void, 0x80000000 | 163,   85,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->highlightError((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->unhighlightLastError(); break;
        case 2: _t->setTabToolBarVisible((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->updatedAnimTval(); break;
        case 4: _t->updatedAnimFps(); break;
        case 5: _t->updatedAnimSteps(); break;
        case 6: _t->updatedAnimDump((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->updateTVal(); break;
        case 8: _t->updateUndockMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->updateReorderMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->setFont((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 11: _t->setColorScheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->showProgress(); break;
        case 13: _t->openCSGSettingsChanged(); break;
        case 14: _t->consoleOutput((*reinterpret_cast< const Message(*)>(_a[1]))); break;
        case 15: _t->setCursor(); break;
        case 16: _t->errorLogOutput((*reinterpret_cast< const Message(*)>(_a[1]))); break;
        case 17: _t->updateRecentFiles((*reinterpret_cast< EditorInterface*(*)>(_a[1]))); break;
        case 18: _t->updateRecentFileActions(); break;
        case 19: _t->handleFileDrop((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 20: _t->actionOpen(); break;
        case 21: _t->actionNewWindow(); break;
        case 22: _t->actionOpenWindow(); break;
        case 23: _t->actionOpenRecent(); break;
        case 24: _t->actionOpenExample(); break;
        case 25: _t->clearRecentFiles(); break;
        case 26: _t->actionSave(); break;
        case 27: _t->actionSaveAs(); break;
        case 28: _t->actionReload(); break;
        case 29: _t->actionShowLibraryFolder(); break;
        case 30: _t->convertTabsToSpaces(); break;
        case 31: _t->instantiateRoot(); break;
        case 32: _t->compileDone((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 33: _t->compileEnded(); break;
        case 34: _t->changeParameterWidget(); break;
        case 35: _t->copyViewportTranslation(); break;
        case 36: _t->copyViewportRotation(); break;
        case 37: _t->copyViewportDistance(); break;
        case 38: _t->copyViewportFov(); break;
        case 39: _t->preferences(); break;
        case 40: _t->hideEditorToolbar(); break;
        case 41: _t->hide3DViewToolbar(); break;
        case 42: _t->showLink((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 43: _t->showEditor(); break;
        case 44: _t->hideEditor(); break;
        case 45: _t->showConsole(); break;
        case 46: _t->hideConsole(); break;
        case 47: _t->showErrorLog(); break;
        case 48: _t->hideErrorLog(); break;
        case 49: _t->showParameters(); break;
        case 50: _t->hideParameters(); break;
        case 51: _t->on_windowActionSelectEditor_triggered(); break;
        case 52: _t->on_windowActionSelectConsole_triggered(); break;
        case 53: _t->on_windowActionSelectCustomizer_triggered(); break;
        case 54: _t->on_windowActionSelectErrorLog_triggered(); break;
        case 55: _t->on_windowActionNextWindow_triggered(); break;
        case 56: _t->on_windowActionPreviousWindow_triggered(); break;
        case 57: _t->on_editActionInsertTemplate_triggered(); break;
        case 58: _t->hideFind(); break;
        case 59: _t->showFind(); break;
        case 60: _t->showFindAndReplace(); break;
        case 61: _t->selectFindType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 62: _t->findString((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 63: _t->findNext(); break;
        case 64: _t->findPrev(); break;
        case 65: _t->useSelectionForFind(); break;
        case 66: _t->replace(); break;
        case 67: _t->replaceAll(); break;
        case 68: _t->findBufferChanged(); break;
        case 69: _t->updateFindBuffer((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 70: { bool _r = _t->event((*reinterpret_cast< QEvent*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 71: _t->actionRenderPreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 72: _t->actionRenderPreview(); break;
        case 73: _t->csgRender(); break;
        case 74: _t->csgReloadRender(); break;
        case 75: _t->action3DPrint(); break;
        case 76: _t->sendToOctoPrint(); break;
        case 77: _t->sendToPrintService(); break;
        case 78: _t->actionRender(); break;
        case 79: _t->actionRenderDone((*reinterpret_cast< shared_ptr<const Geometry>(*)>(_a[1]))); break;
        case 80: _t->cgalRender(); break;
        case 81: _t->actionCheckValidity(); break;
        case 82: _t->actionDisplayAST(); break;
        case 83: _t->actionDisplayCSGTree(); break;
        case 84: _t->actionDisplayCSGProducts(); break;
        case 85: { bool _r = _t->canExport((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 86: _t->actionExport((*reinterpret_cast< FileFormat(*)>(_a[1])),(*reinterpret_cast< const char*(*)>(_a[2])),(*reinterpret_cast< const char*(*)>(_a[3])),(*reinterpret_cast< uint(*)>(_a[4]))); break;
        case 87: _t->actionExportSTL(); break;
        case 88: _t->actionExport3MF(); break;
        case 89: _t->actionExportOFF(); break;
        case 90: _t->actionExportAMF(); break;
        case 91: _t->actionExportDXF(); break;
        case 92: _t->actionExportSVG(); break;
        case 93: _t->actionExportPDF(); break;
        case 94: _t->actionExportCSG(); break;
        case 95: _t->actionExportImage(); break;
        case 96: _t->actionCopyViewport(); break;
        case 97: _t->actionFlushCaches(); break;
        case 98: _t->actionReloadRenderPreview(); break;
        case 99: _t->on_editorDock_visibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 100: _t->on_consoleDock_visibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 101: _t->on_parameterDock_visibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 102: _t->on_errorLogDock_visibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 103: _t->on_toolButtonCompileResultClose_clicked(); break;
        case 104: _t->editorTopLevelChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 105: _t->consoleTopLevelChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 106: _t->parameterTopLevelChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 107: _t->errorLogTopLevelChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 108: _t->processEvents(); break;
        case 109: _t->jumpToLine((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 110: _t->openFileFromPath((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 111: _t->viewModePreview(); break;
        case 112: _t->viewModeSurface(); break;
        case 113: _t->viewModeWireframe(); break;
        case 114: _t->viewModeThrownTogether(); break;
        case 115: _t->viewModeShowEdges(); break;
        case 116: _t->viewModeShowAxes(); break;
        case 117: _t->viewModeShowCrosshairs(); break;
        case 118: _t->viewModeShowScaleProportional(); break;
        case 119: _t->viewModeAnimate(); break;
        case 120: _t->viewAngleTop(); break;
        case 121: _t->viewAngleBottom(); break;
        case 122: _t->viewAngleLeft(); break;
        case 123: _t->viewAngleRight(); break;
        case 124: _t->viewAngleFront(); break;
        case 125: _t->viewAngleBack(); break;
        case 126: _t->viewAngleDiagonal(); break;
        case 127: _t->viewCenter(); break;
        case 128: _t->viewPerspective(); break;
        case 129: _t->viewOrthogonal(); break;
        case 130: _t->viewTogglePerspective(); break;
        case 131: _t->viewResetView(); break;
        case 132: _t->viewAll(); break;
        case 133: _t->animateUpdateDocChanged(); break;
        case 134: _t->animateUpdate(); break;
        case 135: _t->selectObject((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 136: _t->dragEnterEvent((*reinterpret_cast< QDragEnterEvent*(*)>(_a[1]))); break;
        case 137: _t->dropEvent((*reinterpret_cast< QDropEvent*(*)>(_a[1]))); break;
        case 138: _t->helpAbout(); break;
        case 139: _t->helpHomepage(); break;
        case 140: _t->helpManual(); break;
        case 141: _t->helpCheatSheet(); break;
        case 142: _t->helpLibrary(); break;
        case 143: _t->helpFontInfo(); break;
        case 144: _t->quit(); break;
        case 145: _t->checkAutoReload(); break;
        case 146: _t->waitAfterReload(); break;
        case 147: _t->autoReloadSet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< EditorInterface* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::highlightError)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::unhighlightLastError)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::MainWindow"))
        return static_cast< Ui::MainWindow*>(this);
    if (!strcmp(_clname, "InputEventHandler"))
        return static_cast< InputEventHandler*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 148)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 148;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 148)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 148;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::highlightError(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void MainWindow::unhighlightLastError()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
