/********************************************************************************
** Form generated from reading UI file 'ParameterWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PARAMETERWIDGET_H
#define UI_PARAMETERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ParameterWidget
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout_2;
    QPushButton *presetSaveButton;
    QPushButton *deleteButton;
    QComboBox *comboBoxPreset;
    QPushButton *addButton;
    QLabel *labelChangeIndicator;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *checkBoxAutoPreview;
    QComboBox *comboBoxDetails;
    QPushButton *reset;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;

    void setupUi(QWidget *ParameterWidget)
    {
        if (ParameterWidget->objectName().isEmpty())
            ParameterWidget->setObjectName(QString::fromUtf8("ParameterWidget"));
        ParameterWidget->resize(419, 590);
        gridLayout = new QGridLayout(ParameterWidget);
        gridLayout->setSpacing(4);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 4, 4, 0);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        presetSaveButton = new QPushButton(ParameterWidget);
        presetSaveButton->setObjectName(QString::fromUtf8("presetSaveButton"));
        presetSaveButton->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(presetSaveButton->sizePolicy().hasHeightForWidth());
        presetSaveButton->setSizePolicy(sizePolicy);
        QFont font;
        font.setPointSize(11);
        presetSaveButton->setFont(font);

        gridLayout_2->addWidget(presetSaveButton, 0, 5, 1, 1);

        deleteButton = new QPushButton(ParameterWidget);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));
        deleteButton->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(deleteButton->sizePolicy().hasHeightForWidth());
        deleteButton->setSizePolicy(sizePolicy1);
        deleteButton->setMinimumSize(QSize(40, 0));
        deleteButton->setMaximumSize(QSize(40, 16777215));

        gridLayout_2->addWidget(deleteButton, 0, 4, 1, 1);

        comboBoxPreset = new QComboBox(ParameterWidget);
        comboBoxPreset->setObjectName(QString::fromUtf8("comboBoxPreset"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(comboBoxPreset->sizePolicy().hasHeightForWidth());
        comboBoxPreset->setSizePolicy(sizePolicy2);
        comboBoxPreset->setFocusPolicy(Qt::StrongFocus);
        comboBoxPreset->setEditable(true);

        gridLayout_2->addWidget(comboBoxPreset, 0, 0, 1, 2);

        addButton = new QPushButton(ParameterWidget);
        addButton->setObjectName(QString::fromUtf8("addButton"));
        addButton->setEnabled(false);
        sizePolicy1.setHeightForWidth(addButton->sizePolicy().hasHeightForWidth());
        addButton->setSizePolicy(sizePolicy1);
        addButton->setMinimumSize(QSize(40, 0));
        addButton->setMaximumSize(QSize(40, 16777215));

        gridLayout_2->addWidget(addButton, 0, 3, 1, 1);

        labelChangeIndicator = new QLabel(ParameterWidget);
        labelChangeIndicator->setObjectName(QString::fromUtf8("labelChangeIndicator"));

        gridLayout_2->addWidget(labelChangeIndicator, 0, 2, 1, 1);


        gridLayout->addLayout(gridLayout_2, 8, 0, 1, 3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        checkBoxAutoPreview = new QCheckBox(ParameterWidget);
        checkBoxAutoPreview->setObjectName(QString::fromUtf8("checkBoxAutoPreview"));
        checkBoxAutoPreview->setChecked(true);

        horizontalLayout_2->addWidget(checkBoxAutoPreview);

        comboBoxDetails = new QComboBox(ParameterWidget);
        comboBoxDetails->addItem(QString());
        comboBoxDetails->addItem(QString());
        comboBoxDetails->addItem(QString());
        comboBoxDetails->addItem(QString());
        comboBoxDetails->setObjectName(QString::fromUtf8("comboBoxDetails"));
        comboBoxDetails->setFocusPolicy(Qt::StrongFocus);

        horizontalLayout_2->addWidget(comboBoxDetails);

        reset = new QPushButton(ParameterWidget);
        reset->setObjectName(QString::fromUtf8("reset"));

        horizontalLayout_2->addWidget(reset);


        gridLayout->addLayout(horizontalLayout_2, 3, 0, 2, 3);

        scrollArea = new QScrollArea(ParameterWidget);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFocusPolicy(Qt::StrongFocus);
        scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 413, 518));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayout->addWidget(scrollArea, 12, 0, 1, 3);

        QWidget::setTabOrder(checkBoxAutoPreview, comboBoxDetails);
        QWidget::setTabOrder(comboBoxDetails, reset);
        QWidget::setTabOrder(reset, comboBoxPreset);
        QWidget::setTabOrder(comboBoxPreset, addButton);
        QWidget::setTabOrder(addButton, deleteButton);
        QWidget::setTabOrder(deleteButton, presetSaveButton);
        QWidget::setTabOrder(presetSaveButton, scrollArea);

        retranslateUi(ParameterWidget);

        QMetaObject::connectSlotsByName(ParameterWidget);
    } // setupUi

    void retranslateUi(QWidget *ParameterWidget)
    {
        ParameterWidget->setWindowTitle(q_("Form", nullptr));
        presetSaveButton->setText(q_("save preset", nullptr));
#if QT_CONFIG(tooltip)
        deleteButton->setToolTip(q_("delete current preset", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteButton->setText(q_("-", nullptr));
#if QT_CONFIG(tooltip)
        comboBoxPreset->setToolTip(q_("preset selection", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        addButton->setToolTip(q_("add new preset", nullptr));
#endif // QT_CONFIG(tooltip)
        addButton->setText(q_("+", nullptr));
        labelChangeIndicator->setText(QString());
        checkBoxAutoPreview->setText(q_("Automatic Preview", nullptr));
        comboBoxDetails->setItemText(0, q_("Show Details", nullptr));
        comboBoxDetails->setItemText(1, q_("Inline Details", nullptr));
        comboBoxDetails->setItemText(2, q_("Hide Details", nullptr));
        comboBoxDetails->setItemText(3, q_("Description Only", nullptr));

#if QT_CONFIG(tooltip)
        reset->setToolTip(q_("reset all parameters to the currently selected preset", nullptr));
#endif // QT_CONFIG(tooltip)
        reset->setText(q_("Reset", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ParameterWidget: public Ui_ParameterWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PARAMETERWIDGET_H
